@@ -450,6 +450,9 @@ typedef intptr_t (*Scheme_Secondary_Hash_Proc)(Scheme_Object *obj, void *cycle_d
 #define SCHEME_MUTABLE_BOXP(obj)  (SCHEME_BOXP(obj) && SCHEME_MUTABLEP(obj))
 #define SCHEME_IMMUTABLE_BOXP(obj)  (SCHEME_BOXP(obj) && SCHEME_IMMUTABLEP(obj))
 
+#define SCHEME_PROMPT_TAGP(obj) SAME_TYPE(SCHEME_TYPE(obj), scheme_prompt_tag_type)
+#define SCHEME_CONTINUATION_MARK_KEYP(obj) SAME_TYPE(SCHEME_TYPE(obj), scheme_continuation_mark_key_type)
+
 #define SCHEME_BUCKTP(obj) SAME_TYPE(SCHEME_TYPE(obj),scheme_bucket_table_type)
 #define SCHEME_HASHTP(obj) SAME_TYPE(SCHEME_TYPE(obj),scheme_hash_table_type)
 #define SCHEME_HASHTRP(obj) SAME_TYPE(SCHEME_TYPE(obj),scheme_hash_tree_type)
@@ -658,7 +661,7 @@ typedef struct Scheme_Offset_Cptr
 #define SCHEME_PRIM_OPT_MASK (1 | 2)
 #define SCHEME_PRIM_IS_PRIMITIVE 4
 #define SCHEME_PRIM_IS_UNSAFE_OMITABLE 8
-#define SCHEME_PRIM_IS_STRUCT_OTHER 16
+#define SCHEME_PRIM_IS_OMITABLE 16
 #define SCHEME_PRIM_OTHER_TYPE_MASK (32 | 64 | 128 | 256)
 #define SCHEME_PRIM_IS_MULTI_RESULT 512
 #define SCHEME_PRIM_IS_BINARY_INLINED 1024
@@ -674,10 +677,11 @@ typedef struct Scheme_Offset_Cptr
 #define SCHEME_PRIM_OPT_NONCM      1
 
 /* Values with SCHEME_PRIM_OTHER_TYPE_MASK */
-#define SCHEME_PRIM_STRUCT_TYPE_INDEXLESS_GETTER 0
+#define SCHEME_PRIM_STRUCT_TYPE_INDEXLESS_GETTER (32 | 256)
 #define SCHEME_PRIM_STRUCT_TYPE_CONSTR           128
 #define SCHEME_PRIM_STRUCT_TYPE_INDEXLESS_SETTER 256
 #define SCHEME_PRIM_STRUCT_TYPE_INDEXED_SETTER   (128 | 256)
+#define SCHEME_PRIM_STRUCT_TYPE_BROKEN_INDEXED_SETTER   (32 | 128)
 #define SCHEME_PRIM_TYPE_PARAMETER               64
 #define SCHEME_PRIM_TYPE_STRUCT_PROP_GETTER      (64 | 128)
 #define SCHEME_PRIM_SOMETIMES_INLINED            (64 | 256)
@@ -901,7 +905,7 @@ typedef struct {
 #endif
 
 /* Like setjmp & longjmp, but you can jmp to a deeper stack position */
-/* Intialize a Scheme_Jumpup_Buf record before using it */
+/* Initialize a Scheme_Jumpup_Buf record before using it */
 typedef struct Scheme_Jumpup_Buf {
   void *stack_from, *stack_copy;
   intptr_t stack_size, stack_max_size;
@@ -1043,6 +1047,8 @@ typedef struct Scheme_Thread {
   Scheme_Object *dead_box;      /* contains non-zero when the thread is dead */
   Scheme_Object *running_box;   /* contains pointer to thread when it's running */
 
+  struct Scheme_Thread *gc_prep_chain;
+
   struct Scheme_Thread *nester, *nestee;
 
   struct future_t *current_ft;
@@ -1070,7 +1076,7 @@ typedef struct Scheme_Thread {
 
   struct Scheme_Marshal_Tables *current_mt;
 
-  Scheme_Object *constant_folding; /* compiler hack */
+  struct Optimize_Info *constant_folding; /* compiler hack */
   Scheme_Object *reading_delayed; /* reader hack */
 
   Scheme_Object *(*overflow_k)(void);
@@ -1144,9 +1150,13 @@ typedef struct Scheme_Thread {
 
   long saved_errno;
 
+  int futures_slow_path_tracing;
+
 #ifdef MZ_PRECISE_GC
   struct GC_Thread_Info *gc_info; /* managed by the GC */
+  void *place_channel_msg_in_flight;
 #endif
+
 } Scheme_Thread;
 
 #include "schthread.h"
@@ -1271,6 +1281,7 @@ enum {
   MZCONFIG_CODE_INSPECTOR,
 
   MZCONFIG_USE_COMPILED_KIND,
+  MZCONFIG_USE_COMPILED_ROOTS,
   MZCONFIG_USE_USER_PATHS,
   MZCONFIG_USE_LINK_PATHS,
 
@@ -1369,6 +1380,7 @@ struct Scheme_Port
   Scheme_Location_Fun location_fun;
   Scheme_Count_Lines_Fun count_lines_fun;
   Scheme_Buffer_Mode_Fun buffer_mode_fun;
+  Scheme_Object *position_redirect; /* for `file-position' */
 };
 
 struct Scheme_Input_Port
@@ -1758,6 +1770,7 @@ MZ_EXTERN void scheme_set_startup_load_on_demand(int);
 MZ_EXTERN void scheme_set_ignore_user_paths(int);
 MZ_EXTERN void scheme_set_ignore_link_paths(int);
 MZ_EXTERN void scheme_set_logging(int syslog_level, int stderr_level);
+MZ_EXTERN void scheme_set_logging_spec(Scheme_Object *syslog_level, Scheme_Object *stderr_level);
 
 MZ_EXTERN int scheme_get_allow_set_undefined();
 
@@ -1824,9 +1837,11 @@ MZ_EXTERN void scheme_set_addon_dir(Scheme_Object *p);
 MZ_EXTERN void scheme_set_links_file(Scheme_Object *p);
 MZ_EXTERN void scheme_set_command_line_arguments(Scheme_Object *vec);
 MZ_EXTERN void scheme_set_compiled_file_paths(Scheme_Object *list);
+MZ_EXTERN void scheme_set_compiled_file_roots(Scheme_Object *list);
 
 MZ_EXTERN void scheme_init_collection_paths(Scheme_Env *global_env, Scheme_Object *extra_dirs);
 MZ_EXTERN void scheme_init_collection_paths_post(Scheme_Env *global_env, Scheme_Object *extra_dirs, Scheme_Object *extra_post_dirs);
+MZ_EXTERN void scheme_init_compiled_roots(Scheme_Env *global_env, const char *paths);
 
 MZ_EXTERN void scheme_seal_parameters();
 
@@ -1998,10 +2013,10 @@ extern Scheme_Extension_Table *scheme_extension_table;
 # define MZ_FD_ISSET(n, p) scheme_fdisset(p, n)
 #else
 # define MZ_GET_FDSET(p, n) ((void *)(((fd_set *)p) + n))
-# define MZ_FD_ZERO(p) FD_ZERO(p)
-# define MZ_FD_SET(n, p) FD_SET(n, p)
-# define MZ_FD_CLR(n, p) FD_CLR(n, p)
-# define MZ_FD_ISSET(n, p) FD_ISSET(n, p)
+# define MZ_FD_ZERO(p) FD_ZERO((fd_set *)(p))
+# define MZ_FD_SET(n, p) FD_SET(n, (fd_set *)(p))
+# define MZ_FD_CLR(n, p) FD_CLR(n, (fd_set *)(p))
+# define MZ_FD_ISSET(n, p) FD_ISSET(n, (fd_set *)(p))
 #endif
 
 /* For scheme_fd_to_semaphore(): */
