@@ -71,7 +71,7 @@ extern "C" {
 # define MZ_THREAD_EXTERN MZ_EXTERN
 #endif
 
-MZ_EXTERN void scheme_init_os_thread();
+MZ_EXTERN void scheme_init_os_thread(void);
 
 /* **************************************************************** */
 /* Declarations that we wish were elsewhere, but are needed here to */
@@ -105,6 +105,8 @@ typedef intptr_t objhead;
 
 typedef void (*Scheme_Sleep_Proc)(float seconds, void *fds);
 
+typedef void (*Scheme_On_Atomic_Timeout_Proc)(int must_give_up);
+
 /* **************************************** */
 
 #ifndef USE_THREAD_LOCAL
@@ -122,6 +124,8 @@ typedef struct Thread_Local_Variables {
   uintptr_t GC_gen0_alloc_page_ptr_;
   uintptr_t GC_gen0_alloc_page_end_;
   int GC_gen0_alloc_only_;
+  uintptr_t force_gc_for_place_accounting_;
+  int scheme_starting_up_;
   void *bignum_cache_[BIGNUM_CACHE_SIZE];
   int cache_count_;
   struct Scheme_Hash_Table *toplevels_ht_;
@@ -144,13 +148,6 @@ typedef struct Thread_Local_Variables {
   struct Scheme_Prompt *available_regular_prompt_;
   struct Scheme_Dynamic_Wind *available_prompt_dw_;
   struct Scheme_Meta_Continuation *available_prompt_mc_;
-  struct Scheme_Object *cached_beg_stx_;
-  struct Scheme_Object *cached_mod_stx_;
-  struct Scheme_Object *cached_mod_beg_stx_;
-  struct Scheme_Object *cached_dv_stx_;
-  struct Scheme_Object *cached_ds_stx_;
-  struct Scheme_Object *cached_dvs_stx_;
-  int cached_stx_phase_;
   struct Scheme_Object *cwv_stx_;
   int cwv_stx_phase_;
   struct Scheme_Cont *offstack_cont_;
@@ -165,7 +162,9 @@ typedef struct Thread_Local_Variables {
   int fixup_already_in_place_;
   void *retry_alloc_r1_;
   double scheme_jit_save_fp_;
+  double scheme_jit_save_fp2_;
   struct Scheme_Bucket_Table *starts_table_;
+  struct Scheme_Bucket_Table *submodule_empty_modidx_table_;
   struct Scheme_Modidx *modidx_caching_chain_;
   struct Scheme_Object *global_shift_cache_;
   struct mz_proc_thread *proc_thread_self_;
@@ -173,6 +172,12 @@ typedef struct Thread_Local_Variables {
   struct Scheme_Object *scheme_orig_stderr_port_;
   struct Scheme_Object *scheme_orig_stdin_port_;
   struct mz_fd_set *scheme_fd_set_;
+  struct mz_fd_set *scheme_semaphore_fd_set_;
+  struct Scheme_Hash_Table *scheme_semaphore_fd_mapping_;
+  int scheme_semaphore_fd_kqueue_;
+#ifdef USE_FCNTL_AND_FORK_FOR_FILE_LOCKS
+  struct Scheme_Hash_Table *locked_fd_process_map_;
+#endif
   struct Scheme_Custodian *new_port_cust_;
 #if (defined(__WIN32__) || defined(WIN32) || defined(_WIN32))
   void *scheme_break_semaphore_;
@@ -184,6 +189,7 @@ typedef struct Thread_Local_Variables {
   struct ITimer_Data *itimerdata_;
   char *quick_buffer_;
   char *quick_encode_buffer_;
+  char *quick_print_buffer_;
   struct Scheme_Hash_Table *cache_ht_;
   char *regstr_;
   char *regparsestr_;
@@ -207,8 +213,6 @@ typedef struct Thread_Local_Variables {
   rxpos *maybep_buffer_cache_;
   rxpos *match_stack_buffer_cache_;
   uintptr_t scheme_os_thread_stack_base_;
-  int traversers_registered_;
-  struct Finalizations **save_fns_ptr_;
   struct Scheme_Object *scheme_system_idle_channel_;
   struct Scheme_Object *system_idle_put_evt_;
   void *stack_copy_cache_[STACK_COPY_CACHE_SIZE];
@@ -226,6 +230,7 @@ typedef struct Thread_Local_Variables {
   struct Scheme_Thread *scheme_current_thread_;
   struct Scheme_Thread *scheme_main_thread_;
   struct Scheme_Thread *scheme_first_thread_;
+  struct Scheme_Thread *gc_prep_thread_chain_;
   struct Scheme_Thread_Set *scheme_thread_set_top_;
   struct Scheme_Current_LWC *scheme_current_lwc_;
   int num_running_threads_;
@@ -234,7 +239,7 @@ typedef struct Thread_Local_Variables {
   int scheme_did_gc_count_;
   struct Scheme_Future_State *scheme_future_state_;
   struct Scheme_Future_Thread_State *scheme_future_thread_state_;
-  void *jit_future_storage_[2];
+  void *jit_future_storage_[4];
   struct Scheme_Object **scheme_current_runstack_start_;
   struct Scheme_Object **scheme_current_runstack_;
   intptr_t scheme_current_cont_mark_stack_;
@@ -242,6 +247,7 @@ typedef struct Thread_Local_Variables {
   struct Scheme_Custodian *main_custodian_;
   struct Scheme_Custodian *last_custodian_;
   struct Scheme_Hash_Table *limited_custodians_;
+  struct Scheme_Config *initial_config_;
   struct Scheme_Thread *swap_target_;
   struct Scheme_Object *scheduled_kills_;
   int do_atomic_;
@@ -267,11 +273,14 @@ typedef struct Thread_Local_Variables {
   struct gmp_tmp_stack gmp_tmp_xxx_;
   struct gmp_tmp_stack *gmp_tmp_current_;
   struct Scheme_Logger *scheme_main_logger_;
+  struct Scheme_Logger *scheme_gc_logger_;
+  struct Scheme_Logger *scheme_future_logger_;
   int intdef_counter_;
   int builtin_ref_counter_;
   int env_uid_counter_;
   int scheme_overflow_count_;
   struct Scheme_Object *original_pwd_;
+  struct Scheme_Object *inst_links_path_;
   void *file_path_wc_buffer_;
   intptr_t scheme_hash_request_count_;
   intptr_t scheme_hash_iteration_count_;
@@ -298,6 +307,9 @@ typedef struct Thread_Local_Variables {
   intptr_t scheme_total_gc_time_;
   intptr_t start_this_gc_time_;
   intptr_t end_this_gc_time_;
+  double start_this_gc_real_time_;
+  double end_this_gc_real_time_;
+  struct Scheme_Struct_Type *gc_info_prefab_;
   volatile short delayed_break_ready_;
   struct Scheme_Thread *main_break_target_thread_;
   intptr_t scheme_code_page_total_;
@@ -316,12 +328,16 @@ typedef struct Thread_Local_Variables {
   struct Scheme_Hash_Table *place_local_symbol_table_;
   struct Scheme_Hash_Table *place_local_keyword_table_;
   struct Scheme_Hash_Table *place_local_parallel_symbol_table_;
+  struct Scheme_Bucket_Table *literal_string_table_;
+  struct Scheme_Bucket_Table *literal_number_table_;
   struct FFI_Sync_Queue *ffi_sync_queue_;
   struct Scheme_GC_Pre_Post_Callback_Desc *gc_prepost_callback_descs_;
   struct Scheme_Hash_Table *place_local_misc_table_;
   int place_evts_array_size_;
   struct Evt **place_evts_;
   struct Scheme_Place_Object *place_object_;
+  struct Scheme_Place *all_child_places_;
+  struct Scheme_Object **reusable_ifs_stack_;
   struct Scheme_Object *empty_self_shift_cache_;
   struct Scheme_Bucket_Table *scheme_module_code_cache_;
   struct Scheme_Object *group_member_cache_;
@@ -330,6 +346,10 @@ typedef struct Thread_Local_Variables {
   struct Scheme_Hash_Table *fullpath_loaded_extensions_;
   Scheme_Sleep_Proc scheme_place_sleep_;
   struct Scheme_Bucket_Table *taint_intern_table_;
+  struct GHBN_Thread_Data *ghbn_thread_data_;
+  Scheme_On_Atomic_Timeout_Proc on_atomic_timeout_;
+  int atomic_timeout_auto_suspend_;
+  int atomic_timeout_atomic_level_;
 } Thread_Local_Variables;
 
 #if defined(IMPLEMENT_THREAD_LOCAL_VIA_PTHREADS)
@@ -348,8 +368,8 @@ XFORM_GC_VARIABLE_STACK_THROUGH_GETSPECIFIC;
 #  ifdef MZ_XFORM
 START_XFORM_SKIP;
 #  endif
-static inline Thread_Local_Variables *scheme_get_thread_local_variables() __attribute__((used));
-static inline Thread_Local_Variables *scheme_get_thread_local_variables() {
+static inline Thread_Local_Variables *scheme_get_thread_local_variables(void) __attribute__((used));
+static inline Thread_Local_Variables *scheme_get_thread_local_variables(void) {
   Thread_Local_Variables *x = NULL;
 #  if defined(__APPLE__) && defined(__MACH__)
 #   if defined(__x86_64__)
@@ -382,7 +402,7 @@ XFORM_GC_VARIABLE_STACK_THROUGH_FUNCTION;
 # endif
 #elif defined(IMPLEMENT_THREAD_LOCAL_VIA_PROCEDURE)
 /* Using external scheme_get_thread_local_variables() procedure */
-MZ_EXTERN Thread_Local_Variables *scheme_get_thread_local_variables();
+MZ_EXTERN Thread_Local_Variables *scheme_get_thread_local_variables(void);
 # ifdef MZ_XFORM
 XFORM_GC_VARIABLE_STACK_THROUGH_FUNCTION;
 # endif
@@ -390,14 +410,14 @@ XFORM_GC_VARIABLE_STACK_THROUGH_FUNCTION;
 # ifdef MZ_XFORM
 START_XFORM_SKIP;
 # endif
-MZ_EXTERN Thread_Local_Variables *scheme_external_get_thread_local_variables();
+MZ_EXTERN Thread_Local_Variables *scheme_external_get_thread_local_variables(void);
 # ifdef __mzscheme_private__
 /* In the Racket DLL, need thread-local to be fast: */
 MZ_EXTERN uintptr_t scheme_tls_delta;
 #  ifdef MZ_USE_WIN_TLS_VIA_DLL
 MZ_EXTERN int scheme_tls_index;
 #  endif
-static __inline Thread_Local_Variables **scheme_get_thread_local_variables_ptr() {
+static __inline Thread_Local_Variables **scheme_get_thread_local_variables_ptr(void) {
   __asm { mov eax, FS:[0x2C]
 #  ifdef MZ_USE_WIN_TLS_VIA_DLL
           add eax, scheme_tls_index
@@ -406,12 +426,12 @@ static __inline Thread_Local_Variables **scheme_get_thread_local_variables_ptr()
           add eax, scheme_tls_delta }
   /* result is in eax */
 }
-static __inline Thread_Local_Variables *scheme_get_thread_local_variables() {
+static __inline Thread_Local_Variables *scheme_get_thread_local_variables(void) {
   return *scheme_get_thread_local_variables_ptr();
 }
 # else
 /* Outside the Racket DLL, slower thread-local is ok: */
-static __inline Thread_Local_Variables *scheme_get_thread_local_variables() {
+static __inline Thread_Local_Variables *scheme_get_thread_local_variables(void) {
   return scheme_external_get_thread_local_variables();
 }
 # endif
@@ -425,8 +445,8 @@ XFORM_GC_VARIABLE_STACK_THROUGH_FUNCTION;
 #  ifdef MZ_XFORM
 START_XFORM_SKIP;
 #  endif
-MZ_EXTERN Thread_Local_Variables *scheme_external_get_thread_local_variables();
-static __inline Thread_Local_Variables *scheme_get_thread_local_variables() {
+MZ_EXTERN Thread_Local_Variables *scheme_external_get_thread_local_variables(void);
+static __inline Thread_Local_Variables *scheme_get_thread_local_variables(void) {
   return scheme_external_get_thread_local_variables();
 }
 #  ifdef MZ_XFORM
@@ -457,6 +477,8 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define GC_gen0_alloc_page_end XOA (scheme_get_thread_local_variables()->GC_gen0_alloc_page_end_)
 #define GC_gen0_alloc_only XOA (scheme_get_thread_local_variables()->GC_gen0_alloc_only_)
 #define GC_variable_stack XOA (scheme_get_thread_local_variables()->GC_variable_stack_)
+#define force_gc_for_place_accounting XOA (scheme_get_thread_local_variables()->force_gc_for_place_accounting_)
+#define scheme_starting_up XOA (scheme_get_thread_local_variables()->scheme_starting_up_)
 #define bignum_cache XOA (scheme_get_thread_local_variables()->bignum_cache_)
 #define cache_count XOA (scheme_get_thread_local_variables()->cache_count_)
 #define toplevels_ht XOA (scheme_get_thread_local_variables()->toplevels_ht_)
@@ -479,13 +501,6 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define available_regular_prompt XOA (scheme_get_thread_local_variables()->available_regular_prompt_)
 #define available_prompt_dw XOA (scheme_get_thread_local_variables()->available_prompt_dw_)
 #define available_prompt_mc XOA (scheme_get_thread_local_variables()->available_prompt_mc_)
-#define cached_beg_stx XOA (scheme_get_thread_local_variables()->cached_beg_stx_)
-#define cached_mod_stx XOA (scheme_get_thread_local_variables()->cached_mod_stx_)
-#define cached_mod_beg_stx XOA (scheme_get_thread_local_variables()->cached_mod_beg_stx_)
-#define cached_dv_stx XOA (scheme_get_thread_local_variables()->cached_dv_stx_)
-#define cached_ds_stx XOA (scheme_get_thread_local_variables()->cached_ds_stx_)
-#define cached_dvs_stx XOA (scheme_get_thread_local_variables()->cached_dvs_stx_)
-#define cached_stx_phase XOA (scheme_get_thread_local_variables()->cached_stx_phase_)
 #define cwv_stx XOA (scheme_get_thread_local_variables()->cwv_stx_)
 #define cwv_stx_phase XOA (scheme_get_thread_local_variables()->cwv_stx_phase_)
 #define offstack_cont XOA (scheme_get_thread_local_variables()->offstack_cont_)
@@ -501,7 +516,9 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define fixup_already_in_place XOA (scheme_get_thread_local_variables()->fixup_already_in_place_)
 #define retry_alloc_r1 XOA (scheme_get_thread_local_variables()->retry_alloc_r1_)
 #define scheme_jit_save_fp XOA (scheme_get_thread_local_variables()->scheme_jit_save_fp_)
+#define scheme_jit_save_fp2 XOA (scheme_get_thread_local_variables()->scheme_jit_save_fp2_)
 #define starts_table XOA (scheme_get_thread_local_variables()->starts_table_)
+#define submodule_empty_modidx_table XOA (scheme_get_thread_local_variables()->submodule_empty_modidx_table_)
 #define modidx_caching_chain XOA (scheme_get_thread_local_variables()->modidx_caching_chain_)
 #define global_shift_cache XOA (scheme_get_thread_local_variables()->global_shift_cache_)
 #define proc_thread_self XOA (scheme_get_thread_local_variables()->proc_thread_self_)
@@ -509,6 +526,10 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define scheme_orig_stderr_port XOA (scheme_get_thread_local_variables()->scheme_orig_stderr_port_)
 #define scheme_orig_stdin_port XOA (scheme_get_thread_local_variables()->scheme_orig_stdin_port_)
 #define scheme_fd_set XOA (scheme_get_thread_local_variables()->scheme_fd_set_)
+#define scheme_semaphore_fd_set XOA (scheme_get_thread_local_variables()->scheme_semaphore_fd_set_)
+#define scheme_semaphore_fd_mapping XOA (scheme_get_thread_local_variables()->scheme_semaphore_fd_mapping_)
+#define scheme_semaphore_fd_kqueue XOA (scheme_get_thread_local_variables()->scheme_semaphore_fd_kqueue_)
+#define locked_fd_process_map XOA (scheme_get_thread_local_variables()->locked_fd_process_map_)
 #define new_port_cust XOA (scheme_get_thread_local_variables()->new_port_cust_)
 #define scheme_break_semaphore XOA (scheme_get_thread_local_variables()->scheme_break_semaphore_)
 #define external_event_fd XOA (scheme_get_thread_local_variables()->external_event_fd_)
@@ -517,6 +538,7 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define itimerdata XOA (scheme_get_thread_local_variables()->itimerdata_)
 #define quick_buffer XOA (scheme_get_thread_local_variables()->quick_buffer_)
 #define quick_encode_buffer XOA (scheme_get_thread_local_variables()->quick_encode_buffer_)
+#define quick_print_buffer XOA (scheme_get_thread_local_variables()->quick_print_buffer_)
 #define cache_ht XOA (scheme_get_thread_local_variables()->cache_ht_)
 #define regstr XOA (scheme_get_thread_local_variables()->regstr_)
 #define regparsestr XOA (scheme_get_thread_local_variables()->regparsestr_)
@@ -540,8 +562,6 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define maybep_buffer_cache XOA (scheme_get_thread_local_variables()->maybep_buffer_cache_)
 #define match_stack_buffer_cache XOA (scheme_get_thread_local_variables()->match_stack_buffer_cache_)
 #define scheme_os_thread_stack_base XOA (scheme_get_thread_local_variables()->scheme_os_thread_stack_base_)
-#define traversers_registered XOA (scheme_get_thread_local_variables()->traversers_registered_)
-#define save_fns_ptr XOA (scheme_get_thread_local_variables()->save_fns_ptr_)
 #define scheme_system_idle_channel XOA (scheme_get_thread_local_variables()->scheme_system_idle_channel_)
 #define system_idle_put_evt XOA (scheme_get_thread_local_variables()->system_idle_put_evt_)
 #define stack_copy_cache XOA (scheme_get_thread_local_variables()->stack_copy_cache_)
@@ -559,6 +579,7 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define scheme_current_thread XOA (scheme_get_thread_local_variables()->scheme_current_thread_)
 #define scheme_main_thread XOA (scheme_get_thread_local_variables()->scheme_main_thread_)
 #define scheme_first_thread XOA (scheme_get_thread_local_variables()->scheme_first_thread_)
+#define gc_prep_thread_chain XOA (scheme_get_thread_local_variables()->gc_prep_thread_chain_)
 #define scheme_thread_set_top XOA (scheme_get_thread_local_variables()->scheme_thread_set_top_)
 #define scheme_current_lwc XOA (scheme_get_thread_local_variables()->scheme_current_lwc_)
 #define num_running_threads XOA (scheme_get_thread_local_variables()->num_running_threads_)
@@ -575,6 +596,7 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define main_custodian XOA (scheme_get_thread_local_variables()->main_custodian_)
 #define last_custodian XOA (scheme_get_thread_local_variables()->last_custodian_)
 #define limited_custodians XOA (scheme_get_thread_local_variables()->limited_custodians_)
+#define initial_config XOA (scheme_get_thread_local_variables()->initial_config_)
 #define swap_target XOA (scheme_get_thread_local_variables()->swap_target_)
 #define scheduled_kills XOA (scheme_get_thread_local_variables()->scheduled_kills_)
 #define do_atomic XOA (scheme_get_thread_local_variables()->do_atomic_)
@@ -600,11 +622,14 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define gmp_tmp_xxx XOA (scheme_get_thread_local_variables()->gmp_tmp_xxx_)
 #define gmp_tmp_current XOA (scheme_get_thread_local_variables()->gmp_tmp_current_)
 #define scheme_main_logger XOA (scheme_get_thread_local_variables()->scheme_main_logger_)
+#define scheme_gc_logger XOA (scheme_get_thread_local_variables()->scheme_gc_logger_)
+#define scheme_future_logger XOA (scheme_get_thread_local_variables()->scheme_future_logger_)
 #define intdef_counter XOA (scheme_get_thread_local_variables()->intdef_counter_)
 #define builtin_ref_counter XOA (scheme_get_thread_local_variables()->builtin_ref_counter_)
 #define env_uid_counter XOA (scheme_get_thread_local_variables()->env_uid_counter_)
 #define scheme_overflow_count XOA (scheme_get_thread_local_variables()->scheme_overflow_count_)
 #define original_pwd XOA (scheme_get_thread_local_variables()->original_pwd_)
+#define inst_links_path XOA (scheme_get_thread_local_variables()->inst_links_path_)
 #define file_path_wc_buffer XOA (scheme_get_thread_local_variables()->file_path_wc_buffer_)
 #define scheme_hash_request_count XOA (scheme_get_thread_local_variables()->scheme_hash_request_count_)
 #define scheme_hash_iteration_count XOA (scheme_get_thread_local_variables()->scheme_hash_iteration_count_)
@@ -631,6 +656,9 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define scheme_total_gc_time XOA (scheme_get_thread_local_variables()->scheme_total_gc_time_)
 #define start_this_gc_time XOA (scheme_get_thread_local_variables()->start_this_gc_time_)
 #define end_this_gc_time XOA (scheme_get_thread_local_variables()->end_this_gc_time_)
+#define start_this_gc_real_time XOA (scheme_get_thread_local_variables()->start_this_gc_real_time_)
+#define end_this_gc_real_time XOA (scheme_get_thread_local_variables()->end_this_gc_real_time_)
+#define gc_info_prefab XOA (scheme_get_thread_local_variables()->gc_info_prefab_)
 #define delayed_break_ready XOA (scheme_get_thread_local_variables()->delayed_break_ready_)
 #define main_break_target_thread XOA (scheme_get_thread_local_variables()->main_break_target_thread_)
 #define scheme_code_page_total XOA (scheme_get_thread_local_variables()->scheme_code_page_total_)
@@ -649,12 +677,16 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define place_local_symbol_table XOA (scheme_get_thread_local_variables()->place_local_symbol_table_)
 #define place_local_keyword_table XOA (scheme_get_thread_local_variables()->place_local_keyword_table_)
 #define place_local_parallel_symbol_table XOA (scheme_get_thread_local_variables()->place_local_parallel_symbol_table_)
+#define literal_string_table XOA (scheme_get_thread_local_variables()->literal_string_table_)
+#define literal_number_table XOA (scheme_get_thread_local_variables()->literal_number_table_)
 #define ffi_sync_queue XOA (scheme_get_thread_local_variables()->ffi_sync_queue_)
 #define gc_prepost_callback_descs XOA (scheme_get_thread_local_variables()->gc_prepost_callback_descs_)
 #define place_local_misc_table XOA (scheme_get_thread_local_variables()->place_local_misc_table_)
 #define place_evts_array_size XOA (scheme_get_thread_local_variables()->place_evts_array_size_)
 #define place_evts XOA (scheme_get_thread_local_variables()->place_evts_)
 #define place_object XOA (scheme_get_thread_local_variables()->place_object_)
+#define all_child_places XOA (scheme_get_thread_local_variables()->all_child_places_)
+#define reusable_ifs_stack XOA (scheme_get_thread_local_variables()->reusable_ifs_stack_)
 #define empty_self_shift_cache XOA (scheme_get_thread_local_variables()->empty_self_shift_cache_)
 #define scheme_module_code_cache XOA (scheme_get_thread_local_variables()->scheme_module_code_cache_)
 #define group_member_cache XOA (scheme_get_thread_local_variables()->group_member_cache_)
@@ -663,6 +695,10 @@ XFORM_GC_VARIABLE_STACK_THROUGH_THREAD_LOCAL;
 #define fullpath_loaded_extensions XOA (scheme_get_thread_local_variables()->fullpath_loaded_extensions_)
 #define scheme_place_sleep XOA (scheme_get_thread_local_variables()->scheme_place_sleep_)
 #define taint_intern_table XOA (scheme_get_thread_local_variables()->taint_intern_table_)
+#define ghbn_thread_data XOA (scheme_get_thread_local_variables()->ghbn_thread_data_)
+#define on_atomic_timeout XOA (scheme_get_thread_local_variables()->on_atomic_timeout_)
+#define atomic_timeout_auto_suspend XOA (scheme_get_thread_local_variables()->atomic_timeout_auto_suspend_)
+#define atomic_timeout_atomic_level XOA (scheme_get_thread_local_variables()->atomic_timeout_atomic_level_)
 
 /* **************************************** */
 
