@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2012 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt, Scott Owens
 
     This library is free software; you can redistribute it and/or
@@ -406,7 +406,7 @@ int scheme_bignum_get_long_long_val(const Scheme_Object *o, mzlonglong *v)
     *v = 0;
     return 1;
   } else if (SCHEME_BIGDIG(o)[MAX_BN_SIZE_FOR_LL - 1] == FIRST_BIT_MASK_LL 
-# ifndef USE_LONG_LONG_FOR_BIGDIG
+# if !defined(USE_LONG_LONG_FOR_BIGDIG) && !defined(SIXTY_FOUR_BIT_INTEGERS)
 	     && !SCHEME_BIGDIG(o)[0]
 # endif
 	     && !SCHEME_BIGPOS(o)) {
@@ -646,6 +646,35 @@ static bigdig* allocate_bigdig_array(intptr_t length)
   return res;
 }
 
+Scheme_Object *scheme_bignum_copy(const Scheme_Object *n)
+{
+  Scheme_Object *o;
+  intptr_t len;
+  bigdig* digs;
+
+  len = SCHEME_BIGLEN(n);
+
+  if (SCHEME_BIGDIG(n) == ((Small_Bignum *) mzALIAS n)->v) {
+    o = (Scheme_Object *)scheme_malloc_tagged(sizeof(Small_Bignum));
+#if MZ_PRECISE_GC
+    SCHEME_SET_BIGINLINE(o);
+#endif
+    ((Small_Bignum *)o)->v[0] = SCHEME_BIGDIG(n)[0];
+    SCHEME_BIGDIG(o) = ((Small_Bignum *) mzALIAS o)->v;
+  } else {
+    o = (Scheme_Object *)MALLOC_ONE_TAGGED(Scheme_Bignum);
+    digs = allocate_bigdig_array(len);
+    memcpy(digs, SCHEME_BIGDIG(n), len * sizeof(bigdig));
+    SCHEME_BIGDIG(o) = digs;
+  }
+
+  o->type = scheme_bignum_type;
+  SCHEME_SET_BIGPOS(o, SCHEME_BIGPOS(n));
+  SCHEME_BIGLEN(o) = len;
+
+  return o;
+}
+
 /* We don't want to count leading digits of 0 in the bignum's length */
 XFORM_NONGCING static intptr_t bigdig_length(bigdig* array, intptr_t alloced)
 {
@@ -1184,7 +1213,7 @@ char *scheme_bignum_to_allocated_string(const Scheme_Object *b, int radix, int a
 {
   Scheme_Object *c;
   unsigned char* str, *str2;
-  intptr_t i, slen, start, clen;
+  intptr_t i, slen, start;
   bigdig *c_digs;
   SAFE_SPACE(csd)
 
@@ -1215,8 +1244,7 @@ char *scheme_bignum_to_allocated_string(const Scheme_Object *b, int radix, int a
   str = (unsigned char *)MALLOC_PROTECT(slen);
 
   c_digs = SCHEME_BIGDIG_SAFE(c, csd);
-  clen = SCHEME_BIGLEN(c);
-  PROTECT(c_digs, clen);
+  PROTECT(c_digs, SCHEME_BIGLEN(c));
 
   slen = mpn_get_str(str, radix, c_digs, SCHEME_BIGLEN(c) - 1);
 
@@ -1277,7 +1305,7 @@ char *scheme_bignum_to_string(const Scheme_Object *b, int radix)
 
 Scheme_Object *scheme_read_bignum(const mzchar *str, int offset, int radix)
 {
-  intptr_t len, negate, stri, alloc, i, test;
+  intptr_t len, negate, stri, alloc, i;
   Scheme_Object* o;
   bigdig* digs;
   unsigned char* istring;
@@ -1339,7 +1367,7 @@ Scheme_Object *scheme_read_bignum(const mzchar *str, int offset, int radix)
 
   SCHEME_SET_BIGPOS(o, !negate);
 
-  test = mpn_set_str(digs, istring, len, radix);
+  (void)mpn_set_str(digs, istring, len, radix);
 
   FREE_PROTECT(istring);
   FINISH_RESULT(digs, alloc);
