@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2012 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -83,20 +83,24 @@ void scheme_init_bool (Scheme_Env *env)
 
   p = scheme_make_folding_prim(not_prim, "not", 1, 1, 1);
   scheme_not_prim = p;
-  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
+  SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_UNARY_INLINED
+                                | SCHEME_PRIM_IS_OMITABLE);
   scheme_add_global_constant("not", p, env);
 
   p = scheme_make_folding_prim(boolean_p_prim, "boolean?", 1, 1, 1);
-  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
+  SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_UNARY_INLINED
+                                | SCHEME_PRIM_IS_OMITABLE);
   scheme_add_global_constant("boolean?", p, env);
 
   p = scheme_make_folding_prim(eq_prim, "eq?", 2, 2, 1);
-  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_BINARY_INLINED;
+  SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_BINARY_INLINED
+                                | SCHEME_PRIM_IS_OMITABLE);
   scheme_eq_prim = p;
   scheme_add_global_constant("eq?", p, env);
 
   p = scheme_make_folding_prim(eqv_prim, "eqv?", 2, 2, 1);
-  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_BINARY_INLINED;
+  SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_BINARY_INLINED
+                                | SCHEME_PRIM_IS_OMITABLE);
   scheme_eqv_prim = p;
   scheme_add_global_constant("eqv?", scheme_eqv_prim, env);
   
@@ -110,11 +114,13 @@ void scheme_init_bool (Scheme_Env *env)
                              env);
 
   p = scheme_make_immed_prim(chaperone_p, "chaperone?", 1, 1);
-  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
+  SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_UNARY_INLINED
+                                | SCHEME_PRIM_IS_OMITABLE);
   scheme_add_global_constant("chaperone?", p, env);
 
   p = scheme_make_immed_prim(impersonator_p, "impersonator?", 1, 1);
-  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
+  SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_UNARY_INLINED
+                                | SCHEME_PRIM_IS_OMITABLE);
   scheme_add_global_constant("impersonator?", p, env);
 
   scheme_add_global_constant("chaperone-of?",
@@ -485,6 +491,14 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
     l2 = SCHEME_CHAR_STRTAG_VAL(obj2);
     return ((l1 == l2)
 	    && !memcmp(SCHEME_CHAR_STR_VAL(obj1), SCHEME_CHAR_STR_VAL(obj2), l1 * sizeof(mzchar)));
+  } else if (t1 == scheme_regexp_type) {
+    if (scheme_regexp_is_byte(obj1) != scheme_regexp_is_byte(obj2))
+      return 0;
+    if (scheme_regexp_is_pregexp(obj1) != scheme_regexp_is_pregexp(obj2))
+      return 0;
+    obj1 = scheme_regexp_source(obj1);
+    obj2 = scheme_regexp_source(obj2);
+    goto top;
   } else if ((t1 == scheme_structure_type)
              || (t1 == scheme_proc_struct_type)) {
     Scheme_Struct_Type *st1, *st2;
@@ -618,6 +632,12 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
     obj1 = SCHEME_PTR_VAL(obj1);
     obj2 = SCHEME_PTR_VAL(obj2);
     goto top;
+  } else if (t1 == scheme_place_bi_channel_type) {
+    Scheme_Place_Bi_Channel *bc1, *bc2;
+    bc1 = (Scheme_Place_Bi_Channel *)obj1;
+    bc2 = (Scheme_Place_Bi_Channel *)obj2;
+   return (SAME_OBJ(bc1->recvch, bc2->recvch)
+           && SAME_OBJ(bc1->sendch, bc2->sendch));
   } else if (!eql->for_chaperone && ((t1 == scheme_chaperone_type)
                                      || (t1 == scheme_proc_chaperone_type))) {
     /* both chaperones */
@@ -750,18 +770,22 @@ static Scheme_Object *apply_impersonator_of(int for_chaperone, Scheme_Object *pr
   
   oprocs = scheme_struct_type_property_ref(scheme_impersonator_of_property, v);  
   if (!oprocs || !SAME_OBJ(SCHEME_CAR(oprocs), SCHEME_CAR(procs)))
-    scheme_arg_mismatch((for_chaperone ? "impersonator-of?" : "equal?"),
-                        "impersonator-of property procedure returned a value with a different prop:impersonator-of source: ",
-                        v);
+    scheme_contract_error((for_chaperone ? "impersonator-of?" : "equal?"),
+                          "impersonator-of property procedure returned a value with a different prop:impersonator-of source",
+                          "original value", 1, obj,
+                          "returned value", 1, v,
+                          NULL);
 
   procs = scheme_struct_type_property_ref(scheme_equal_property, obj);
   oprocs = scheme_struct_type_property_ref(scheme_equal_property, v);  
   if (procs || oprocs)
     if (!procs || !oprocs || !SAME_OBJ(SCHEME_VEC_ELS(oprocs)[0], 
                                        SCHEME_VEC_ELS(procs)[0]))
-      scheme_arg_mismatch((for_chaperone ? "impersonator-of?" : "equal?"),
-                          "impersonator-of property procedure returned a value with a different prop:equal+hash source: ",
-                          v);
+      scheme_contract_error((for_chaperone ? "impersonator-of?" : "equal?"),
+                            "impersonator-of property procedure returned a value with a different prop:equal+hash source",
+                            "original value", 1, obj,
+                            "returned value", 1, v,
+                            NULL);
 
   return v;
 }
