@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2012 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -43,7 +43,7 @@
 SHARED_OK int scheme_allow_set_undefined;
 void scheme_set_allow_set_undefined(int v) { scheme_allow_set_undefined =  v; }
 int scheme_get_allow_set_undefined() { return scheme_allow_set_undefined; }
-SHARED_OK int scheme_starting_up;
+THREAD_LOCAL_DECL(int scheme_starting_up);
 
 /* globals READ-ONLY SHARED */
 Scheme_Object *scheme_varref_const_p_proc;
@@ -56,6 +56,9 @@ READ_ONLY static Scheme_Env    *futures_env;
 THREAD_LOCAL_DECL(static int builtin_ref_counter);
 THREAD_LOCAL_DECL(static int intdef_counter);
 
+THREAD_LOCAL_DECL(static Scheme_Bucket_Table *literal_string_table);
+THREAD_LOCAL_DECL(static Scheme_Bucket_Table *literal_number_table);
+
 /* local functions */
 static void make_kernel_env(void);
 
@@ -72,6 +75,7 @@ static Scheme_Object *namespace_undefine_variable(int, Scheme_Object *[]);
 static Scheme_Object *namespace_mapped_symbols(int, Scheme_Object *[]);
 static Scheme_Object *namespace_module_registry(int, Scheme_Object *[]);
 static Scheme_Object *variable_p(int, Scheme_Object *[]);
+static Scheme_Object *variable_modidx(int, Scheme_Object *[]);
 static Scheme_Object *variable_module_path(int, Scheme_Object *[]);
 static Scheme_Object *variable_module_source(int, Scheme_Object *[]);
 static Scheme_Object *variable_namespace(int, Scheme_Object *[]);
@@ -96,6 +100,7 @@ static Scheme_Object *local_module_introduce(int argc, Scheme_Object *argv[]);
 static Scheme_Object *local_get_shadower(int argc, Scheme_Object *argv[]);
 static Scheme_Object *local_module_exports(int argc, Scheme_Object *argv[]);
 static Scheme_Object *local_module_definitions(int argc, Scheme_Object *argv[]);
+static Scheme_Object *local_submodules(int argc, Scheme_Object *argv[]);
 static Scheme_Object *local_module_imports(int argc, Scheme_Object *argv[]);
 static Scheme_Object *local_module_expanding_provides(int argc, Scheme_Object *argv[]);
 static Scheme_Object *local_lift_expr(int argc, Scheme_Object *argv[]);
@@ -200,9 +205,7 @@ Scheme_Env *scheme_restart_instance() {
   scheme_init_port_fun_config();
   scheme_init_error_config();
   scheme_init_logger_config();
-#ifndef NO_SCHEME_EXNS
   scheme_init_exn_config();
-#endif
 
   boot_module_resolver();
 
@@ -277,6 +280,8 @@ Scheme_Env *scheme_engine_instance_init()
 #endif
   make_kernel_env();
 
+  scheme_init_logging_once();
+
 #if defined(MZ_PLACES_WAITPID)
   scheme_places_start_child_signal_handler();
 #endif
@@ -453,6 +458,7 @@ static Scheme_Env *place_instance_init(void *stack_base, int initial_main_os_thr
   scheme_init_stx_places(initial_main_os_thread);
   scheme_init_sema_places();
   scheme_init_gmp_places();
+  scheme_init_kqueue();
   scheme_alloc_global_fdset();
   scheme_init_file_places();
 #ifndef DONT_USE_FOREIGN
@@ -488,13 +494,19 @@ static Scheme_Env *place_instance_init(void *stack_base, int initial_main_os_thr
 
   scheme_init_foreign(env);
 
+  REGISTER_SO(literal_string_table);
+  REGISTER_SO(literal_number_table);
+  literal_string_table = scheme_make_weak_equal_table();
+  literal_number_table = scheme_make_weak_eqv_table();
+
+  scheme_starting_up = 1; /* in case it's not set already */
+
   scheme_add_embedded_builtins(env);
 
   boot_module_resolver();
 
   scheme_save_initial_module_set(env);
 
-
   scheme_starting_up = 0;
 
   --scheme_current_thread->suspend_break; /* created with breaks suspended */
@@ -558,6 +570,7 @@ void scheme_place_instance_destroy(int force)
 #endif
   scheme_free_all_code();
   scheme_free_ghbn_data();
+  scheme_release_kqueue();
 }
 
 static void make_kernel_env(void)
@@ -641,6 +654,7 @@ static void make_kernel_env(void)
   GLOBAL_PRIM_W_ARITY("namespace-module-registry", namespace_module_registry, 1, 1, env);
 
   GLOBAL_PRIM_W_ARITY("variable-reference?", variable_p, 1, 1, env);
+  GLOBAL_PRIM_W_ARITY("variable-reference->module-path-index", variable_modidx, 1, 1, env);
   GLOBAL_PRIM_W_ARITY("variable-reference->resolved-module-path", variable_module_path, 1, 1, env);
   GLOBAL_PRIM_W_ARITY("variable-reference->module-source", variable_module_source, 1, 1, env);
   GLOBAL_PRIM_W_ARITY("variable-reference->empty-namespace", variable_namespace, 1, 1, env);
@@ -673,6 +687,7 @@ static void make_kernel_env(void)
 
   GLOBAL_PRIM_W_ARITY("syntax-local-module-exports", local_module_exports, 1, 1, env);
   GLOBAL_PRIM_W_ARITY("syntax-local-module-defined-identifiers", local_module_definitions, 0, 0, env);
+  GLOBAL_PRIM_W_ARITY("syntax-local-submodules", local_submodules, 0, 0, env);
   GLOBAL_PRIM_W_ARITY("syntax-local-module-required-identifiers", local_module_imports, 2, 2, env);
   GLOBAL_PRIM_W_ARITY("syntax-local-transforming-module-provides?", local_module_expanding_provides, 0, 0, env);
 
@@ -751,7 +766,7 @@ void scheme_prepare_env_renames(Scheme_Env *env, int kind)
   if (!env->rename_set) {
     Scheme_Object *rns, *insp;
 
-    insp = env->insp;
+    insp = env->access_insp;
     if (!insp)
       insp = scheme_get_param(scheme_current_config(), MZCONFIG_CODE_INSPECTOR);
 
@@ -824,10 +839,12 @@ static Scheme_Env *make_env(Scheme_Env *base, int toplevel_size)
   if (base) {
     env->modchain = base->modchain;
     env->module_registry = base->module_registry;
+    env->module_pre_registry = base->module_pre_registry;
     env->label_env = base->label_env;
   } else {
     env->modchain = NULL;
     env->module_registry = NULL;
+    env->module_pre_registry = NULL;
     env->label_env = NULL;
   }
 
@@ -838,9 +855,17 @@ Scheme_Env *
 scheme_new_module_env(Scheme_Env *env, Scheme_Module *m, int new_exp_module_tree)
 {
   Scheme_Env *menv;
+  Scheme_Module_Registry *reg;
 
   menv = make_env(env, 7);
 
+  if (new_exp_module_tree && !menv->module_pre_registry) {
+    /* pre_registry is for declarations to be used by submodules */
+    reg = MALLOC_ONE_TAGGED(Scheme_Module_Registry);
+    reg->so.type = scheme_module_registry_type;
+    menv->module_pre_registry = reg;
+  }
+
   menv->module = m;
 
   scheme_prepare_label_env(env);
@@ -880,7 +905,9 @@ void scheme_prepare_exp_env(Scheme_Env *env)
 
     eenv->module = env->module;
     eenv->module_registry = env->module_registry;
-    eenv->insp = env->insp;
+    eenv->module_pre_registry = env->module_pre_registry;
+    eenv->access_insp = env->access_insp;
+    eenv->guard_insp = env->guard_insp;
 
     modchain = SCHEME_VEC_ELS(env->modchain)[1];
     if (SCHEME_FALSEP(modchain)) {
@@ -921,7 +948,9 @@ void scheme_prepare_template_env(Scheme_Env *env)
 
     eenv->module = env->module;
     eenv->module_registry = env->module_registry;
-    eenv->insp = env->insp;
+    eenv->module_pre_registry = env->module_pre_registry;
+    eenv->guard_insp = env->guard_insp;
+    eenv->access_insp = env->access_insp;
 
     modchain = SCHEME_VEC_ELS(env->modchain)[2];
     if (SCHEME_FALSEP(modchain)) {
@@ -961,7 +990,9 @@ void scheme_prepare_label_env(Scheme_Env *env)
 
     lenv->module = env->module;
     lenv->module_registry = env->module_registry;
-    lenv->insp = env->insp;
+    lenv->module_pre_registry = env->module_pre_registry;
+    lenv->guard_insp = env->guard_insp;
+    lenv->access_insp = env->access_insp;
 
     modchain = scheme_make_vector(5, scheme_false);    
     prev_modules = scheme_make_hash_table(SCHEME_hash_ptr);
@@ -993,7 +1024,9 @@ Scheme_Env *scheme_copy_module_env(Scheme_Env *menv, Scheme_Env *ns, Scheme_Obje
 
   menv2->module = menv->module;
   menv2->module_registry = ns->module_registry;
-  menv2->insp = menv->insp;
+  menv2->module_pre_registry = ns->module_pre_registry;
+  menv2->guard_insp = menv->guard_insp;
+  menv2->access_insp = menv->access_insp;
 
   menv2->instance_env = menv2;
 
@@ -1026,6 +1059,9 @@ Scheme_Env *scheme_copy_module_env(Scheme_Env *menv, Scheme_Env *ns, Scheme_Obje
 
   menv2->require_names = menv->require_names;
   menv2->et_require_names = menv->et_require_names;
+  menv2->tt_require_names = menv->tt_require_names;
+  menv2->dt_require_names = menv->dt_require_names;
+  menv2->other_require_names = menv->other_require_names;
 
   if (menv->phase <= clone_phase) {
     menv2->toplevel = menv->toplevel;
@@ -1129,7 +1165,10 @@ Scheme_Object *scheme_get_home_weak_link(Scheme_Env *e)
 {
   if (!e->weak_self_link) {
     Scheme_Object *wb;
-    wb = scheme_make_weak_box((Scheme_Object *)e);
+    if (scheme_starting_up)
+      wb = scheme_box((Scheme_Object *)e);
+    else
+      wb = scheme_make_weak_box((Scheme_Object *)e);
     e->weak_self_link = wb;
   }
 
@@ -1338,6 +1377,10 @@ Scheme_Object **scheme_make_builtin_references_table(void)
   scheme_misc_count += sizeof(Scheme_Object *) * (builtin_ref_counter + 1);
 #endif
 
+  for (j = builtin_ref_counter + 1; j--; ) {
+    t[j] = scheme_false;
+  }
+
   for (j = 0; j < 4; j++) {
     if (!j)
       kenv = kernel_env;
@@ -1433,7 +1476,37 @@ const char *scheme_look_for_primitive(void *code)
 }
 
 /*========================================================================*/
-/*             run-time and expansion-time Scheme interface               */
+/*                  intern literal strings and numbers                    */
+/*========================================================================*/
+
+Scheme_Object *scheme_intern_literal_string(Scheme_Object *str)
+{
+  Scheme_Bucket *b;
+
+  scheme_start_atomic();
+  b = scheme_bucket_from_table(literal_string_table, (const char *)str);
+  scheme_end_atomic_no_swap();
+  if (!b->val)
+    b->val = scheme_true;
+
+  return(Scheme_Object *)HT_EXTRACT_WEAK(b->key);
+}
+
+Scheme_Object *scheme_intern_literal_number(Scheme_Object *num)
+{
+  Scheme_Bucket *b;
+
+  scheme_start_atomic();
+  b = scheme_bucket_from_table(literal_number_table, (const char *)num);
+  scheme_end_atomic_no_swap();
+  if (!b->val)
+    b->val = scheme_true;
+
+  return(Scheme_Object *)HT_EXTRACT_WEAK(b->key);
+}
+
+/*========================================================================*/
+/*             run-time and expansion-time Racket interface               */
 /*========================================================================*/
 
 static Scheme_Object *
@@ -1443,9 +1516,9 @@ namespace_identifier(int argc, Scheme_Object *argv[])
   Scheme_Env *genv;
 
   if (!SCHEME_SYMBOLP(argv[0]))
-    scheme_wrong_type("namespace-symbol->identifier", "symbol", 0, argc, argv);
+    scheme_wrong_contract("namespace-symbol->identifier", "symbol?", 0, argc, argv);
   if ((argc > 1) && !SCHEME_NAMESPACEP(argv[1]))
-    scheme_wrong_type("namespace-symbol->identifier", "namespace", 1, argc, argv);
+    scheme_wrong_contract("namespace-symbol->identifier", "namespace?", 1, argc, argv);
 
   if (argc > 1)
     genv = (Scheme_Env *)argv[1];
@@ -1477,7 +1550,7 @@ namespace_module_identifier(int argc, Scheme_Object *argv[])
     } else if (SCHEME_INTP(argv[0]) || SCHEME_BIGNUMP(argv[0])) {
       phase = argv[0];
     } else {
-      scheme_wrong_type("namespace-module-identifier", "namespace, #f, or exact integer", 0, argc, argv);
+      scheme_wrong_contract("namespace-module-identifier", "(or/c namespace? #f exact-integer?)", 0, argc, argv);
       return NULL;
     }
   } else {
@@ -1495,7 +1568,7 @@ namespace_base_phase(int argc, Scheme_Object *argv[])
   Scheme_Env *genv;
 
   if ((argc > 0) && !SCHEME_NAMESPACEP(argv[0]))
-    scheme_wrong_type("namespace-base-phase", "namespace", 0, argc, argv);
+    scheme_wrong_contract("namespace-base-phase", "namespace?", 0, argc, argv);
 
   if (argc)
     genv = (Scheme_Env *)argv[0];
@@ -1513,13 +1586,13 @@ namespace_variable_value(int argc, Scheme_Object *argv[])
   int use_map;
 
   if (!SCHEME_SYMBOLP(argv[0]))
-    scheme_wrong_type("namespace-variable-value", "symbol", 0, argc, argv);
+    scheme_wrong_contract("namespace-variable-value", "symbol?", 0, argc, argv);
   use_map = ((argc > 1) ? SCHEME_TRUEP(argv[1]) : 1);
   if ((argc > 2) && SCHEME_TRUEP(argv[2])
       && !scheme_check_proc_arity(NULL, 0, 2, argc, argv))
-    scheme_wrong_type("namespace-variable-value", "procedure (arity 0) or #f", 1, argc, argv);
+    scheme_wrong_contract("namespace-variable-value", "(or/c (-> any) #f)", 1, argc, argv);
   if ((argc > 3) && !SCHEME_NAMESPACEP(argv[3]))
-    scheme_wrong_type("namespace-variable-value", "namespace", 3, argc, argv);
+    scheme_wrong_contract("namespace-variable-value", "namespace?", 3, argc, argv);
 
   if (argc > 3)
     genv = (Scheme_Env *)argv[3];
@@ -1539,7 +1612,8 @@ namespace_variable_value(int argc, Scheme_Object *argv[])
       return NULL;
     } else {
       scheme_raise_exn(MZEXN_FAIL_CONTRACT_VARIABLE, argv[0],
-		       "namespace-variable-value: %S is not defined",
+		       "namespace-variable-value: given name is not defined\n"
+                       "  name: %S",
 		       argv[0]);
       return NULL;
     }
@@ -1555,9 +1629,9 @@ namespace_set_variable_value(int argc, Scheme_Object *argv[])
   Scheme_Bucket *bucket;
 
   if (!SCHEME_SYMBOLP(argv[0]))
-    scheme_wrong_type("namespace-set-variable-value!", "symbol", 0, argc, argv);
+    scheme_wrong_contract("namespace-set-variable-value!", "symbol?", 0, argc, argv);
   if ((argc > 3) && !SCHEME_NAMESPACEP(argv[3]))
-    scheme_wrong_type("namespace-set-variable-value!", "namespace", 3, argc, argv);
+    scheme_wrong_contract("namespace-set-variable-value!", "namespace?", 3, argc, argv);
 
   if (argc > 3)
     env = (Scheme_Env *)argv[3];
@@ -1582,9 +1656,9 @@ namespace_undefine_variable(int argc, Scheme_Object *argv[])
   Scheme_Bucket *bucket;
 
   if (!SCHEME_SYMBOLP(argv[0]))
-    scheme_wrong_type("namespace-undefine-variable!", "symbol", 0, argc, argv);
+    scheme_wrong_contract("namespace-undefine-variable!", "symbol?", 0, argc, argv);
   if ((argc > 1) && !SCHEME_NAMESPACEP(argv[1]))
-    scheme_wrong_type("namespace-undefine-variable!", "namespace", 1, argc, argv);
+    scheme_wrong_contract("namespace-undefine-variable!", "namespace?", 1, argc, argv);
 
   if (argc > 1)
     env = (Scheme_Env *)argv[1];
@@ -1600,7 +1674,8 @@ namespace_undefine_variable(int argc, Scheme_Object *argv[])
     bucket->val = NULL;
   } else {
     scheme_raise_exn(MZEXN_FAIL_CONTRACT_VARIABLE, argv[0],
-		     "namespace-undefine-variable!: %S is not defined",
+		     "namespace-undefine-variable!: given name is not defined\n"
+                     "  name: %S",
 		     argv[0]);
   }
 
@@ -1618,7 +1693,7 @@ namespace_mapped_symbols(int argc, Scheme_Object *argv[])
   intptr_t i, j;
 
   if ((argc > 0) && !SCHEME_NAMESPACEP(argv[0]))
-    scheme_wrong_type("namespace-mapped-symbols", "namespace", 0, argc, argv);
+    scheme_wrong_contract("namespace-mapped-symbols", "namespace?", 0, argc, argv);
 
   if (argc)
     env = (Scheme_Env *)argv[0];
@@ -1657,7 +1732,7 @@ namespace_mapped_symbols(int argc, Scheme_Object *argv[])
 static Scheme_Object *namespace_module_registry(int argc, Scheme_Object **argv)
 {
   if (!SCHEME_NAMESPACEP(argv[0]))
-    scheme_wrong_type("namespace-module-registry", "namespace", 0, argc, argv);
+    scheme_wrong_contract("namespace-module-registry", "namespace?", 0, argc, argv);
 
   return (Scheme_Object *)((Scheme_Env *)argv[0])->module_registry;
 }
@@ -1678,9 +1753,7 @@ static Scheme_Object *do_variable_namespace(const char *who, int tl, int argc, S
   }
 
   if (!env)
-    scheme_wrong_type(who, 
-                      "variable-reference", 
-                      0, argc, argv);
+    scheme_wrong_contract(who, "variable-reference?", 0, argc, argv);
 
   ph = env->phase;
   if (tl == 2) {
@@ -1690,11 +1763,12 @@ static Scheme_Object *do_variable_namespace(const char *who, int tl, int argc, S
   } else if (tl == 4) {
     if (((Scheme_Object *)((Scheme_Bucket *)v)->key != scheme_stack_dump_key)
         || !env->module) {
-      scheme_arg_mismatch(who, 
-                          "variable reference does not refer to an anonymous module variable: ",
-                          v);
+      scheme_contract_error(who, 
+                            "variable reference does not refer to an anonymous module variable",
+                            "variable reference", 1, v,
+                            NULL);
     }
-    return env->module->insp;
+    return env->access_insp;
   } else if (tl) {
     /* return env directly; need to set up  */
     if (!env->phase && env->module)
@@ -1742,7 +1816,7 @@ static Scheme_Object *variable_const_p(int argc, Scheme_Object *argv[])
   v = argv[0];
 
   if (!SAME_TYPE(SCHEME_TYPE(v), scheme_global_ref_type))
-    scheme_wrong_type("variable-reference-constant?", "variable-reference", 0, argc, argv);
+    scheme_wrong_contract("variable-reference-constant?", "variable-reference?", 0, argc, argv);
 
   if (SCHEME_VARREF_FLAGS(v) & 0x1)
     return scheme_true;
@@ -1776,7 +1850,7 @@ static Scheme_Object *variable_module_path(int argc, Scheme_Object *argv[])
     env = scheme_get_bucket_home((Scheme_Bucket *)SCHEME_PTR1_VAL(argv[0]));
 
   if (!env)
-    scheme_wrong_type("variable-reference->resolved-module-path", "variable-reference", 0, argc, argv);
+    scheme_wrong_contract("variable-reference->resolved-module-path", "variable-reference?", 0, argc, argv);
 
   if (env->module)
     return env->module->modname;
@@ -1784,6 +1858,27 @@ static Scheme_Object *variable_module_path(int argc, Scheme_Object *argv[])
     return scheme_false;
 }
 
+static Scheme_Object *variable_modidx(int argc, Scheme_Object *argv[])
+{
+  Scheme_Env *env;
+
+  if (!SAME_TYPE(SCHEME_TYPE(argv[0]), scheme_global_ref_type))
+    env = NULL;
+  else
+    env = scheme_get_bucket_home((Scheme_Bucket *)SCHEME_PTR1_VAL(argv[0]));
+
+  if (!env)
+    scheme_wrong_contract("variable-reference->module-path-index", "variable-reference?", 0, argc, argv);
+
+  if (env->module) {
+    if (!env->link_midx) 
+      return env->module->self_modidx;
+    else
+      return env->link_midx;
+  } else
+    return scheme_false;
+}
+
 static Scheme_Object *variable_module_source(int argc, Scheme_Object *argv[])
 {
   Scheme_Env *env;
@@ -1794,7 +1889,7 @@ static Scheme_Object *variable_module_source(int argc, Scheme_Object *argv[])
     env = scheme_get_bucket_home((Scheme_Bucket *)SCHEME_PTR1_VAL(argv[0]));
 
   if (!env)
-    scheme_wrong_type("variable-reference->module-source", "variable-reference", 0, argc, argv);
+    scheme_wrong_contract("variable-reference->module-source", "variable-reference?", 0, argc, argv);
 
   if (env->module)
     return scheme_resolved_module_path_value(env->module->modsrc);
@@ -1818,6 +1913,13 @@ now_transforming_module(int argc, Scheme_Object *argv[])
   return scheme_false;
 }
 
+static void not_currently_transforming(const char *name)
+{
+  scheme_contract_error(name,
+                        "not currently transforming",
+                        NULL);
+}
+
 static Scheme_Object *
 do_local_exp_time_value(const char *name, int argc, Scheme_Object *argv[], int recur)
 {
@@ -1828,9 +1930,7 @@ do_local_exp_time_value(const char *name, int argc, Scheme_Object *argv[], int r
 
   env = scheme_current_thread->current_local_env;
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "%s: not currently transforming",
-                     name);
+    not_currently_transforming(name);
 
   sym = argv[0];
 
@@ -1838,7 +1938,7 @@ do_local_exp_time_value(const char *name, int argc, Scheme_Object *argv[], int r
   SCHEME_EXPAND_OBSERVE_LOCAL_VALUE(observer, sym);
 
   if (!(SCHEME_STXP(sym) && SCHEME_SYMBOLP(SCHEME_STX_VAL(sym))))
-    scheme_wrong_type(name, "syntax identifier", 0, argc, argv);
+    scheme_wrong_contract(name, "identifier?", 0, argc, argv);
 
   if (argc > 1) {
     scheme_check_proc_arity2(name, 0, 1, argc, argv, 1);
@@ -1846,12 +1946,12 @@ do_local_exp_time_value(const char *name, int argc, Scheme_Object *argv[], int r
         && SCHEME_TRUEP(argv[2])) { 
       Scheme_Comp_Env *stx_env;
       if (!SAME_TYPE(scheme_intdef_context_type, SCHEME_TYPE(argv[2])))
-	scheme_wrong_type(name, "internal-definition context or #f", 2, argc, argv);
+	scheme_wrong_contract(name, "(or/c internal-definition-context? #f)", 2, argc, argv);
       stx_env = (Scheme_Comp_Env *)((void **)SCHEME_PTR1_VAL(argv[2]))[0];
       if (!scheme_is_sub_env(stx_env, env)) {
-	scheme_raise_exn(MZEXN_FAIL_CONTRACT, "%s: transforming context does "
-			 "not match given internal-definition context",
-                         name);
+	scheme_contract_error(name, 
+                              "transforming context does not match given internal-definition context",
+                              NULL);
       }
       env = stx_env;
     }
@@ -1869,7 +1969,7 @@ do_local_exp_time_value(const char *name, int argc, Scheme_Object *argv[], int r
 			       + SCHEME_APP_POS + SCHEME_ENV_CONSTANTS_OK
 			       + SCHEME_OUT_OF_CONTEXT_OK + SCHEME_ELIM_CONST),
 			      scheme_current_thread->current_local_modidx, 
-			      &menv, NULL, NULL);
+			      &menv, NULL, NULL, NULL);
 
     SCHEME_EXPAND_OBSERVE_RESOLVE(observer, sym);
 
@@ -1882,11 +1982,12 @@ do_local_exp_time_value(const char *name, int argc, Scheme_Object *argv[], int r
       if ((argc > 1) && SCHEME_TRUEP(argv[1]))
 	return _scheme_tail_apply(argv[1], 0, NULL);
       else
-	scheme_arg_mismatch(name,
-			    (renamed 
-			     ? "not defined as syntax (after renaming): "
-			     : "not defined as syntax: "),
-			    argv[0]);
+	scheme_contract_error(name,
+                              (renamed 
+                               ? "not defined as syntax (after renaming)"
+                               : "not defined as syntax"),
+                              "identifier", 1, argv[0],
+                              NULL);
     }
     
     v = SCHEME_PTR_VAL(v);
@@ -1933,8 +2034,7 @@ local_exp_time_name(int argc, Scheme_Object *argv[])
 
   sym = scheme_current_thread->current_local_name;
   if (!sym)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-name: not currently transforming");
+    not_currently_transforming("syntax-local-name");
 
   return sym;
 }
@@ -1946,8 +2046,7 @@ local_context(int argc, Scheme_Object *argv[])
 
   env = scheme_current_thread->current_local_env;
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-context: not currently transforming");
+    not_currently_transforming("syntax-local-context");
 
   if (env->flags & SCHEME_INTDEF_FRAME) {
     if (!env->intdef_name) {
@@ -2013,15 +2112,17 @@ local_make_intdef_context(int argc, Scheme_Object *argv[])
 
   env = scheme_current_thread->current_local_env;
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, "syntax-local-make-definition-context: not currently transforming");
+    not_currently_transforming("syntax-local-make-definition-context");
   
   if (argc && SCHEME_TRUEP(argv[0])) {
     if (!SAME_TYPE(scheme_intdef_context_type, SCHEME_TYPE(argv[0])))
-      scheme_wrong_type("syntax-local-bind-syntaxes", "internal-definition context or #f", 0, argc, argv);
+      scheme_wrong_contract("syntax-local-bind-syntaxes", "(or/c internal-definition-context? #f)", 0, argc, argv);
     senv = (Scheme_Comp_Env *)((void **)SCHEME_PTR1_VAL(argv[0]))[0];
     if (!scheme_is_sub_env(senv, env)) {
-      scheme_raise_exn(MZEXN_FAIL_CONTRACT, "syntax-local-make-definition-context: transforming context does "
-                       "not match given internal-definition context");
+      scheme_contract_error("syntax-local-make-definition-context",
+                            "transforming context does "
+                            "not match given internal-definition context",
+                            NULL);
     }
     env = senv;
     d[1] = argv[0];
@@ -2049,8 +2150,8 @@ intdef_context_p(int argc, Scheme_Object *argv[])
 static Scheme_Object *intdef_context_seal(int argc, Scheme_Object *argv[])
 {
   if (!SAME_TYPE(SCHEME_TYPE(argv[0]), scheme_intdef_context_type))
-    scheme_wrong_type("internal-definition-context-seal", 
-                      "internal-definition context", 0, argc, argv);
+    scheme_wrong_contract("internal-definition-context-seal", 
+                          "internal-definition-context?", 0, argc, argv);
   
   scheme_stx_seal_rib(SCHEME_PTR2_VAL(argv[0]));
   return scheme_void;
@@ -2062,8 +2163,8 @@ id_intdef_remove(int argc, Scheme_Object *argv[])
   Scheme_Object *l, *res, *skips;
 
   if (!SCHEME_STXP(argv[0]) || !SCHEME_SYMBOLP(SCHEME_STX_VAL(argv[0])))
-    scheme_wrong_type("identifier-remove-from-definition-context", 
-                      "syntax identifier", 0, argc, argv);
+    scheme_wrong_contract("identifier-remove-from-definition-context", 
+                          "identifier?", 0, argc, argv);
   
   l = argv[1];
   if (!SAME_TYPE(SCHEME_TYPE(l), scheme_intdef_context_type)) {
@@ -2073,9 +2174,9 @@ id_intdef_remove(int argc, Scheme_Object *argv[])
       l = SCHEME_CDR(l);
     }
     if (!SCHEME_NULLP(l))
-      scheme_wrong_type("identifier-remove-from-definition-context", 
-                        "internal-definition context or list of internal-definition contexts", 
-                        1, argc, argv);
+      scheme_wrong_contract("identifier-remove-from-definition-context", 
+                            "(or/c internal-definition-context? (listof internal-definition-context?))", 
+                            1, argc, argv);
   }
 
   l = argv[1];
@@ -2109,12 +2210,11 @@ local_introduce(int argc, Scheme_Object *argv[])
 
   env = scheme_current_thread->current_local_env;
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-introduce: not currently transforming");
+    not_currently_transforming("syntax-local-introduce");
 
   s = argv[0];
   if (!SCHEME_STXP(s))
-    scheme_wrong_type("syntax-local-introduce", "syntax", 0, argc, argv);
+    scheme_wrong_contract("syntax-local-introduce", "syntax?", 0, argc, argv);
 
   if (scheme_current_thread->current_local_mark)
     s = scheme_add_remove_mark(s, scheme_current_thread->current_local_mark);
@@ -2130,15 +2230,24 @@ local_module_introduce(int argc, Scheme_Object *argv[])
 
   env = scheme_current_thread->current_local_env;
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-module-introduce: not currently transforming");
+    not_currently_transforming("syntax-local-module-introduce");
 
   s = argv[0];
   if (!SCHEME_STXP(s))
-    scheme_wrong_type("syntax-local-module-introduce", "syntax", 0, argc, argv);
+    scheme_wrong_contract("syntax-local-module-introduce", "syntax?", 0, argc, argv);
 
   v = scheme_stx_source_module(s, 0, 0);
   if (SCHEME_FALSEP(v)) {
+    if (env->genv->module
+        && env->genv->module->rn_stx
+        && SCHEME_VECTORP(env->genv->module->rn_stx)) {
+      /* This is a submodule, and `rn_stx' has renames for the enclosing modules */
+      int i;
+      for (i = SCHEME_VEC_SIZE(env->genv->module->rn_stx); i-- > 1; ) {
+        v = SCHEME_VEC_ELS(env->genv->module->rn_stx)[i];
+        s = scheme_add_rename(s, scheme_stx_to_rename(v));
+      }
+    }
     if (env->genv->rename_set)
       s = scheme_add_rename(s, env->genv->rename_set);
     if (env->genv->post_ex_rename_set)
@@ -2156,14 +2265,13 @@ local_get_shadower(int argc, Scheme_Object *argv[])
 
   env = scheme_current_thread->current_local_env;
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-get-shadower: not currently transforming");
+    not_currently_transforming("syntax-local-get-shadower");
 
   sym = argv[0];
   orig_sym = sym;
 
   if (!(SCHEME_STXP(sym) && SCHEME_SYMBOLP(SCHEME_STX_VAL(sym))))
-    scheme_wrong_type("syntax-local-get-shadower", "syntax identifier", 0, argc, argv);
+    scheme_wrong_contract("syntax-local-get-shadower", "identifier?", 0, argc, argv);
 
   sym_marks = scheme_stx_extract_marks(sym);
 
@@ -2206,7 +2314,7 @@ introducer_proc(void *mark, int argc, Scheme_Object *argv[])
 
   s = argv[0];
   if (!SCHEME_STXP(s))
-    scheme_wrong_type("syntax-introducer", "syntax", 0, argc, argv);
+    scheme_wrong_contract("syntax-introducer", "syntax?", 0, argc, argv);
 
   return scheme_add_remove_mark(s, (Scheme_Object *)mark);
 }
@@ -2230,7 +2338,7 @@ delta_introducer_proc(void *_i_plus_m, int argc, Scheme_Object *argv[])
 
   v = argv[0];
   if (!SCHEME_STXP(v) || !SCHEME_SYMBOLP(SCHEME_STX_VAL(v))) {
-    scheme_wrong_type(who, "identifier", 0, argc, argv);
+    scheme_wrong_contract(who, "identifier?", 0, argc, argv);
   }
   
   /* Apply mapping functions: */
@@ -2248,7 +2356,7 @@ delta_introducer_proc(void *_i_plus_m, int argc, Scheme_Object *argv[])
     v = _scheme_apply(SCHEME_CAR(l), 1, a);
     if (!SCHEME_STXP(v) || !SCHEME_SYMBOLP(SCHEME_STX_VAL(v))) {
       a[0] = v;
-      scheme_wrong_type(who, "identifier", -1, -1, a);
+      scheme_wrong_contract(who, "identifier?", -1, -1, a);
     }
     l = SCHEME_CDR(l);
   }
@@ -2266,11 +2374,10 @@ local_make_delta_introduce(int argc, Scheme_Object *argv[])
 
   env = scheme_current_thread->current_local_env;
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-make-delta-introducer: not currently transforming");
+    not_currently_transforming("syntax-local-make-delta-introducer");
 
   if (!SCHEME_STXP(argv[0]) || !SCHEME_SYMBOLP(SCHEME_STX_VAL(argv[0])))
-    scheme_wrong_type("syntax-local-make-delta-introducer", "syntax identifier", 0, argc, argv);
+    scheme_wrong_contract("syntax-local-make-delta-introducer", "identifier?", 0, argc, argv);
 
   sym = argv[0];
 
@@ -2283,18 +2390,19 @@ local_make_delta_introduce(int argc, Scheme_Object *argv[])
 			       + SCHEME_APP_POS + SCHEME_ENV_CONSTANTS_OK
 			       + SCHEME_OUT_OF_CONTEXT_OK + SCHEME_ELIM_CONST),
 			      scheme_current_thread->current_local_modidx, 
-			      NULL, NULL, &binder);
+			      NULL, NULL, &binder, NULL);
     
     /* Deref globals */
     if (v && SAME_TYPE(SCHEME_TYPE(v), scheme_variable_type))
       v = (Scheme_Object *)(SCHEME_VAR_BUCKET(v))->val;
     
     if (!v || NOT_SAME_TYPE(SCHEME_TYPE(v), scheme_macro_type)) {
-      scheme_arg_mismatch("syntax-local-make-delta-introducer",
-                          (renamed 
-                           ? "not defined as syntax (after renaming): "
-                           : "not defined as syntax: "),
-                          argv[0]);
+      scheme_contract_error("syntax-local-make-delta-introducer",
+                            (renamed 
+                             ? "not defined as syntax (after renaming)"
+                             : "not defined as syntax"),
+                            "identifier", 1, argv[0],
+                            NULL);
     }
 
     if (!binder) {
@@ -2333,7 +2441,7 @@ Scheme_Object *scheme_get_local_inspector()
   Scheme_Thread *p = scheme_current_thread;
 
   if (p->current_local_menv)
-    return p->current_local_menv->module->insp;
+    return p->current_local_menv->access_insp;
   else
     return scheme_get_param(scheme_current_config(), MZCONFIG_CODE_INSPECTOR);
 }
@@ -2346,19 +2454,46 @@ local_module_exports(int argc, Scheme_Object *argv[])
   env = scheme_current_thread->current_local_env;
   
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-module-exports: not currently transforming");
+    not_currently_transforming("syntax-local-module-exports");
 
   return scheme_module_exported_list(argv[0], env->genv);
 }
 
+static Scheme_Object *local_submodules(int argc, Scheme_Object *argv[])
+{
+  Scheme_Comp_Env *env;
+  Scheme_Object *l, *r = scheme_null, *n;
+
+  env = scheme_current_thread->current_local_env;
+  
+  if (!env)
+    not_currently_transforming("syntax-local-submodules");
+
+  if (env->genv->module) {
+    l = env->genv->module->pre_submodules;
+    if (l) {
+      while (!SCHEME_NULLP(l)) {
+        n = scheme_resolved_module_path_value(((Scheme_Module *)SCHEME_CAR(l))->modname);
+        while (SCHEME_PAIRP(SCHEME_CDR(n))) {
+          n = SCHEME_CDR(n);
+        }
+        r = scheme_make_pair(SCHEME_CAR(n), r);
+        l = SCHEME_CDR(l);
+      }
+    }
+  }
+   
+  return r;
+}
+
 static Scheme_Object *
 local_module_definitions(int argc, Scheme_Object *argv[])
 {
   if (!scheme_current_thread->current_local_env
       || !scheme_current_thread->current_local_bindings)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-module-defined-identifiers: not currently transforming module provides");
+    scheme_contract_error("syntax-local-module-defined-identifiers",
+                          "not currently transforming module provides",
+                          NULL);
   
   return SCHEME_CDR(scheme_current_thread->current_local_bindings);
 }
@@ -2368,17 +2503,18 @@ local_module_imports(int argc, Scheme_Object *argv[])
 {
   if (!scheme_current_thread->current_local_env
       || !scheme_current_thread->current_local_bindings)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-module-required-identifiers: not currently transforming module provides");
-
+    scheme_contract_error("syntax-local-module-required-identifiers",
+                          "not currently transforming module provides",
+                          NULL);
+  
   if (SCHEME_TRUEP(argv[0]) && !scheme_is_module_path(argv[0]))
-    scheme_wrong_type("syntax-local-module-required-identifiers", "module-path or #f", 0, argc, argv);
+    scheme_wrong_contract("syntax-local-module-required-identifiers", "(or/c module-path? #f)", 0, argc, argv);
   
   if (!SCHEME_FALSEP(argv[1]) 
       && !SAME_OBJ(scheme_true, argv[1])
       && !SCHEME_INTP(argv[1])
       && !SCHEME_BIGNUMP(argv[1]))
-    scheme_wrong_type("syntax-local-module-required-identifiers", "exact integer, #f, or #t", 1, argc, argv);
+    scheme_wrong_contract("syntax-local-module-required-identifiers", "(or/c exact-integer? #f #t)", 1, argc, argv);
   
   return scheme_module_imported_list(scheme_current_thread->current_local_env->genv,
                                      scheme_current_thread->current_local_bindings,
@@ -2418,8 +2554,7 @@ local_lift_context(int argc, Scheme_Object *argv[])
   env = scheme_current_thread->current_local_env;
 
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-lift-context: not currently transforming");
+    not_currently_transforming("syntax-local-lift-context");
 
   return scheme_local_lift_context(env);
 }
@@ -2432,14 +2567,13 @@ local_lift_end_statement(int argc, Scheme_Object *argv[])
 
   expr = argv[0];
   if (!SCHEME_STXP(expr))
-    scheme_wrong_type("syntax-local-lift-module-end-declaration", "syntax", 0, argc, argv);
+    scheme_wrong_contract("syntax-local-lift-module-end-declaration", "syntax?", 0, argc, argv);
 
   env = scheme_current_thread->current_local_env;
   local_mark = scheme_current_thread->current_local_mark;
 
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-lift-module-end-declaration: not currently transforming");
+    not_currently_transforming("syntax-local-lift-module-end-declaration");
 
   return scheme_local_lift_end_statement(expr, local_mark, env);
 }
@@ -2451,15 +2585,15 @@ static Scheme_Object *local_lift_require(int argc, Scheme_Object *argv[])
   intptr_t phase;
 
   if (!SCHEME_STXP(argv[1]))
-    scheme_wrong_type("syntax-local-lift-require", "syntax", 1, argc, argv);
+    scheme_wrong_contract("syntax-local-lift-require", "syntax?", 1, argc, argv);
 
   env = scheme_current_thread->current_local_env;
   local_mark = scheme_current_thread->current_local_mark;
-  phase = env->genv->phase;
 
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-lift-require: not currently transforming");
+    not_currently_transforming("syntax-local-lift-require");
+
+  phase = env->genv->phase;
 
   return scheme_local_lift_require(argv[0], argv[1], phase, local_mark, env);
 }
@@ -2471,14 +2605,13 @@ static Scheme_Object *local_lift_provide(int argc, Scheme_Object *argv[])
 
   form = argv[0];
   if (!SCHEME_STXP(form))
-    scheme_wrong_type("syntax-local-lift-provide", "syntax", 1, argc, argv);
+    scheme_wrong_contract("syntax-local-lift-provide", "syntax?", 1, argc, argv);
 
   env = scheme_current_thread->current_local_env;
   local_mark = scheme_current_thread->current_local_mark;
 
   if (!env)
-    scheme_raise_exn(MZEXN_FAIL_CONTRACT, 
-		     "syntax-local-lift-provide: not currently transforming");
+    not_currently_transforming("syntax-local-lift-provide");
 
   return scheme_local_lift_provide(form, local_mark, env);
 }
@@ -2509,7 +2642,7 @@ static Scheme_Object *
 set_transformer_proc(int argc, Scheme_Object *argv[])
 {
   if (!scheme_is_set_transformer(argv[0]))
-    scheme_wrong_type("set!-transformer-procedure", "set!-transformer", 1, argc, argv);
+    scheme_wrong_contract("set!-transformer-procedure", "set!-transformer?", 0, argc, argv);
 
   return scheme_set_transformer_proc(argv[0]);
 }
@@ -2520,7 +2653,7 @@ make_rename_transformer(int argc, Scheme_Object *argv[])
   Scheme_Object *v;
 
   if (!SCHEME_STXP(argv[0]) || !SCHEME_SYMBOLP(SCHEME_STX_VAL(argv[0])))
-    scheme_wrong_type("make-rename-transformer", "syntax identifier", 0, argc, argv);
+    scheme_wrong_contract("make-rename-transformer", "identifier?", 0, argc, argv);
 
   if (argc > 1)
     scheme_check_proc_arity("make-rename-transformer", 1, 1, argc, argv);
@@ -2537,7 +2670,7 @@ static Scheme_Object *
 rename_transformer_target(int argc, Scheme_Object *argv[])
 {
   if (!scheme_is_rename_transformer(argv[0]))
-    scheme_wrong_type("rename-transformer-target", "rename transformer", 0, argc, argv);
+    scheme_wrong_contract("rename-transformer-target", "rename-transformer?", 0, argc, argv);
 
   return scheme_rename_transformer_id(argv[0]);
 }
