@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2006-2011 PLT Scheme Inc.
+  Copyright (c) 2006-2012 PLT Scheme Inc.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -78,7 +78,8 @@ static jit_insn *generate_proc_struct_retry(mz_jit_state *jitter, int num_rands,
   CHECK_LIMIT();
 
   /* It's a native closure, but we can't just jump to it, in case
-     the arity is wrong. */
+     the arity is wrong and an error needs to be reported using
+     the original wrapper. */
   mz_prepare(2);
   jit_movi_i(JIT_R0, num_rands);
   jit_pusharg_i(JIT_R0); /* argc */
@@ -148,9 +149,38 @@ Scheme_Object *scheme_ts_scheme_force_value_same_mark(Scheme_Object *v)
 
 #endif
 
+#ifdef MZ_USE_FUTURES
+static Scheme_Object *ts__scheme_tail_apply_from_native(Scheme_Object *rator, int argc, Scheme_Object **argv)
+  XFORM_SKIP_PROC
+{
+  if (scheme_use_rtcall) {
+    /* try thread-local allocation: */
+    Scheme_Object **a;
+#ifdef MZ_PRECISE_GC
+    a = MALLOC_N(Scheme_Object *, argc);
+#else
+    a = NULL; /* future-local allocation is not supported */
+#endif
+    if (a) {
+      Scheme_Thread *p = scheme_current_thread;
+      memcpy(a, argv, argc * sizeof(Scheme_Object*));
+      p->ku.apply.tail_rator = rator;
+      p->ku.apply.tail_num_rands = argc;
+      p->ku.apply.tail_rands = a;
+      return SCHEME_TAIL_CALL_WAITING;
+    } else
+      return scheme_rtcall_tail_apply(rator, argc, argv);
+  } else
+    return _scheme_tail_apply_from_native(rator, argc, argv);
+}
+#else
+# define ts__scheme_tail_apply_from_native _scheme_tail_apply_from_native
+#endif
+
 static Scheme_Object *_scheme_tail_apply_from_native_fixup_args(Scheme_Object *rator,
                                                                 int argc,
                                                                 Scheme_Object **argv)
+  XFORM_SKIP_PROC
 {
   int already = fixup_already_in_place, i;
   Scheme_Object **base;
@@ -162,9 +192,52 @@ static Scheme_Object *_scheme_tail_apply_from_native_fixup_args(Scheme_Object *r
     base[already + i] = argv[i];
   }
 
+  /* In futures mode, it's important that the argument array matches
+     runstack: */
+  MZ_RUNSTACK = base;
+
   return ts__scheme_tail_apply_from_native(rator, argc + already, base);
 }
 
+#if defined(MZ_USE_FUTURES) && defined(MZ_PRECISE_GC)
+
+static Scheme_Object *try_future_local_stack_overflow(Scheme_Object *rator, int argc, Scheme_Object **argv, int multi)
+  XFORM_SKIP_PROC
+{
+  if (SAME_TYPE(SCHEME_TYPE(rator), scheme_native_closure_type)
+      && scheme_can_apply_native_in_future(rator)) {
+    /* the only reason to get here is stack overflow, 
+       either for the runstack or C stack */
+    return scheme_rtcall_apply_with_new_stack(rator, argc, argv, multi);
+  } else if (multi)
+    return ts__scheme_apply_multi_from_native(rator, argc, argv);
+  else
+    return ts__scheme_apply_from_native(rator, argc, argv);
+}
+
+static Scheme_Object *x_ts__scheme_apply_multi_from_native(Scheme_Object *rator, int argc, Scheme_Object **argv)
+  XFORM_SKIP_PROC
+{
+  if (scheme_use_rtcall)
+    return try_future_local_stack_overflow(rator, argc, argv, 1);
+  else
+    return _scheme_apply_multi_from_native(rator, argc, argv);
+}
+
+static Scheme_Object *x_ts__scheme_apply_from_native(Scheme_Object *rator, int argc, Scheme_Object **argv)
+  XFORM_SKIP_PROC
+{
+  if (scheme_use_rtcall)
+    return try_future_local_stack_overflow(rator, argc, argv, 0);
+  else
+    return _scheme_apply_from_native(rator, argc, argv);
+}
+
+#else
+# define x_ts__scheme_apply_multi_from_native ts__scheme_apply_multi_from_native
+# define x_ts__scheme_apply_from_native ts__scheme_apply_from_native
+#endif
+
 static int generate_pause_for_gc_and_retry(mz_jit_state *jitter,
                                            int in_short_jumps,
                                            int gc_reg, /* must not be JIT_R1 */
@@ -375,16 +448,9 @@ int scheme_generate_tail_call(mz_jit_state *jitter, int num_rands, int direct_na
     JIT_UPDATE_THREAD_RSPTR();
   }
   if (direct_native && direct_to_code) {
-    int retptr;
     __END_SHORT_JUMPS__(num_rands < 100);
     /* load closure pointer into R0: */
-    retptr = mz_retain(direct_to_code);
-#ifdef JIT_PRECISE_GC
-    if (retptr)
-      scheme_mz_load_retained(jitter, JIT_R0, retptr);
-    else
-#endif
-      (void)jit_patchable_movi_p(JIT_R0, direct_to_code);
+    scheme_mz_load_retained(jitter, JIT_R0, direct_to_code);
     /* jump directly: */
     (void)jit_jmpi(direct_to_code->code->u.tail_code);
     /* no slow path in this mode */
@@ -457,7 +523,8 @@ int scheme_generate_finish_tail_call(mz_jit_state *jitter, int direct_native)
   jit_pusharg_i(JIT_R0);
   jit_pusharg_p(JIT_V1);
   if (direct_native > 1) { /* => some_args_already_in_place */
-    (void)mz_finish(_scheme_tail_apply_from_native_fixup_args);
+    GC_CAN_IGNORE jit_insn *refr;
+    (void)mz_finish_lwe(_scheme_tail_apply_from_native_fixup_args, refr);
   } else {
     GC_CAN_IGNORE jit_insn *refr;
     (void)mz_finish_lwe(ts__scheme_tail_apply_from_native, refr);
@@ -600,11 +667,16 @@ static int generate_clear_slow_previous_args(mz_jit_state *jitter)
 }
 
 int scheme_generate_non_tail_call(mz_jit_state *jitter, int num_rands, int direct_native, int need_set_rs, 
-				  int multi_ok, int nontail_self, int pop_and_jump, int is_inlined)
+				  int multi_ok, int nontail_self, int pop_and_jump, int is_inlined,
+                                  int unboxed_args)
 {
   /* Non-tail call.
      Proc is in V1, args are at RUNSTACK.
      If nontail_self, then R0 has proc pointer, and R2 has max_let_depth.
+     If unboxed_args, LOCAL3 holds address with argument-copying code,
+      where R2 is set before jumping to the old FP, and R1 holds
+      return address back here, and V1 and R0 must be preserved; 
+      num_rands >= 0 in this case, and the "slow path" returns NULL.
      If num_rands < 0, then argc is in R0, and need to pop runstack before returning.
      If num_rands == -1, skip prolog. */
   GC_CAN_IGNORE jit_insn *ref, *ref2, *ref4, *ref5, *ref6, *ref7, *ref8, *ref9;
@@ -671,6 +743,9 @@ int scheme_generate_non_tail_call(mz_jit_state *jitter, int num_rands, int direc
       jit_movr_p(JIT_R2, JIT_V1);
       jit_movr_p(JIT_V1, JIT_R0);
     }
+    if (unboxed_args) {
+      jit_movr_p(JIT_R2, JIT_FP); /* save old FP */
+    }
     jit_shuffle_saved_regs(); /* maybe copies V registers to be restored */
 #ifdef MZ_USE_JIT_I386
     /* keep call & ret paired by jumping to where we really 
@@ -680,12 +755,12 @@ int scheme_generate_non_tail_call(mz_jit_state *jitter, int num_rands, int direc
     jit_base_prolog();
 #else
     refr = jit_patchable_movi_p(JIT_R1, jit_forward());
-    _jit_prolog_again(jitter, 3, JIT_R1); /* saves V registers (or copied V registers) */
+    _jit_prolog_again(jitter, NATIVE_ARG_COUNT, JIT_R1); /* saves V registers (or copied V registers) */
 #endif
     if (num_rands >= 0) {
       if (nontail_self) { jit_movr_p(JIT_R1, JIT_R0); }
       jit_movr_p(JIT_R0, JIT_V1); /* closure */
-      if (!nontail_self) {
+      if (!nontail_self && !unboxed_args) {
         /* nontail_self is only enabled when there are no rest args: */
         jit_movi_i(JIT_R1, num_rands); /* argc */
         jit_movr_p(JIT_R2, JIT_RUNSTACK); /* argv */
@@ -702,8 +777,20 @@ int scheme_generate_non_tail_call(mz_jit_state *jitter, int num_rands, int direc
       jit_movr_p(JIT_R2, JIT_RUNSTACK); /* argv */
     }
     CHECK_LIMIT();
+    if (unboxed_args) {
+      /* old FP is still in R2 */
+      mz_get_local_p_x(JIT_V1, JIT_LOCAL3, JIT_R2);
+    }
     mz_push_locals();
     mz_repush_threadlocal();
+    if (unboxed_args) {
+      GC_CAN_IGNORE jit_insn *refrr;
+      refrr = jit_patchable_movi_p(JIT_R1, jit_forward());
+      jit_jmpr(JIT_V1);
+      jit_patch_movi(refrr, _jit.x.pc);
+      jit_movi_i(JIT_R1, num_rands); /* argc */
+      jit_movr_p(JIT_R2, JIT_RUNSTACK); /* argv */
+    }
     if (!nontail_self) {
       jit_ldxi_p(JIT_V1, JIT_R0, &((Scheme_Native_Closure *)0x0)->code);
       if (direct_native) {
@@ -858,10 +945,13 @@ int scheme_generate_non_tail_call(mz_jit_state *jitter, int num_rands, int direc
 
   /* The slow way: */
   mz_patch_branch(ref9);
-  generate_pause_for_gc_and_retry(jitter,
-                                  1,  /* in short jumps */
-                                  JIT_V1, /* expose V1 to GC */
-                                  refagain); /* retry code pointer */
+  if (!unboxed_args) {
+    generate_pause_for_gc_and_retry(jitter,
+                                    1,  /* in short jumps */
+                                    JIT_V1, /* expose V1 to GC */
+                                    refagain); /* retry code pointer */
+  }
+
   CHECK_LIMIT();
   if (!direct_native) {
     mz_patch_branch(ref);
@@ -872,24 +962,34 @@ int scheme_generate_non_tail_call(mz_jit_state *jitter, int num_rands, int direc
 #ifndef FUEL_AUTODECEREMENTS
   mz_patch_branch(ref11);
 #endif
-  if (need_set_rs) {
-    JIT_UPDATE_THREAD_RSPTR();
-  }
-  if (num_rands >= 0) {
-    jit_movi_i(JIT_R0, num_rands);
-  }
-  mz_prepare(3);
-  CHECK_LIMIT();
-  jit_pusharg_p(JIT_RUNSTACK);
-  jit_pusharg_i(JIT_R0);
-  jit_pusharg_p(JIT_V1);
-  if (num_rands < 0) { jit_movr_p(JIT_V1, JIT_R0); } /* save argc to manually pop runstack */
-  if (multi_ok) {
-    (void)mz_finish_lwe(ts__scheme_apply_multi_from_native, refrts);
+  if (unboxed_args) {
+    /* no slow path here; return NULL to box arguments fall back to generic */
+    jit_movi_p(JIT_R0, NULL);
+    if (pop_and_jump) {
+      mz_epilog(JIT_V1);
+    }
   } else {
-    (void)mz_finish_lwe(ts__scheme_apply_from_native, refrts);
+    /* normal slow path: */
+    if (need_set_rs) {
+      JIT_UPDATE_THREAD_RSPTR();
+    }
+    if (num_rands >= 0) {
+      jit_movi_i(JIT_R0, num_rands);
+    }
+    mz_prepare(3);
+    CHECK_LIMIT();
+    jit_pusharg_p(JIT_RUNSTACK);
+    jit_pusharg_i(JIT_R0);
+    jit_pusharg_p(JIT_V1);
+    if (num_rands < 0) { jit_movr_p(JIT_V1, JIT_R0); } /* save argc to manually pop runstack */
+    if (multi_ok) {
+      (void)mz_finish_lwe(x_ts__scheme_apply_multi_from_native, refrts);
+    } else {
+      (void)mz_finish_lwe(x_ts__scheme_apply_from_native, refrts);
+    }
+    CHECK_LIMIT();
   }
-  CHECK_LIMIT();
+
   mz_patch_ucbranch(ref5);
   if (!direct_native) {
     mz_patch_ucbranch(ref8);
@@ -923,8 +1023,9 @@ static int generate_self_tail_call(Scheme_Object *rator, mz_jit_state *jitter, i
   GC_CAN_IGNORE jit_insn *refslow, *refagain;
   int i, jmp_tiny, jmp_short;
   int closure_size = jitter->self_closure_size;
-  int space, offset, arg_offset, arg_tmp_offset;
+  int space, offset;
 #ifdef USE_FLONUM_UNBOXING
+  int arg_offset = 1, arg_tmp_offset;
   Scheme_Object *rand;
 #endif
 
@@ -949,10 +1050,11 @@ static int generate_self_tail_call(Scheme_Object *rator, mz_jit_state *jitter, i
 
   __END_TINY_OR_SHORT_JUMPS__(jmp_tiny, jmp_short);
 
-  arg_tmp_offset = offset = jitter->flostack_offset;
+  offset = jitter->flostack_offset;
   space = jitter->flostack_space;
-
-  arg_offset = 1;
+#ifdef USE_FLONUM_UNBOXING
+  arg_tmp_offset = offset;
+#endif
 
   /* Copy args to runstack after closure data: */
   mz_ld_runstack_base_alt(JIT_R2);
@@ -963,13 +1065,11 @@ static int generate_self_tail_call(Scheme_Object *rator, mz_jit_state *jitter, i
     int is_flonum, already_unboxed = 0;
     if ((SCHEME_CLOSURE_DATA_FLAGS(jitter->self_data) & CLOS_HAS_TYPED_ARGS)
         && CLOSURE_ARGUMENT_IS_FLONUM(jitter->self_data, i + args_already_in_place)) {
-      int aoffset;
       is_flonum = 1;
       rand = (alt_rands 
               ? alt_rands[i+1+args_already_in_place] 
               : app->args[i+1+args_already_in_place]);
-      aoffset = JIT_FRAME_FLONUM_OFFSET - (arg_tmp_offset * sizeof(double));
-      jit_ldxi_d_fppush(JIT_FPR0, JIT_FP, aoffset);
+      mz_ld_fppush(JIT_FPR0, arg_tmp_offset);
       --arg_tmp_offset;
       already_unboxed = 1;
       if (!already_loaded && !SAME_TYPE(SCHEME_TYPE(rand), scheme_local_type)) {
@@ -984,11 +1084,9 @@ static int generate_self_tail_call(Scheme_Object *rator, mz_jit_state *jitter, i
     jit_stxi_p(WORDS_TO_BYTES(i + closure_size + args_already_in_place), JIT_R2, JIT_R0);
 #ifdef USE_FLONUM_UNBOXING
     if (is_flonum) {
-      int aoffset;
       if (!already_unboxed)
         jit_ldxi_d_fppush(JIT_FPR0, JIT_R0, &((Scheme_Double *)0x0)->double_val); 
-      aoffset = JIT_FRAME_FLONUM_OFFSET - (arg_offset * sizeof(double));
-      (void)jit_stxi_d_fppop(aoffset, JIT_FP, JIT_FPR0);
+      mz_st_fppop(arg_offset, JIT_FPR0);
       arg_offset++;
     }
 #endif
@@ -1067,20 +1165,19 @@ static int generate_self_tail_call(Scheme_Object *rator, mz_jit_state *jitter, i
         GC_CAN_IGNORE jit_insn *iref;
         mz_pushr_p(JIT_R0);
         mz_ld_runstack_base_alt(JIT_R2);
-        jit_subi_p(JIT_R2, JIT_RUNSTACK_BASE_OR_ALT(JIT_R2), WORDS_TO_BYTES(num_rands + closure_size + args_already_in_place)); 
-        jit_ldxi_p(JIT_R0, JIT_R2, WORDS_TO_BYTES(i+closure_size));
+        jit_subi_p(JIT_R2, JIT_RUNSTACK_BASE_OR_ALT(JIT_R2), WORDS_TO_BYTES(num_rands + args_already_in_place)); 
+        jit_ldxi_p(JIT_R0, JIT_R2, WORDS_TO_BYTES(i));
         mz_rs_sync();
         __START_TINY_JUMPS__(1);
         iref = jit_bnei_p(jit_forward(), JIT_R0, NULL);
         __END_TINY_JUMPS__(1);
         {
-          int aoffset;
-          aoffset = JIT_FRAME_FLONUM_OFFSET - (arg_tmp_offset * sizeof(double));
-          jit_ldxi_d_fppush(JIT_FPR0, JIT_FP, aoffset);
+          int aoffset = JIT_FRAME_FLONUM_OFFSET - (arg_tmp_offset * sizeof(double));
+          jit_movi_l(JIT_R0, aoffset);
           (void)jit_calli(sjc.box_flonum_from_stack_code);
           mz_ld_runstack_base_alt(JIT_R2);
-          jit_subi_p(JIT_R2, JIT_RUNSTACK_BASE_OR_ALT(JIT_R2), WORDS_TO_BYTES(num_rands + closure_size + args_already_in_place)); 
-          jit_stxi_p(WORDS_TO_BYTES(i+closure_size), JIT_R2, JIT_R0);
+          jit_subi_p(JIT_R2, JIT_RUNSTACK_BASE_OR_ALT(JIT_R2), WORDS_TO_BYTES(num_rands + args_already_in_place)); 
+          jit_stxi_p(WORDS_TO_BYTES(i), JIT_R2, JIT_R0);
         }
         __START_TINY_JUMPS__(1);
         mz_patch_branch(iref);
@@ -1117,7 +1214,7 @@ typedef struct {
   mz_jit_state *old_jitter;
   int multi_ok;
   int is_tail;
-  int direct_prim, direct_native, nontail_self;
+  int direct_prim, direct_native, nontail_self, unboxed_args;
 } Generate_Call_Data;
 
 void scheme_jit_register_sub_func(mz_jit_state *jitter, void *code, Scheme_Object *protocol)
@@ -1134,7 +1231,7 @@ void scheme_jit_register_sub_func(mz_jit_state *jitter, void *code, Scheme_Objec
 
 void scheme_jit_register_helper_func(mz_jit_state *jitter, void *code)
 {
-#ifdef MZ_USE_DWARF_LIBUNWIND
+#if defined(MZ_USE_DWARF_LIBUNWIND) || defined(_WIN64)
   /* Null indicates that there's no function name to report, but the
      stack should be unwound manually using the JJIT-generated convention. */
   scheme_jit_register_sub_func(jitter, code, scheme_null);
@@ -1174,7 +1271,7 @@ static int do_generate_shared_call(mz_jit_state *jitter, void *_data)
       ok = generate_direct_prim_non_tail_call(jitter, data->num_rands, data->multi_ok, 1);
     else
       ok = scheme_generate_non_tail_call(jitter, data->num_rands, data->direct_native, 1, 
-                                         data->multi_ok, data->nontail_self, 1, 0);
+                                         data->multi_ok, data->nontail_self, 1, 0, data->unboxed_args);
 
     scheme_jit_register_sub_func(jitter, code, scheme_false);
 
@@ -1183,7 +1280,7 @@ static int do_generate_shared_call(mz_jit_state *jitter, void *_data)
 }
 
 void *scheme_generate_shared_call(int num_rands, mz_jit_state *old_jitter, int multi_ok, int is_tail, 
-				  int direct_prim, int direct_native, int nontail_self)
+				  int direct_prim, int direct_native, int nontail_self, int unboxed_args)
 {
   Generate_Call_Data data;
 
@@ -1194,6 +1291,7 @@ void *scheme_generate_shared_call(int num_rands, mz_jit_state *old_jitter, int m
   data.direct_prim = direct_prim;
   data.direct_native = direct_native;
   data.nontail_self = nontail_self;
+  data.unboxed_args = unboxed_args;
 
   return scheme_generate_one(old_jitter, do_generate_shared_call, &data, 0, NULL, NULL);
 }
@@ -1203,7 +1301,7 @@ void scheme_ensure_retry_available(mz_jit_state *jitter, int multi_ok)
   int mo = multi_ok ? 1 : 0;
   if (!sjc.shared_non_tail_retry_code[mo]) {
     void *code;
-    code = scheme_generate_shared_call(-1, jitter, multi_ok, 0, 0, 0, 0);
+    code = scheme_generate_shared_call(-1, jitter, multi_ok, 0, 0, 0, 0, 0);
     sjc.shared_non_tail_retry_code[mo] = code;
   }
 }
@@ -1341,6 +1439,128 @@ static jit_direct_arg *check_special_direct_args(Scheme_App_Rec *app, Scheme_Obj
   return inline_direct_args;
 }
 
+#ifdef USE_FLONUM_UNBOXING
+
+static int generate_fp_argument_shuffle(int direct_flostack_offset, mz_jit_state *jitter)
+{
+  int i, j;
+
+  /* Copy unboxed flonums into place where the target code expects them,
+     which is shifted and reverse of the order that we pushed. */
+  if (direct_flostack_offset
+      && ((direct_flostack_offset > 1)
+          || (direct_flostack_offset != jitter->flostack_offset))) {
+    /* If the source and target areas don't overlap (or if they
+       overlap only by one item), we can do it in one step, otherwise
+       reverse then shift. */
+    if (jitter->flostack_offset >= ((2 * direct_flostack_offset) - 1)) {
+      /* one step: */
+      if (direct_flostack_offset != jitter->flostack_offset) {
+        /* shift: */
+        for (i = 0; i < direct_flostack_offset; i++) {
+          int i_pos, a_pos;
+          i_pos = jitter->flostack_offset - direct_flostack_offset + i + 1;
+          a_pos = direct_flostack_offset - i;
+          if (i_pos != a_pos) {
+            mz_ld_fppush(JIT_FPR0, i_pos);
+            mz_st_fppop(a_pos, JIT_FPR0);
+            CHECK_LIMIT();
+          }
+        }
+      }
+    } else {
+      /* reverse: */          
+      for (i = 0, j = direct_flostack_offset-1; i < j; i++, j--) {
+        int i_pos, j_pos;
+        i_pos = jitter->flostack_offset - direct_flostack_offset + i + 1;
+        j_pos = jitter->flostack_offset - direct_flostack_offset + j + 1;
+        mz_ld_fppush(JIT_FPR1, i_pos);
+        mz_ld_fppush(JIT_FPR0, j_pos);
+        mz_st_fppop(i_pos, JIT_FPR0);
+        mz_st_fppop(j_pos, JIT_FPR1);
+        CHECK_LIMIT();
+      }
+     
+      if (direct_flostack_offset != jitter->flostack_offset) {
+        /* shift: */
+        for (i = 0; i < direct_flostack_offset; i++) {
+          int i_pos, a_pos;
+          i_pos = jitter->flostack_offset - direct_flostack_offset + i + 1;
+          mz_ld_fppush(JIT_FPR0, i_pos);
+          a_pos = i + 1;
+          mz_st_fppop(a_pos, JIT_FPR0);
+          CHECK_LIMIT();
+        }
+      }
+    }
+  }
+
+  return 1;
+}
+
+static int generate_call_path_with_unboxes(mz_jit_state *jitter, int direct_flostack_offset, void *unboxed_code,
+                                           GC_CAN_IGNORE jit_insn **_refdone,
+                                           int num_rands, Scheme_Closure_Data *direct_data, Scheme_Object *rator)
+{
+  GC_CAN_IGNORE jit_insn *refdone, *refgo, *refcopy;
+  int i, k, offset;
+
+  refgo = jit_jmpi(jit_forward());
+  refcopy = _jit.x.pc;
+
+  /* Callback code to copy unboxed arguments.
+     R1 has the return address, R2 holds the old FP */
+
+  offset = FLOSTACK_SPACE_CHUNK * ((direct_flostack_offset + (FLOSTACK_SPACE_CHUNK - 1)) 
+                                   / FLOSTACK_SPACE_CHUNK);
+  jit_subi_l(JIT_SP, JIT_SP, offset * sizeof(double));
+  
+  for (i = 0; i < direct_flostack_offset; i++) {
+    int i_pos, a_pos;
+    i_pos = jitter->flostack_offset - direct_flostack_offset + i + 1;
+    a_pos = direct_flostack_offset - i;
+    mz_ld_fppush_x(JIT_FPR0, i_pos, JIT_R2);
+    mz_st_fppop(a_pos, JIT_FPR0);
+    CHECK_LIMIT();
+  }
+
+  jit_jmpr(JIT_R1);
+
+  mz_patch_ucbranch(refgo);
+
+  /* install callback pointer and jump to shared code: */
+
+  (void)jit_patchable_movi_p(JIT_R1, refcopy);
+  mz_set_local_p(JIT_R1, JIT_LOCAL3);
+
+  (void)jit_calli(unboxed_code);
+
+  refdone = jit_bnei_p(jit_forward(), JIT_R0, NULL);
+  *_refdone = refdone;
+
+  CHECK_LIMIT();
+
+  /* box arguments for slow path */
+  for (i = 0, k = 0; i < num_rands; i++) {
+    if ((SCHEME_CLOSURE_DATA_FLAGS(direct_data) & CLOS_HAS_TYPED_ARGS)
+        && (CLOSURE_ARGUMENT_IS_FLONUM(direct_data, i))) {
+      k++;
+      offset = jitter->flostack_offset - direct_flostack_offset + k;
+      offset = JIT_FRAME_FLONUM_OFFSET - (offset * sizeof(double));
+      jit_ldxi_p(JIT_R0, JIT_RUNSTACK, WORDS_TO_BYTES(i));
+      scheme_generate_flonum_local_boxing(jitter, i, offset, JIT_R0);
+    }
+  }
+
+  /* Reset V1 to rator for slow path: */
+  scheme_generate(rator, jitter, 0, 0, 0, JIT_V1, NULL);
+  mz_rs_sync();
+  
+  return 1;
+}
+
+#endif
+
 int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_rands, 
 			mz_jit_state *jitter, int is_tail, int multi_ok, int no_call)
 /* de-sync'd ok 
@@ -1350,6 +1570,8 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
   int i, offset, need_safety = 0, apply_to_list = 0;
   int direct_prim = 0, need_non_tail = 0, direct_native = 0, direct_self = 0, nontail_self = 0;
   Scheme_Native_Closure *inline_direct_native = NULL;
+  Scheme_Closure_Data *direct_data = NULL;
+  int direct_flostack_offset = 0, unboxed_non_tail_args = 0;
   jit_direct_arg *inline_direct_args = NULL;
   int proc_already_in_place = 0;
   Scheme_Object *rator, *v, *arg;
@@ -1399,36 +1621,30 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
 	}
       }
     } else if (t == scheme_toplevel_type) {
-      if (0 && (SCHEME_TOPLEVEL_FLAGS(rator) & SCHEME_TOPLEVEL_FLAGS_MASK) >= SCHEME_TOPLEVEL_FIXED) {
+      if ((SCHEME_TOPLEVEL_FLAGS(rator) & SCHEME_TOPLEVEL_FLAGS_MASK) >= SCHEME_TOPLEVEL_FIXED) {
         /* We can re-order evaluation of the rator. */
         reorder_ok = 1;
 
-        if (jitter->nc && 0
+        if (jitter->nc
             && ((SCHEME_TOPLEVEL_FLAGS(rator) & SCHEME_TOPLEVEL_FLAGS_MASK) >= SCHEME_TOPLEVEL_CONST)) {
           Scheme_Object *p;
 
-          p = scheme_extract_global(rator, jitter->nc);
-          p = ((Scheme_Bucket *)p)->val;
-          if (can_direct_native(p, num_rands, &extract_case)) {
-            direct_native = 1;
+          p = scheme_extract_global(rator, jitter->nc, 1);
+          if (p) {
+            p = ((Scheme_Bucket *)p)->val;
+            if (can_direct_native(p, num_rands, &extract_case)) {
+              direct_native = 1;
             
-            if ((SCHEME_TOPLEVEL_POS(rator) == jitter->self_toplevel_pos)
-                && (num_rands < MAX_SHARED_CALL_RANDS)) {
-              if (is_tail)
-                direct_self = 1;
-              else if (jitter->self_nontail_code)
-                nontail_self = 1;
+              if ((SCHEME_TOPLEVEL_POS(rator) == jitter->self_toplevel_pos)
+                  && (num_rands < MAX_SHARED_CALL_RANDS)) {
+                if (is_tail)
+                  direct_self = 1;
+                else if (jitter->self_nontail_code)
+                  nontail_self = 1;
+              }
             }
           }
         }
-      } else if (jitter->nc) {
-        Scheme_Object *p;
-
-        p = scheme_extract_global(rator, jitter->nc);
-        if (((Scheme_Bucket_With_Flags *)p)->flags & GLOB_IS_CONSISTENT) {
-          if (can_direct_native(((Scheme_Bucket *)p)->val, num_rands, &extract_case))
-            direct_native = 1;
-        }
       }
     } else if (SAME_TYPE(t, scheme_closure_type)) {
       Scheme_Closure_Data *data;
@@ -1443,7 +1659,7 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
             direct_self = 1;
           else if (jitter->self_nontail_code)
             nontail_self = 1;
-        } else if (is_tail) {
+        } else {
           Scheme_Closure *c = (Scheme_Closure *)rator;
           if (ZERO_SIZED_CLOSUREP(c)) {
             /* If we're calling a constant function in tail position, then
@@ -1454,16 +1670,25 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
                of the call. */
             Scheme_Native_Closure *nc;
             nc = (Scheme_Native_Closure *)scheme_jit_closure((Scheme_Object *)data, NULL);
-            if (nc->code->code == scheme_on_demand_jit_code) {
+            if (nc->code->start_code == scheme_on_demand_jit_code) {
               if (nc->code->arity_code != sjc.in_progress_on_demand_jit_arity_code) {
-                scheme_on_demand_generate_lambda(nc, 0, NULL);
+                scheme_on_demand_generate_lambda(nc, 0, NULL, 0);
               }
             }
-            if (nc->code->code != scheme_on_demand_jit_code) {
-              if (nc->code->max_let_depth > jitter->max_tail_depth)
-                jitter->max_tail_depth = nc->code->max_let_depth;
-
-              inline_direct_native = nc;
+            if (nc->code->start_code != scheme_on_demand_jit_code) {
+              if (is_tail) {
+                if (nc->code->max_let_depth > jitter->max_tail_depth)
+                  jitter->max_tail_depth = nc->code->max_let_depth;
+                
+                direct_data = data; /* for flonum handling */
+                
+                inline_direct_native = nc;
+              } else {
+                if (num_rands < MAX_SHARED_CALL_RANDS) {
+                  direct_data = data;
+                  unboxed_non_tail_args = 1;
+                }
+              }
             }
           }
         }
@@ -1583,6 +1808,9 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
     }
   }
   /* not sync'd...*/
+  
+  if (direct_self && is_tail)
+    direct_data = jitter->self_data;
 
   for (i = 0; i < num_rands; i++) {
     PAUSE_JIT_DATA();
@@ -1595,11 +1823,9 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
       need_safety = 0;
     }
 #ifdef USE_FLONUM_UNBOXING
-    if (direct_self 
-        && is_tail
-        && (SCHEME_CLOSURE_DATA_FLAGS(jitter->self_data) & CLOS_HAS_TYPED_ARGS)
-        && (CLOSURE_ARGUMENT_IS_FLONUM(jitter->self_data, i+args_already_in_place))) {
-      
+    if (direct_data
+        && (SCHEME_CLOSURE_DATA_FLAGS(direct_data) & CLOS_HAS_TYPED_ARGS)
+        && (CLOSURE_ARGUMENT_IS_FLONUM(direct_data, i+args_already_in_place))) {
       int directly;
       jitter->unbox++;
       if (scheme_can_unbox_inline(arg, 5, JIT_FPR_NUM-1, 0))
@@ -1622,6 +1848,7 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
       } else {
         (void)jit_movi_p(JIT_R0, NULL);
       }
+      direct_flostack_offset++;
     } else
 #endif
       if (inline_direct_args) {
@@ -1713,16 +1940,18 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
           generate_nontail_self_setup(jitter);
         }
 	scheme_generate_non_tail_call(jitter, num_rands, direct_native, jitter->need_set_rs, 
-                                      multi_ok, nontail_self, 0, 1);
+                                      multi_ok, nontail_self, 0, 1, 0);
       }
     }
   } else {
-    /* Jump to code to implement a tail call for num_rands arguments */
+    /* Jump to code to implement a [tail-]call for `num_rands' arguments */
     void *code;
     int dp = (direct_prim ? 1 : (direct_native ? (1 + direct_native + (nontail_self ? 1 : 0)) : 0));
+    /* if unboxed_non_tail_args, then we'll also use index 4 in place of dp */
+
     if (is_tail) {
       if (!sjc.shared_tail_code[dp][num_rands]) {
-	code = scheme_generate_shared_call(num_rands, jitter, multi_ok, is_tail, direct_prim, direct_native, 0);
+	code = scheme_generate_shared_call(num_rands, jitter, multi_ok, is_tail, direct_prim, direct_native, 0, 0);
 	sjc.shared_tail_code[dp][num_rands] = code;
       }
       code = sjc.shared_tail_code[dp][num_rands];
@@ -1732,7 +1961,17 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
 	CHECK_LIMIT();
       } else if (inline_direct_native) {
         LOG_IT(("<-native-tail\n"));
-        scheme_mz_flostack_restore(jitter, 0, 0, 1, 1);
+#ifdef USE_FLONUM_UNBOXING
+        /* Copy unboxed flonums into place where the target code expects them: */
+        generate_fp_argument_shuffle(direct_flostack_offset, jitter);
+        CHECK_LIMIT();
+#endif
+        scheme_mz_flostack_restore(jitter, 
+                                   FLOSTACK_SPACE_CHUNK * ((direct_flostack_offset + (FLOSTACK_SPACE_CHUNK - 1)) 
+                                                           / FLOSTACK_SPACE_CHUNK),
+                                   direct_flostack_offset, 
+                                   1, 1);
+        /* move args and call function: */
         if (args_already_in_place) {
           jit_movi_l(JIT_R2, args_already_in_place);
           mz_set_local_p(JIT_R2, JIT_LOCAL2);
@@ -1756,10 +1995,24 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
       }
     } else {
       int mo = (multi_ok ? 1 : 0);
+      void *unboxed_code;
+
+      if (unboxed_non_tail_args && !direct_flostack_offset)
+        unboxed_non_tail_args = 0;
+
+      if (unboxed_non_tail_args) {
+        if (!sjc.shared_non_tail_code[4][num_rands][mo]) {
+          scheme_ensure_retry_available(jitter, multi_ok);
+          code = scheme_generate_shared_call(num_rands, jitter, multi_ok, is_tail, direct_prim, direct_native, nontail_self, 1);
+          sjc.shared_non_tail_code[4][num_rands][mo] = code;
+        }
+        unboxed_code = sjc.shared_non_tail_code[4][num_rands][mo];
+      } else
+        unboxed_code = NULL;
 
       if (!sjc.shared_non_tail_code[dp][num_rands][mo]) {
         scheme_ensure_retry_available(jitter, multi_ok);
-	code = scheme_generate_shared_call(num_rands, jitter, multi_ok, is_tail, direct_prim, direct_native, nontail_self);
+	code = scheme_generate_shared_call(num_rands, jitter, multi_ok, is_tail, direct_prim, direct_native, nontail_self, 0);
 	sjc.shared_non_tail_code[dp][num_rands][mo] = code;
       }
       LOG_IT(("<-non-tail %d %d %d\n", dp, num_rands, mo));
@@ -1776,7 +2029,20 @@ int scheme_generate_app(Scheme_App_Rec *app, Scheme_Object **alt_rands, int num_
         else
           (void)jit_calli(sjc.apply_to_list_code);
       } else {
+        GC_CAN_IGNORE jit_insn *refdone = NULL;
+
+#ifdef USE_FLONUM_UNBOXING
+        if (unboxed_code) {
+          generate_call_path_with_unboxes(jitter, direct_flostack_offset, unboxed_code, &refdone,
+                                          num_rands, direct_data, rator);
+          CHECK_LIMIT();
+        }
+#endif
+
         (void)jit_calli(code);
+
+        if (refdone)
+          mz_patch_branch(refdone);
       }
 
       if (direct_prim) {
