@@ -367,7 +367,7 @@ cont_proc {
   gcMARK2(c->native_trace, gc);
 #endif
 
-  MARK_jmpup(&c->buf, gc);
+  gcMARK2(c->buf_ptr, gc);
   MARK_cjs(&c->cjs, gc);
   MARK_stack_state(&c->ss, gc);
   gcMARK2(c->barrier_prompt, gc);
@@ -391,6 +391,16 @@ cont_proc {
   gcBYTES_TO_WORDS(sizeof(Scheme_Cont));
 }
 
+cont_jmp_proc {
+ mark:
+  Scheme_Cont_Jmp *c = (Scheme_Cont_Jmp *)p;
+
+  MARK_jmpup(&c->buf, gc);
+  
+ size:
+  gcBYTES_TO_WORDS(sizeof(Scheme_Cont_Jmp));
+}
+
 meta_cont_proc {
  mark:
   Scheme_Meta_Continuation *c = (Scheme_Meta_Continuation *)p;
@@ -589,12 +599,14 @@ input_port {
  mark:
   Scheme_Input_Port *ip = (Scheme_Input_Port *)p;
   
+  gcMARK2(ip->p.position_redirect, gc);
   gcMARK2(ip->sub_type, gc);
   gcMARK2(ip->port_data, gc);
   gcMARK2(ip->name, gc);
   gcMARK2(ip->peeked_read, gc);
   gcMARK2(ip->peeked_write, gc);
   gcMARK2(ip->read_handler, gc);
+  gcMARK2(ip->closed_evt, gc);
   gcMARK2(ip->mref, gc);
   gcMARK2(ip->output_half, gc);
   gcMARK2(ip->special, gc);
@@ -615,12 +627,14 @@ output_port {
  mark:
   Scheme_Output_Port *op = (Scheme_Output_Port *)p;
 
+  gcMARK2(op->p.position_redirect, gc);
   gcMARK2(op->sub_type, gc);
   gcMARK2(op->port_data, gc);
   gcMARK2(op->name, gc);
   gcMARK2(op->display_handler, gc);
   gcMARK2(op->write_handler, gc);
   gcMARK2(op->print_handler, gc);
+  gcMARK2(op->closed_evt, gc);
   gcMARK2(op->mref, gc);
   gcMARK2(op->input_half, gc);
 
@@ -638,7 +652,7 @@ syntax_compiler {
 thread_val {
  mark:
   Scheme_Thread *pr = (Scheme_Thread *)p;
-  
+
   gcMARK2(pr->next, gc);
   gcMARK2(pr->prev, gc);
 
@@ -866,7 +880,9 @@ namespace_val {
 
   gcMARK2(e->module, gc);
   gcMARK2(e->module_registry, gc);
-  gcMARK2(e->insp, gc);
+  gcMARK2(e->module_pre_registry, gc);
+  gcMARK2(e->guard_insp, gc);
+  gcMARK2(e->access_insp, gc);
 
   gcMARK2(e->rename_set, gc);
   gcMARK2(e->temp_marked_names, gc);
@@ -933,6 +949,7 @@ prefix_val {
   int i;
   for (i = pf->num_slots; i--; )
     gcMARK2(pf->a[i], gc);
+  gcMARK2(pf->import_map, gc);
  size:
   gcBYTES_TO_WORDS((sizeof(Scheme_Prefix) 
 		    + ((pf->num_slots-mzFLEX_DELTA) * sizeof(Scheme_Object *))
@@ -956,6 +973,7 @@ comp_prefix_val {
  mark:
   Comp_Prefix *cp = (Comp_Prefix *)p;
   gcMARK2(cp->toplevels, gc);
+  gcMARK2(cp->inline_variants, gc);
   gcMARK2(cp->unbound, gc);
   gcMARK2(cp->stxes, gc);
   gcMARK2(cp->uses_unsafe, gc);
@@ -1025,6 +1043,7 @@ module_val {
 
   gcMARK2(m->hints, gc);
   gcMARK2(m->ii_src, gc);
+  gcMARK2(m->super_bxs_info, gc);
 
   gcMARK2(m->comp_prefix, gc);
   gcMARK2(m->prefix, gc);
@@ -1032,6 +1051,12 @@ module_val {
 
   gcMARK2(m->rn_stx, gc);
 
+  gcMARK2(m->submodule_path, gc);
+  gcMARK2(m->pre_submodules, gc);
+  gcMARK2(m->post_submodules, gc);
+  gcMARK2(m->supermodule, gc);
+  gcMARK2(m->submodule_ancestry, gc);
+
   gcMARK2(m->primitive, gc);
  size:
   gcBYTES_TO_WORDS(sizeof(Scheme_Module));
@@ -1149,8 +1174,10 @@ mark_logger {
   Scheme_Logger *l = (Scheme_Logger *)p;
   gcMARK2(l->name, gc);
   gcMARK2(l->parent, gc);
-  gcMARK2(l->readers, gc);
   gcMARK2(l->timestamp, gc);
+  gcMARK2(l->syslog_level, gc);
+  gcMARK2(l->stderr_level, gc);
+  gcMARK2(l->readers, gc);
  size:
   gcBYTES_TO_WORDS(sizeof(Scheme_Logger));
 }
@@ -1158,6 +1185,7 @@ mark_logger {
 mark_log_reader {
  mark:
   Scheme_Log_Reader *lr = (Scheme_Log_Reader *)p;
+  gcMARK2(lr->level, gc);
   gcMARK2(lr->sema, gc);
   gcMARK2(lr->head, gc);
   gcMARK2(lr->tail, gc);
@@ -1230,6 +1258,19 @@ mark_resolve_info {
   gcBYTES_TO_WORDS(sizeof(Resolve_Info));
 }
 
+mark_unresolve_info {
+ mark:
+  Unresolve_Info *i = (Unresolve_Info *)p;
+  
+  gcMARK2(i->flags, gc);
+  gcMARK2(i->depths, gc);
+  gcMARK2(i->prefix, gc);
+  gcMARK2(i->closures, gc);
+
+ size:
+  gcBYTES_TO_WORDS(sizeof(Unresolve_Info));
+}
+
 END resolve;
 
 /**********************************************************************/
@@ -1263,10 +1304,12 @@ mark_optimize_info {
   gcMARK2(i->next, gc);
   gcMARK2(i->use, gc);
   gcMARK2(i->consts, gc);
+  gcMARK2(i->cp, gc);
   gcMARK2(i->top_level_consts, gc);
   gcMARK2(i->transitive_use, gc);
   gcMARK2(i->transitive_use_len, gc);
   gcMARK2(i->context, gc);
+  gcMARK2(i->logger, gc);
 
  size:
   gcBYTES_TO_WORDS(sizeof(Optimize_Info));
@@ -1414,22 +1457,22 @@ hash_tree_val {
   gcBYTES_TO_WORDS(sizeof(Scheme_Hash_Tree));
 }
 
-mark_rb_node {
+mark_avl_node {
  mark:
-  RBNode *rb = (RBNode *)p;
+  AVLNode *avl = (AVLNode *)p;
 
   /* Short-circuit on NULL pointers, which are especially likely */
-  if (rb->left) {
-    gcMARK2(rb->left, gc);
+  if (avl->left) {
+    gcMARK2(avl->left, gc);
   }
-  if (rb->right) {
-    gcMARK2(rb->right, gc);
+  if (avl->right) {
+    gcMARK2(avl->right, gc);
   }
-  gcMARK2(rb->key, gc);
-  gcMARK2(rb->val, gc);
+  gcMARK2(avl->key, gc);
+  gcMARK2(avl->val, gc);
 
  size:
-  gcBYTES_TO_WORDS(sizeof(RBNode));
+  gcBYTES_TO_WORDS(sizeof(AVLNode));
 }
 
 END hash;
@@ -1460,6 +1503,9 @@ place_val {
   gcMARK2(pr->channel, gc);
   gcMARK2(pr->mref, gc);
   gcMARK2(pr->pumper_threads, gc);
+  gcMARK2(pr->place_obj, gc);
+  gcMARK2(pr->prev, gc);
+  gcMARK2(pr->next, gc);
 
  size:
   gcBYTES_TO_WORDS(sizeof(Scheme_Place));
@@ -1468,10 +1514,25 @@ place_val {
 place_async_channel_val {
  mark:
   Scheme_Place_Async_Channel *pac = (Scheme_Place_Async_Channel *)p;
+  Scheme_Object *pr;
+  int i, j, sz;
   gcMARK2(pac->msgs, gc);
   gcMARK2(pac->msg_memory, gc);
+  gcMARK2(pac->msg_chains, gc);
   gcMARK2(pac->wakeup_signal, gc);
 
+  /* mark master-allocated objects within each messages: */
+  j = pac->out;
+  sz = pac->size;
+  for (i = pac->count; i--; ) {
+    pr = pac->msg_chains[j];
+    while (pr) {
+      gcMARK2(SCHEME_CAR(pr), gc);
+      pr = SCHEME_CDR(pr);
+    }
+    j = ((j + 1) & sz);
+  }
+
  size:
   gcBYTES_TO_WORDS(sizeof(Scheme_Place_Async_Channel));
 }
@@ -1509,7 +1570,6 @@ mark_load_handler_data {
   gcMARK2(d->p, gc);
   gcMARK2(d->stxsrc, gc);
   gcMARK2(d->expected_module, gc);
-  gcMARK2(d->delay_load_info, gc);
   
  size:
   gcBYTES_TO_WORDS(sizeof(LoadHandlerData));
@@ -1699,6 +1759,9 @@ mark_listener {
  mark:
 
   gcMARK2(l->mref, gc);
+# ifdef HAVE_POLL_SYSCALL
+  gcMARK2(l->pfd, gc);
+# endif
 
  size:
   gcBYTES_TO_WORDS(sizeof(listener_t) + ((l->count - mzFLEX_DELTA) * sizeof(tcp_t)));
@@ -1872,6 +1935,7 @@ mark_syncing {
   gcMARK2(w->reposts, gc);
   gcMARK2(w->accepts, gc);
   gcMARK2(w->disable_break, gc);
+  gcMARK2(w->thread, gc);
 
  size:
   gcBYTES_TO_WORDS(sizeof(Syncing));
@@ -2409,7 +2473,6 @@ future {
   gcMARK2(f->arg_S1, gc);
   gcMARK2(f->arg_s2, gc);
   gcMARK2(f->arg_S2, gc);
-  gcMARK2(f->arg_p, gc);
   gcMARK2(f->arg_S4, gc);
   gcMARK2(f->retval_s, gc);
   gcMARK2(f->retval, gc);
@@ -2422,6 +2485,7 @@ future {
   gcMARK2(f->next_waiting_lwc, gc);
   gcMARK2(f->next_waiting_touch, gc);
   gcMARK2(f->suspended_lw, gc);
+  gcMARK2(f->suspended_lw_stack, gc);
   gcMARK2(f->prev_in_fsema_queue, gc);
   gcMARK2(f->next_in_fsema_queue, gc);
   gcMARK2(f->touching, gc);
