@@ -24,11 +24,11 @@
 */
 
 #define NEED_NUMBER(name) \
-  scheme_wrong_type(#name, "number", 0, argc, argv)
+  scheme_wrong_contract(#name, "number?", 0, argc, argv)
 #define NEED_REAL(name) \
-  scheme_wrong_type(#name, REAL_NUMBER_STR, 0, argc, argv)
+  scheme_wrong_contract(#name, "real?", 0, argc, argv)
 #define NEED_INTEGER(name) \
-  scheme_wrong_type(#name, "integer", 0, argc, argv)
+  scheme_wrong_contract(#name, "integer", 0, argc, argv)
 
 #define rat_from_float(d, sr) force_rat(scheme_rational_from_float(d), sr)
 #define rat_from_double(d, sr) force_rat(scheme_rational_from_double(d), sr)
@@ -61,13 +61,13 @@ name ## __slow (Scheme_Object *p, int argc, Scheme_Object *argv[]) \
   for (i = 1; i < argc; i++) {\
     o = argv[i]; \
     if (!TYPEP(o)) { \
-      scheme_wrong_type(scheme_name, type, i, argc, argv); \
+      scheme_wrong_contract(scheme_name, type, i, argc, argv); \
       return NULL; \
     } \
     if (!bin_name(p, o)) { \
         for (i++; i < argc; i++) { \
           if (!TYPEP(argv[i])) \
-           scheme_wrong_type(scheme_name, type, i, argc, argv); \
+           scheme_wrong_contract(scheme_name, type, i, argc, argv); \
         } \
         return scheme_false; \
     } \
@@ -84,11 +84,11 @@ name (int argc, Scheme_Object *argv[]) \
   Scheme_Object *p, *p2; \
   p = argv[0]; \
   if (!TYPEP(p)) \
-   scheme_wrong_type(scheme_name, type, 0, argc, argv); \
+   scheme_wrong_contract(scheme_name, type, 0, argc, argv); \
   if (argc == 2) { \
     p2 = argv[1]; \
     if (!TYPEP(p2)) \
-      scheme_wrong_type(scheme_name, type, 1, argc, argv); \
+      scheme_wrong_contract(scheme_name, type, 1, argc, argv); \
     return name ## __bin(p, p2); \
   } else \
     return name ## __slow(p, argc, argv); \
@@ -111,11 +111,11 @@ static Scheme_Object *name (const Scheme_Object *n1, const Scheme_Object *n2)
                       toi_or_toe, \
                       check_exact_zero1, check_exact_one1, check_exact_zero2, check_exact_one2) \
 rettype name (const Scheme_Object *n1, const Scheme_Object *n2); \
-static rettype name ## __wrong_type(const Scheme_Object *v) \
+static rettype name ## __wrong_contract(const Scheme_Object *v) \
 { \
   Scheme_Object *a[1]; \
   a[0] = (Scheme_Object *)v; \
-  scheme_wrong_type(scheme_name, numbertype, -1, 0, a); \
+  scheme_wrong_contract(scheme_name, numbertype, -1, 0, a); \
   return 0; \
 } \
 static MZ_INLINE rettype name ## __int_big(const Scheme_Object *n1, const Scheme_Object *n2) { \
@@ -144,21 +144,23 @@ static MZ_INLINE rettype name ## __int_comp(const Scheme_Object *n1, const Schem
 }) \
 FLOATWRAP( \
 static MZ_INLINE rettype name ## __flt_big(float d1, const Scheme_Object *n1, const Scheme_Object *n2) { \
-  toi_or_toe(,Small_Rational sr2); \
+  toi_or_toe(float tmp2, Small_Rational sr2); \
   snanchk_more(d1); \
   wrap(if (MZ_IS_POS_INFINITY(d1)) return combineinf(swaybigf, n2);) \
   wrap(if (MZ_IS_NEG_INFINITY(d1)) return combineinf(swaysmallf, n2);) \
-  return toi_or_toe(fsop(d1, scheme_bignum_to_float(n2)), \
+  toi_or_toe(tmp2=scheme_bignum_to_float(n2),); \
+  return toi_or_toe(fsop(d1, tmp2), \
                     rop(rat_from_float(d1, &sr2), scheme_integer_to_rational(n2))); \
 }) \
 FLOATWRAP( \
 static MZ_INLINE rettype name ## __flt_rat(float d1, const Scheme_Object *n1, const Scheme_Object *n2) { \
-  toi_or_toe(,Small_Rational sr3); \
+  toi_or_toe(float tmp3, Small_Rational sr3); \
   snanchk_more(d1); \
   wrap(if (MZ_IS_POS_INFINITY(d1)) return combineinf(swaybigf, n2);) \
   wrap(if (MZ_IS_NEG_INFINITY(d1)) return combineinf(swaysmallf, n2);) \
   wrap(if (d1 == 0.0) return combinezero(sfirstzero, n2, d1);) \
-  return toi_or_toe(fsop(d1, scheme_rational_to_float(n2)), \
+  toi_or_toe(tmp3=scheme_rational_to_float(n2),); \
+  return toi_or_toe(fsop(d1, tmp3), \
                     rop(rat_from_float(d1, &sr3), (n2))); \
 })\
 FLOATWRAP(complexwrap(  \
@@ -169,20 +171,22 @@ static MZ_INLINE rettype name ## __flt_comp(float d1, const Scheme_Object *n1, c
   (n2)); \
 }))    \
 static MZ_INLINE rettype name ## __dbl_big(double d1, const Scheme_Object *n1, const Scheme_Object *n2) { \
-  toi_or_toe(,Small_Rational sr4); \
+  toi_or_toe(double tmp4, Small_Rational sr4); \
   nanchk_more(d1); \
   wrap(if (MZ_IS_POS_INFINITY(d1)) return combineinf(waybigf, n2);) \
   wrap(if (MZ_IS_NEG_INFINITY(d1)) return combineinf(waysmallf, n2);) \
-  return toi_or_toe(fop(d1, scheme_bignum_to_double(n2)), \
+  toi_or_toe(tmp4=scheme_bignum_to_double(n2),); \
+  return toi_or_toe(fop(d1, tmp4), \
                     rop(rat_from_double(d1, &sr4), scheme_integer_to_rational(n2))); \
 } \
 static MZ_INLINE rettype name ## __dbl_rat(double d1, const Scheme_Object *n1, const Scheme_Object *n2) { \
-  toi_or_toe(,Small_Rational sr5);       \
+  toi_or_toe(double tmp5, Small_Rational sr5);       \
   nanchk_more(d1); \
   wrap(if (MZ_IS_POS_INFINITY(d1)) return combineinf(waybigf, n2);) \
   wrap(if (MZ_IS_NEG_INFINITY(d1)) return combineinf(waysmallf, n2);) \
   wrap(if (d1 == 0.0) return combinezero(firstzero, n2, d1);) \
-  return toi_or_toe(fop(d1, scheme_rational_to_double(n2)), \
+  toi_or_toe(tmp5=scheme_rational_to_double(n2),); \
+  return toi_or_toe(fop(d1, tmp5), \
                     rop(rat_from_double(d1, &sr5), (n2))); \
 } \
 complexwrap( \
@@ -201,22 +205,24 @@ static MZ_INLINE rettype name ## __big_int(const Scheme_Object *n1, const Scheme
 FLOATWRAP( \
 static MZ_INLINE rettype name ## __big_flt(const Scheme_Object *n1, const Scheme_Object *n2) { \
   float d2; \
-  toi_or_toe(,Small_Rational sr6); \
+  toi_or_toe(float tmp6, Small_Rational sr6); \
   d2 = SCHEME_FLT_VAL(n2); \
   snanchk_more(d2); \
   wrap(if (MZ_IS_POS_INFINITY(d2)) return combineinf(swaysmalls, n1);) \
   wrap(if (MZ_IS_NEG_INFINITY(d2)) return combineinf(swaybigs, n1);) \
-  return toi_or_toe(fsop(scheme_bignum_to_float(n1), d2), \
+  toi_or_toe(tmp6=scheme_bignum_to_float(n1),); \
+  return toi_or_toe(fsop(tmp6, d2), \
                     rop(scheme_integer_to_rational(n1), rat_from_float(d2, &sr6))); \
 }) \
 static MZ_INLINE rettype name ## __big_dbl(const Scheme_Object *n1, const Scheme_Object *n2) { \
   double d2; \
-  toi_or_toe(,Small_Rational sr7); \
+  toi_or_toe(double tmp7, Small_Rational sr7); \
   d2 = SCHEME_DBL_VAL(n2); \
   nanchk_more(d2); \
   wrap(if (MZ_IS_POS_INFINITY(d2)) return combineinf(waysmalls, n1);) \
   wrap(if (MZ_IS_NEG_INFINITY(d2)) return combineinf(waybigs, n1);) \
-  return toi_or_toe(fop(scheme_bignum_to_double(n1), d2), \
+  toi_or_toe(tmp7=scheme_bignum_to_double(n1),); \
+  return toi_or_toe(fop(tmp7, d2), \
                     rop(scheme_integer_to_rational(n1), rat_from_double(d2, &sr7))); \
 } \
 static MZ_INLINE rettype name ## __big_rat(const Scheme_Object *n1, const Scheme_Object *n2) { \
@@ -237,24 +243,26 @@ static MZ_INLINE rettype name ## __rat_int(const Scheme_Object *n1, const Scheme
 FLOATWRAP( \
 static MZ_INLINE rettype name ## __rat_flt(const Scheme_Object *n1, const Scheme_Object *n2) { \
   float d2; \
-  toi_or_toe(,Small_Rational sr9); \
+  toi_or_toe(float tmp9, Small_Rational sr9); \
   d2 = SCHEME_FLT_VAL(n2); \
   snanchk_more(d2); \
   wrap(if (MZ_IS_POS_INFINITY(d2)) return combineinf(swaysmalls, n1);) \
   wrap(if (MZ_IS_NEG_INFINITY(d2)) return combineinf(swaybigs, n1);) \
   wrap(if (d2 == 0.0) return combinezero(ssecondzero, n1, d2);) \
-  return toi_or_toe(fsop(scheme_rational_to_float(n1), d2), \
+  toi_or_toe(tmp9=scheme_rational_to_float(n1),); \
+  return toi_or_toe(fsop(tmp9, d2), \
                     rop((n1), rat_from_float(d2, &sr9))); \
 }) \
 static MZ_INLINE rettype name ## __rat_dbl(const Scheme_Object *n1, const Scheme_Object *n2) { \
   double d2; \
-  toi_or_toe(,Small_Rational sr10);      \
+  toi_or_toe(double tmp10, Small_Rational sr10);      \
   d2 = SCHEME_DBL_VAL(n2); \
   nanchk_more(d2); \
   wrap(if (MZ_IS_POS_INFINITY(d2)) return combineinf(waysmalls, n1);) \
   wrap(if (MZ_IS_NEG_INFINITY(d2)) return combineinf(waybigs, n1);) \
   wrap(if (d2 == 0.0) return combinezero(secondzero, n1, d2);) \
-  return toi_or_toe(fop(scheme_rational_to_double(n1), d2), \
+  toi_or_toe(tmp10=scheme_rational_to_double(n1),); \
+  return toi_or_toe(fop(tmp10, d2), \
                     rop((n1), rat_from_double(d2, &sr10))); \
 } \
 static MZ_INLINE rettype name ## __rat_big(const Scheme_Object *n1, const Scheme_Object *n2) { \
@@ -328,7 +336,7 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
         return name ## __int_comp(n1, n2); \
       } \
       ) \
-      return name ## __wrong_type(n2); \
+      return name ## __wrong_contract(n2); \
     } \
   else { \
    t1 = _SCHEME_TYPE(n1); \
@@ -364,7 +372,7 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
         return name ## __flt_comp(d1, n1, n2);        \
       } \
       )\
-      return name ## __wrong_type(n2); \
+      return name ## __wrong_contract(n2); \
     } else \
    ) \
    if (t1 == scheme_double_type) \
@@ -400,7 +408,7 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
         return name ## __dbl_comp(d1, n1, n2); \
       } \
       )\
-      return name ## __wrong_type(n2); \
+      return name ## __wrong_contract(n2); \
     } \
   else if (t1 == scheme_bignum_type) \
     { \
@@ -425,7 +433,7 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
 	 return name ## __big_comp(n1, n2); \
        } \
        )\
-       return name ## __wrong_type(n2); \
+       return name ## __wrong_contract(n2); \
     } \
   else if (t1 == scheme_rational_type) \
     { \
@@ -450,7 +458,7 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
          return name ## __rat_comp(n1, n2); \
        } \
        )\
-       return name ## __wrong_type(n2); \
+       return name ## __wrong_contract(n2); \
     } \
   complexwrap( \
   else if (noniziwrap((t1 == scheme_complex_type))) \
@@ -472,11 +480,11 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
          return name ## __comp_rat(n1, n2); \
        if (noniziwrap((t2 == scheme_complex_type))) \
 	 return cxop((n1), (n2)); \
-       return name ## __wrong_type(n2); \
+       return name ## __wrong_contract(n2); \
     } \
   ) \
   else \
-       return name ## __wrong_type(n1); \
+       return name ## __wrong_contract(n1); \
   } \
 }
 
@@ -532,15 +540,15 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
 
 # define NAN_CHECK_0(x) if (MZ_IS_NAN(x)) return 0
 
-#define GEN_BIN_OP(name, scheme_name, iop, fop, fsop, bn_op, rop, cxop, exzeopl, exzeopr, nanckop, snanckop, c0_1, c1_1, c0_2, c1_2) \
+#define GEN_BIN_OP(name, scheme_name, iop, fop, fsop, bn_op, rop, cxop, exzeopl, exzeopr, nanckop, snanckop, nanckop_more, snanckop_more, c0_1, c1_1, c0_2, c1_2) \
   GEN_BIN_THING(Scheme_Object *, name, scheme_name, \
                 iop, fop, fsop, bn_op, rop, cxop, \
                 GEN_OMIT, GEN_FIRST_ONLY, \
                 0, 0, 0, 0, \
                 0, 0, 0, 0, \
                 GEN_SCHEME_BOOL_APPLY, badfunc, badfunc, badfunc, badfunc, \
-                nanckop, snanckop, nanckop, snanckop, \
-                GEN_IDENT, GEN_IDENT, exzeopl, exzeopr, "number", GEN_TOI, \
+                nanckop, snanckop, nanckop_more, snanckop_more, \
+                GEN_IDENT, GEN_IDENT, exzeopl, exzeopr, "number?", GEN_TOI, \
                 c0_1, c1_1, c0_2, c1_2)
 
 #define GEN_BIN_DIV_OP(name, scheme_name, iop, fop, fsop, bn_op, rop, cxop, c0_1, c1_1, c0_2, c1_2) \
@@ -551,7 +559,7 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
                 GEN_MAKE_NZERO, GEN_MAKE_NSZERO, GEN_MAKE_PZERO, GEN_MAKE_PSZERO, \
                 GEN_APPLY3, GEN_MAKE_ZERO_Z, GEN_MAKE_SZERO_Z, GEN_SAME_INF_Z, GEN_SAME_SINF_Z, \
                 NAN_CHECK_NAN_IF_WEIRD, SNAN_CHECK_NAN_IF_WEIRD, NAN_CHECK_NAN_IF_WEIRD, SNAN_CHECK_NAN_IF_WEIRD, \
-                GEN_IDENT, GEN_IDENT, GEN_RETURN_0, GEN_OMIT, "number", GEN_TOI, \
+                GEN_IDENT, GEN_IDENT, GEN_RETURN_0, GEN_OMIT, "number?", GEN_TOI, \
                 c0_1, c1_1, c0_2, c1_2)
 
 #define GEN_BIN_COMP(name, scheme_name, iop, fop, bn_op, rop, cxop, waybig, waysmall, firstzero, secondzero, complexwrap, noniziwrap, numbertype) \
@@ -567,11 +575,11 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
 
 #define GEN_BIN_INT_OP(name, scheme_name, op, bigop) \
 static Scheme_Object *name (const Scheme_Object *n1, const Scheme_Object *n2); \
-static Scheme_Object *name ## __wrong_type(const Scheme_Object *v) \
+static Scheme_Object *name ## __wrong_contract(const Scheme_Object *v) \
 { \
   Scheme_Object *a[1]; \
   a[0] = (Scheme_Object *)v; \
-  scheme_wrong_type(scheme_name, "exact integer", -1, 0, a); \
+  scheme_wrong_contract(scheme_name, "exact-integer?", -1, 0, a); \
   return NULL; \
 } \
 static MZ_INLINE Scheme_Object * name ## __int_big(const Scheme_Object *n1, const Scheme_Object *n2) { \
@@ -601,10 +609,10 @@ name (const Scheme_Object *n1, const Scheme_Object *n2) \
     if (SCHEME_BIGNUMP(n2)) \
       return bigop(n1, n2); \
   } else { \
-    return name ## __wrong_type(n1);       \
+    return name ## __wrong_contract(n1);       \
   } \
  \
-  return name ## __wrong_type(n2); \
+  return name ## __wrong_contract(n2); \
 }
 
 #define GEN_NARY_OP(stat, name, scheme_name, bin_name, ident, TYPEP, type, single) \
@@ -616,7 +624,7 @@ name ## __slow (Scheme_Object *ret, int argc, Scheme_Object *argv[])  \
   for (i = 1 ; i<argc ; ++i ) { \
     Scheme_Object *o; \
     o = argv[i]; \
-    if (!TYPEP(o)) { scheme_wrong_type(scheme_name, type, i, argc, argv); return NULL; } \
+    if (!TYPEP(o)) { scheme_wrong_contract(scheme_name, type, i, argc, argv); return NULL; } \
     ret = bin_name (ret, o); \
   } \
   return (ret); \
@@ -627,11 +635,11 @@ name (int argc, Scheme_Object *argv[]) \
   Scheme_Object *ret;                          \
   if (!argc) return scheme_make_integer(ident); \
   ret = argv[0]; \
-  if (!TYPEP(ret)) { scheme_wrong_type(scheme_name, type, 0, argc, argv); return NULL; } \
+  if (!TYPEP(ret)) { scheme_wrong_contract(scheme_name, type, 0, argc, argv); return NULL; } \
   if (argc == 2) { \
     Scheme_Object *b; \
     b = argv[1]; \
-    if (!TYPEP(b)) { scheme_wrong_type(scheme_name, type, 1, argc, argv); return NULL; } \
+    if (!TYPEP(b)) { scheme_wrong_contract(scheme_name, type, 1, argc, argv); return NULL; } \
     return bin_name(ret, b); \
   } \
   if (argc == 1) { return single(ret); } \
@@ -646,7 +654,7 @@ name ## __slow (Scheme_Object *ret, int argc, Scheme_Object *argv[]) \
   int i; \
   for ( i=1 ; i<argc ; ++i ) { \
     if (!TYPEP(argv[i])) \
-      scheme_wrong_type(scheme_name, type, i, argc, argv); \
+      scheme_wrong_contract(scheme_name, type, i, argc, argv); \
     ret = bin_name (ret, argv[i]); \
   } \
   return ret; \
@@ -656,11 +664,11 @@ name (int argc, Scheme_Object *argv[]) \
 { \
   Scheme_Object *ret = argv[0]; \
   if (!TYPEP(ret)) \
-    scheme_wrong_type(scheme_name, type, 0, argc, argv); \
+    scheme_wrong_contract(scheme_name, type, 0, argc, argv); \
   if (argc == 1) return ret; \
   if (argc == 2) { \
     if (!TYPEP(argv[1])) \
-      scheme_wrong_type(scheme_name, type, 1, argc, argv); \
+      scheme_wrong_contract(scheme_name, type, 1, argc, argv); \
     return bin_name(ret, argv[1]); \
   } \
   return name ## __slow(ret, argc, argv); \
@@ -699,7 +707,7 @@ name (int argc, Scheme_Object *argv[]) \
    } else if (t == scheme_complex_type) \
      return complex_fun(o); \
    else { \
-     scheme_wrong_type(#scheme_name, "number", 0, argc, argv); \
+     scheme_wrong_contract(#scheme_name, "number?", 0, argc, argv); \
      return NULL; \
     } \
   } \
