@@ -1,8 +1,8 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2012 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
- 
+
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
@@ -383,7 +383,7 @@ void scheme_init_os_thread_like(void *other) XFORM_SKIP_PROC
 #endif
 }
 
-void scheme_init_os_thread() XFORM_SKIP_PROC
+void scheme_init_os_thread(void) XFORM_SKIP_PROC
 {
   scheme_init_os_thread_like(NULL);
 }
@@ -632,6 +632,17 @@ static void raise_out_of_memory(void)
   scheme_raise_out_of_memory(NULL, NULL);
 }
 
+intptr_t scheme_check_overflow(intptr_t n, intptr_t m, intptr_t a)
+{
+  intptr_t v;
+
+  v = (n * m) + a;
+  if ((v < n) || (v < m) || (v < a) || (((v - a) / n) != m))
+    scheme_signal_error("allocation size overflow");
+
+  return v;
+}
+
 void *scheme_malloc_fail_ok(void *(*f)(size_t), size_t s)
 {
   void *v;
@@ -875,39 +886,13 @@ static intptr_t get_page_size()
 }
 
 #if defined(MZ_JIT_USE_WINDOWS_VIRTUAL_ALLOC) && defined(_WIN64)
-static RUNTIME_FUNCTION rtf;
-typedef struct mzUNWIND_INFO {
-  unsigned char version:3;
-  unsigned char flags:5;
-  unsigned char prolog_size;
-  unsigned char unwind_code_count;
-  unsigned char fp:4;
-  unsigned char fp_off:4;
-  unsigned short *unwind_codes;
-} mzUNWIND_INFO;
-static mzUNWIND_INFO uwi;
-
 PRUNTIME_FUNCTION get_rewind_info(DWORD64 ControlPc, PVOID Context)
 {
   /* When Win64 wants to unwind the stack and hit hits FFI- or 
      JIT-generated code, it invokes this callback. We should return
-     information that lets the unwind continue. For now, though,
-     we give up, which means that certain attempts to report crashes
-     turn into unwind-failure aborts. */
-  rtf.BeginAddress = ControlPc;
-  rtf.EndAddress = ControlPc;
-  rtf.UnwindData = (intptr_t)&uwi;
-  uwi.version = 1;
-  uwi.flags = 0;
-  uwi.prolog_size = 0;
-  uwi.fp = 5;
-  uwi.fp_off = 0;
-
-  /* that's not right, yet, so... */
-  scheme_log_abort("Cannot provide unwind info for generated code");
-  abort();
-
-  return &rtf;
+     information that lets the unwind continue, if possible. For
+     now, though, we just cut off the unwind. */
+  return NULL;
 }
 #endif
 
@@ -1393,7 +1378,7 @@ static void do_next_finalization(void *o, void *_data)
 
   if (fns->scheme_first) {
     if (fns->scheme_first->next || fns->ext_f || fns->prim_first) {
-      /* Re-install low-level finalizer and run a scheme finalizer */
+      /* Re-install low-level finalizer and run a Racket finalizer */
       GC_register_eager_finalizer(o, fns->scheme_first->next ? 1 : 2, 
 				  do_next_finalization, _data, NULL, NULL);
     }
@@ -1668,6 +1653,13 @@ void scheme_free_key(Scheme_Object *k)
   free(k);
 }
 
+/************************************************************************/
+/*                              Misc                                   */
+/************************************************************************/
+
+void scheme_unused_object(Scheme_Object *o) { }
+void scheme_unused_intptr(intptr_t i) { }
+
 /************************************************************************/
 /*                             GC_dump                                  */
 /************************************************************************/
@@ -1898,6 +1890,13 @@ static void cons_onto_list(void *p)
 }
 #endif
 
+#if MZ_PRECISE_GC_TRACE
+static void count_struct_instance(void *p) {
+  Scheme_Structure *s = (Scheme_Structure *)p;
+  s->stype->instance_count++;
+}
+#endif
+
 #if defined(USE_TAGGED_ALLOCATION) || MZ_PRECISE_GC_TRACE
 
 # ifdef MZ_PRECISE_GC
@@ -1933,14 +1932,19 @@ static void print_tagged_value(const char *prefix,
 			       const char *suffix)
 {
   char buffer[256];
-  char *type, *sep, diffstr[30];
+  char *type, *sep, diffstr[30], hashstr[30];
   intptr_t len;
+  const char *hash_code = "";
   
   sep = "";
   
   scheme_check_print_is_obj = check_home;
 
   if (!xtagged) {
+    if (SCHEME_TYPE(v) > _scheme_compiled_values_types_) {
+      sprintf(hashstr, "{%" PRIdPTR "}", scheme_hash_key(v));
+      hash_code = hashstr;
+    }
     type = scheme_write_to_string_w_max((Scheme_Object *)v, &len, max_w);
     if (!scheme_strncmp(type, "#<thread", 8) 
 	&& ((type[8] == '>') || (type[8] == ':'))) {
@@ -2117,11 +2121,12 @@ static void print_tagged_value(const char *prefix,
   
   if (diff)
     sprintf(diffstr, "%lx", diff);
-  
+
   object_console_printf(stderr,
-			"%s%p%s%s%s%s%s", 
+			"%s%p%s%s%s%s%s%s", 
 			prefix,
 			v, 
+                        hash_code,
 			sep,
 			type,
 			diff ? "+" : "",
@@ -2145,14 +2150,16 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
 #if MZ_PRECISE_GC_TRACE
   int trace_for_tag = 0;
   int flags = 0;
-  int path_length_limit = 1000;
+  int path_length_limit = 10000;
   GC_for_each_found_proc for_each_found = NULL;
+  GC_for_each_struct_proc for_each_struct = NULL;
 #else
 # ifndef USE_TAGGED_ALLOCATION
 #  define flags 0
 #  define trace_for_tag 0
-#  define path_length_limit 1000
+#  define path_length_limit 10000
 #  define for_each_found NULL
+#  define for_each_struct NULL
 #  define GC_get_xtagged_name NULL
 #  define print_tagged_value NULL
 # endif
@@ -2218,8 +2225,6 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
   if (scheme_external_dump_arg)
     scheme_external_dump_arg(c ? p[0] : NULL);
 
-  scheme_console_printf("Begin Dump\n");
-
 #ifdef USE_TAGGED_ALLOCATION
   trace_path_type = -1;
   obj_type = -1;
@@ -2240,7 +2245,7 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
 		    && !strcmp(SCHEME_SYM_VAL(p[1]), "objects"));
 
     for (i = 0; i < maxpos; i++) {
-      void *tn = scheme_get_type_name(i);
+      void *tn = scheme_get_type_name_or_null(i);
       if (tn && !strcmp(tn, s)) {
 	if (just_objects)
 	  obj_type = i;
@@ -2433,7 +2438,7 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
 
     for (i = 0; i < maxpos; i++) {
       void *tn;
-      tn = scheme_get_type_name(i);
+      tn = scheme_get_type_name_or_null(i);
       if (tn && !strcmp(tn, s)) {
 	trace_for_tag = i;
 	flags |= GC_DUMP_SHOW_TRACE;
@@ -2444,6 +2449,13 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
     if (!strcmp("fnl", s))
       flags |= GC_DUMP_SHOW_FINALS;
 
+    if (!strcmp("struct", s)) {
+      for_each_struct = count_struct_instance;
+      trace_for_tag = scheme_struct_type_type;
+      for_each_found = cons_onto_list;
+      cons_accum_result = scheme_null;
+    }
+
     if (!strcmp("peek", s) && (c == 3)) {
       intptr_t n;
       scheme_end_atomic();
@@ -2495,7 +2507,7 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
         limit = (char *)t->jmpup_buf.stack_copy + t->jmpup_buf.stack_size;
       }
       GC_dump_variable_stack(var_stack, delta, limit, NULL,
-                             scheme_get_type_name,
+                             scheme_get_type_name_or_null,
                              GC_get_xtagged_name,
                              print_tagged_value);
     } else {
@@ -2513,21 +2525,39 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
     flags -= (flags & GC_DUMP_SHOW_TRACE);
   }
   scheme_console_printf("Begin Dump\n");
+#else
+  scheme_console_printf("Begin Dump\n");
 #endif
 
 # ifdef MZ_PRECISE_GC
   GC_dump_with_traces(flags, 
-		      scheme_get_type_name,
+		      scheme_get_type_name_or_null,
 		      GC_get_xtagged_name,
 		      for_each_found,
-		      trace_for_tag,
+		      trace_for_tag, trace_for_tag,
 		      print_tagged_value,
-		      path_length_limit);
+		      path_length_limit,
+                      for_each_struct);
 # else
   GC_dump();
 # endif
 #endif
 
+#if MZ_PRECISE_GC_TRACE
+  if (for_each_struct) {
+    scheme_console_printf("Begin Struct\n");
+    while (SCHEME_PAIRP(cons_accum_result)) {
+      Scheme_Struct_Type *stype = (Scheme_Struct_Type *)SCHEME_CAR(cons_accum_result);
+      if (stype->instance_count) {
+        scheme_console_printf(" %32.32s: %10" PRIdPTR "\n", SCHEME_SYM_VAL(stype->name), stype->instance_count);
+        stype->instance_count = 0;
+      }
+      cons_accum_result = SCHEME_CDR(cons_accum_result);
+    }
+    scheme_console_printf("End Struct\n");
+  }
+#endif
+
   if (scheme_external_dump_info)
     scheme_external_dump_info();
 
@@ -2599,6 +2629,7 @@ Scheme_Object *scheme_dump_gc_stats(int c, Scheme_Object *p[])
   scheme_console_printf("Begin Help\n");
   scheme_console_printf(" (dump-memory-stats sym) - prints paths to instances of type named by sym.\n");
   scheme_console_printf("   Example: (dump-memory-stats '<pair>)\n");
+  scheme_console_printf(" (dump-memory-stats 'struct) - show counts for specific structure types.\n");
   scheme_console_printf(" (dump-memory-stats 'fnl) - prints not-yet-finalized objects.\n");
   scheme_console_printf(" (dump-memory-stats num) - prints paths to objects with tag num.\n");
   scheme_console_printf(" (dump-memory-stats -num) - prints paths to objects of size num.\n");
