@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2012 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
   All rights reserved.
 
@@ -55,6 +55,7 @@ MZ_EXTERN Scheme_Jumpup_Buf_Holder *scheme_new_jmpupbuf_holder(void);
 /*========================================================================*/
 
 MZ_EXTERN Scheme_Config *scheme_current_config(void);
+MZ_EXTERN Scheme_Config *scheme_minimal_config(void);
 MZ_EXTERN Scheme_Config *scheme_extend_config(Scheme_Config *c, int pos, Scheme_Object *init_val);
 MZ_EXTERN void scheme_install_config(Scheme_Config *);
 
@@ -66,6 +67,7 @@ MZ_EXTERN void scheme_set_thread_param(Scheme_Config *c, Scheme_Thread_Cell_Tabl
 
 MZ_EXTERN Scheme_Env *scheme_get_env(Scheme_Config *config);
 
+MZ_EXTERN Scheme_Thread_Cell_Table *scheme_empty_cell_table();
 MZ_EXTERN Scheme_Thread_Cell_Table *scheme_inherit_cells(Scheme_Thread_Cell_Table *cells);
 
 MZ_EXTERN Scheme_Object *scheme_current_break_cell();
@@ -86,6 +88,7 @@ MZ_EXTERN volatile int *scheme_fuel_counter_ptr;
 
 MZ_EXTERN Scheme_Thread *scheme_get_current_thread();
 
+MZ_EXTERN int scheme_is_atomic(void);
 MZ_EXTERN void scheme_start_atomic(void);
 MZ_EXTERN void scheme_end_atomic(void);
 MZ_EXTERN void scheme_end_atomic_no_swap(void);
@@ -105,8 +108,10 @@ MZ_EXTERN Scheme_Object *scheme_thread_w_details(Scheme_Object *thunk,
 						 int suspend_to_kill);
 MZ_EXTERN void scheme_kill_thread(Scheme_Thread *p);
 MZ_EXTERN void scheme_break_thread(Scheme_Thread *p);
+MZ_EXTERN void scheme_break_kind_thread(Scheme_Thread *p, int kind);
 MZ_EXTERN void scheme_break_main_thread();
 MZ_EXTERN void scheme_break_main_thread_at(void *);
+MZ_EXTERN void scheme_break_kind_main_thread_at(void *, int kind);
 MZ_EXTERN void *scheme_get_main_thread_break_handle();
 MZ_EXTERN void scheme_set_break_main_target(Scheme_Thread *p);
 
@@ -151,6 +156,9 @@ MZ_EXTERN Scheme_Custodian *scheme_make_custodian(Scheme_Custodian *);
 MZ_EXTERN Scheme_Custodian_Reference *scheme_add_managed(Scheme_Custodian *m, Scheme_Object *o,
 							 Scheme_Close_Custodian_Client *f, void *data,
 							 int strong);
+MZ_EXTERN Scheme_Custodian_Reference *scheme_add_managed_close_on_exit(Scheme_Custodian *m, Scheme_Object *o, 
+                                                                       Scheme_Close_Custodian_Client *f, void *data);
+
 MZ_EXTERN void scheme_custodian_check_available(Scheme_Custodian *m, const char *who, const char *what);
 MZ_EXTERN int scheme_custodian_is_available(Scheme_Custodian *m);
 MZ_EXTERN void scheme_remove_managed(Scheme_Custodian_Reference *m, Scheme_Object *o);
@@ -229,10 +237,17 @@ MZ_EXTERN void scheme_case_lambda_wrong_count(const char *name, int argc,
 MZ_EXTERN void scheme_wrong_type(const char *name, const char *expected,
 				 int which, int argc,
 				 Scheme_Object **argv);
+MZ_EXTERN void scheme_wrong_contract(const char *name, const char *expected,
+                                     int which, int argc,
+                                     Scheme_Object **argv);
 MZ_EXTERN void scheme_wrong_field_type(Scheme_Object *c_name,
 				       const char *expected,
 				       Scheme_Object *o);
+MZ_EXTERN void scheme_wrong_field_contract(Scheme_Object *c_name,
+                                           const char *expected,
+                                           Scheme_Object *o);
 MZ_EXTERN void scheme_arg_mismatch(const char *name, const char *msg, Scheme_Object *o);
+MZ_EXTERN void scheme_contract_error(const char *name, const char *msg, ...);
 MZ_EXTERN void scheme_wrong_return_arity(const char *where,
 					 int expected, int got,
 					 Scheme_Object **argv,
@@ -252,6 +267,7 @@ MZ_EXTERN Scheme_Object *scheme_dynamic_wind(void (*pre)(void *),
 MZ_EXTERN Scheme_Type scheme_make_type(const char *name);
 
 MZ_EXTERN char *scheme_get_type_name(Scheme_Type type);
+MZ_EXTERN char *scheme_get_type_name_or_null(Scheme_Type type);
 
 /*========================================================================*/
 /*                              constants                                 */
@@ -615,13 +631,15 @@ XFORM_NONGCING MZ_EXTERN int scheme_get_unsigned_int_val(Scheme_Object *o, uintp
 XFORM_NONGCING MZ_EXTERN int scheme_get_long_long_val(Scheme_Object *o, mzlonglong *v);
 XFORM_NONGCING MZ_EXTERN int scheme_get_unsigned_long_long_val(Scheme_Object *o, umzlonglong *v);
 
-XFORM_NONGCING MZ_EXTERN double scheme_real_to_double(Scheme_Object *r);
+MZ_EXTERN double scheme_real_to_double(Scheme_Object *r);
 
 MZ_EXTERN Scheme_Object *scheme_make_cptr(void *cptr, Scheme_Object *typetag);
 MZ_EXTERN Scheme_Object *scheme_make_offset_cptr(void *cptr, intptr_t offset, Scheme_Object *typetag);
 MZ_EXTERN Scheme_Object *scheme_make_external_cptr(void *cptr, Scheme_Object *typetag);
 MZ_EXTERN Scheme_Object *scheme_make_offset_external_cptr(void *cptr, intptr_t offset, Scheme_Object *typetag);
 
+MZ_EXTERN int scheme_is_cpointer(Scheme_Object *cp);
+
 MZ_EXTERN const char *scheme_get_proc_name(Scheme_Object *p, int *len, int for_error);
 
 /*========================================================================*/
@@ -673,10 +691,10 @@ MZ_EXTERN Scheme_Object *scheme_make_bignum(intptr_t v);
 MZ_EXTERN Scheme_Object *scheme_make_bignum_from_unsigned(uintptr_t v);
 MZ_EXTERN Scheme_Object *scheme_make_bignum_from_long_long(mzlonglong v);
 MZ_EXTERN Scheme_Object *scheme_make_bignum_from_unsigned_long_long(umzlonglong v);
-MZ_EXTERN double scheme_bignum_to_double(const Scheme_Object *n);
+XFORM_NONGCING MZ_EXTERN double scheme_bignum_to_double(const Scheme_Object *n);
 MZ_EXTERN Scheme_Object *scheme_bignum_from_double(double d);
 #ifdef MZ_USE_SINGLE_FLOATS
-MZ_EXTERN float scheme_bignum_to_float(const Scheme_Object *n);
+XFORM_NONGCING MZ_EXTERN float scheme_bignum_to_float(const Scheme_Object *n);
 MZ_EXTERN Scheme_Object *scheme_bignum_from_float(float d);
 #else
 # define scheme_bignum_to_float scheme_bignum_to_double
@@ -805,6 +823,7 @@ MZ_EXTERN intptr_t scheme_get_char_string(const char *who,
 MZ_EXTERN intptr_t scheme_get_bytes(Scheme_Object *port, intptr_t size, char *buffer, int offset);
 MZ_EXTERN Scheme_Object *scheme_get_ready_special(Scheme_Object *port, Scheme_Object *stxsrc, int peek);
 MZ_EXTERN intptr_t scheme_tell(Scheme_Object *port);
+MZ_EXTERN intptr_t scheme_tell_can_redirect(Scheme_Object *port, int not_via_loc);
 MZ_EXTERN intptr_t scheme_output_tell(Scheme_Object *port);
 MZ_EXTERN intptr_t scheme_tell_line(Scheme_Object *port);
 MZ_EXTERN intptr_t scheme_tell_column(Scheme_Object *port);
@@ -906,6 +925,7 @@ MZ_EXTERN Scheme_Object *scheme_split_path(const char *path, int len, Scheme_Obj
 MZ_EXTERN Scheme_Object *scheme_build_path(int argc, Scheme_Object **argv);
 MZ_EXTERN Scheme_Object *scheme_path_to_directory_path(Scheme_Object *p);
 MZ_EXTERN Scheme_Object *scheme_path_to_complete_path(Scheme_Object *path, Scheme_Object *relto_path);
+MZ_EXTERN Scheme_Object *scheme_simplify_path(int argc, Scheme_Object *argv[]);
 
 MZ_EXTERN Scheme_Object *scheme_make_path(const char *chars);
 MZ_EXTERN Scheme_Object *scheme_make_sized_path(char *chars, intptr_t len, int copy);
@@ -949,6 +969,8 @@ MZ_EXTERN int scheme_get_port_socket(Scheme_Object *p, intptr_t *_s);
 MZ_EXTERN void scheme_socket_to_ports(intptr_t s, const char *name, int takeover,
                                       Scheme_Object **_inp, Scheme_Object **_outp);
 
+MZ_EXTERN Scheme_Object *scheme_fd_to_semaphore(intptr_t fd, int mode, int is_socket);
+
 MZ_EXTERN void scheme_set_type_printer(Scheme_Type stype, Scheme_Type_Printer printer);
 MZ_EXTERN void scheme_print_bytes(Scheme_Print_Params *pp, const char *str, int offset, int len);
 MZ_EXTERN void scheme_print_utf8(Scheme_Print_Params *pp, const char *str, int offset, int len);
@@ -999,6 +1021,8 @@ MZ_EXTERN int scheme_is_module_path(Scheme_Object *);
 
 MZ_EXTERN Scheme_Object *scheme_datum_to_kernel_stx(Scheme_Object *e);
 
+MZ_EXTERN int scheme_module_is_declared(Scheme_Object *name, int try_load);
+
 /*========================================================================*/
 /*                                symbols                                 */
 /*========================================================================*/
@@ -1131,10 +1155,10 @@ MZ_EXTERN Scheme_Object *scheme_load_extension(const char *filename, Scheme_Env
 MZ_EXTERN void scheme_register_extension_global(void *ptr, intptr_t size);
 
 MZ_EXTERN intptr_t scheme_get_seconds(void);
-MZ_EXTERN intptr_t scheme_get_milliseconds(void);
-MZ_EXTERN double scheme_get_inexact_milliseconds(void);
-MZ_EXTERN intptr_t scheme_get_process_milliseconds(void);
-MZ_EXTERN intptr_t scheme_get_thread_milliseconds(Scheme_Object *thrd);
+XFORM_NONGCING MZ_EXTERN intptr_t scheme_get_milliseconds(void);
+XFORM_NONGCING MZ_EXTERN double scheme_get_inexact_milliseconds(void);
+XFORM_NONGCING MZ_EXTERN intptr_t scheme_get_process_milliseconds(void);
+XFORM_NONGCING MZ_EXTERN intptr_t scheme_get_thread_milliseconds(Scheme_Object *thrd);
 
 MZ_EXTERN char *scheme_banner(void);
 MZ_EXTERN char *scheme_version(void);
@@ -1146,13 +1170,15 @@ MZ_EXTERN int scheme_check_proc_arity2(const char *where, int a,
 				       int false_ok);
 
 MZ_EXTERN char *scheme_make_provided_string(Scheme_Object *o, int count, intptr_t *len);
-MZ_EXTERN char *scheme_make_args_string(char *s, int which, int argc, Scheme_Object **argv, intptr_t *len);
+MZ_EXTERN char *scheme_make_args_string(const char *s, int which, int argc, Scheme_Object **argv, intptr_t *len);
+MZ_EXTERN char *scheme_make_arg_lines_string(const char *s, int which, int argc, Scheme_Object **argv, intptr_t *len);
 
 MZ_EXTERN const char *scheme_system_library_subpath();
 
 MZ_EXTERN void scheme_signal_received(void);
 MZ_EXTERN void scheme_signal_received_at(void *);
 MZ_EXTERN void *scheme_get_signal_handle();
+MZ_EXTERN void scheme_wait_until_signal_received(void);
 
 MZ_EXTERN intptr_t scheme_char_strlen(const mzchar *s);
 
