@@ -55,6 +55,7 @@ volatile int *scheme_fuel_counter_ptr;
 # endif
 #endif
 Scheme_Thread *(*scheme_get_current_thread)();
+int (*scheme_is_atomic)(void);
 void (*scheme_start_atomic)(void);
 void (*scheme_end_atomic)(void);
 void (*scheme_end_atomic_no_swap)(void);
@@ -72,8 +73,10 @@ Scheme_Object *(*scheme_thread_w_details)(Scheme_Object *thunk,
 						 int suspend_to_kill);
 void (*scheme_kill_thread)(Scheme_Thread *p);
 void (*scheme_break_thread)(Scheme_Thread *p);
+void (*scheme_break_kind_thread)(Scheme_Thread *p, int kind);
 void (*scheme_break_main_thread)();
 void (*scheme_break_main_thread_at)(void *);
+void (*scheme_break_kind_main_thread_at)(void *, int kind);
 void *(*scheme_get_main_thread_break_handle)();
 void (*scheme_set_break_main_target)(Scheme_Thread *p);
 void (*scheme_thread_block)(float sleep_time);
@@ -107,6 +110,8 @@ Scheme_Custodian *(*scheme_make_custodian)(Scheme_Custodian *);
 Scheme_Custodian_Reference *(*scheme_add_managed)(Scheme_Custodian *m, Scheme_Object *o,
 							 Scheme_Close_Custodian_Client *f, void *data,
 							 int strong);
+Scheme_Custodian_Reference *(*scheme_add_managed_close_on_exit)(Scheme_Custodian *m, Scheme_Object *o, 
+                                                                       Scheme_Close_Custodian_Client *f, void *data);
 void (*scheme_custodian_check_available)(Scheme_Custodian *m, const char *who, const char *what);
 int (*scheme_custodian_is_available)(Scheme_Custodian *m);
 void (*scheme_remove_managed)(Scheme_Custodian_Reference *m, Scheme_Object *o);
@@ -170,10 +175,17 @@ void (*scheme_case_lambda_wrong_count)(const char *name, int argc,
 void (*scheme_wrong_type)(const char *name, const char *expected,
 				 int which, int argc,
 				 Scheme_Object **argv);
+void (*scheme_wrong_contract)(const char *name, const char *expected,
+                                     int which, int argc,
+                                     Scheme_Object **argv);
 void (*scheme_wrong_field_type)(Scheme_Object *c_name,
 				       const char *expected,
 				       Scheme_Object *o);
+void (*scheme_wrong_field_contract)(Scheme_Object *c_name,
+                                           const char *expected,
+                                           Scheme_Object *o);
 void (*scheme_arg_mismatch)(const char *name, const char *msg, Scheme_Object *o);
+void (*scheme_contract_error)(const char *name, const char *msg, ...);
 void (*scheme_wrong_return_arity)(const char *where,
 					 int expected, int got,
 					 Scheme_Object **argv,
@@ -189,6 +201,7 @@ Scheme_Object *(*scheme_dynamic_wind)(void (*pre)(void *),
 /*========================================================================*/
 Scheme_Type (*scheme_make_type)(const char *name);
 char *(*scheme_get_type_name)(Scheme_Type type);
+char *(*scheme_get_type_name_or_null)(Scheme_Type type);
 /*========================================================================*/
 /*                              constants                                 */
 /*========================================================================*/
@@ -499,6 +512,7 @@ Scheme_Object *(*scheme_make_cptr)(void *cptr, Scheme_Object *typetag);
 Scheme_Object *(*scheme_make_offset_cptr)(void *cptr, intptr_t offset, Scheme_Object *typetag);
 Scheme_Object *(*scheme_make_external_cptr)(void *cptr, Scheme_Object *typetag);
 Scheme_Object *(*scheme_make_offset_external_cptr)(void *cptr, intptr_t offset, Scheme_Object *typetag);
+int (*scheme_is_cpointer)(Scheme_Object *cp);
 const char *(*scheme_get_proc_name)(Scheme_Object *p, int *len, int for_error);
 /*========================================================================*/
 /*                               strings                                  */
@@ -662,6 +676,7 @@ intptr_t (*scheme_get_char_string)(const char *who,
 intptr_t (*scheme_get_bytes)(Scheme_Object *port, intptr_t size, char *buffer, int offset);
 Scheme_Object *(*scheme_get_ready_special)(Scheme_Object *port, Scheme_Object *stxsrc, int peek);
 intptr_t (*scheme_tell)(Scheme_Object *port);
+intptr_t (*scheme_tell_can_redirect)(Scheme_Object *port, int not_via_loc);
 intptr_t (*scheme_output_tell)(Scheme_Object *port);
 intptr_t (*scheme_tell_line)(Scheme_Object *port);
 intptr_t (*scheme_tell_column)(Scheme_Object *port);
@@ -747,6 +762,7 @@ Scheme_Object *(*scheme_split_path)(const char *path, int len, Scheme_Object **b
 Scheme_Object *(*scheme_build_path)(int argc, Scheme_Object **argv);
 Scheme_Object *(*scheme_path_to_directory_path)(Scheme_Object *p);
 Scheme_Object *(*scheme_path_to_complete_path)(Scheme_Object *path, Scheme_Object *relto_path);
+Scheme_Object *(*scheme_simplify_path)(int argc, Scheme_Object *argv[]);
 Scheme_Object *(*scheme_make_path)(const char *chars);
 Scheme_Object *(*scheme_make_sized_path)(char *chars, intptr_t len, int copy);
 Scheme_Object *(*scheme_make_sized_offset_path)(char *chars, intptr_t d, intptr_t len, int copy);
@@ -816,6 +832,7 @@ Scheme_Object *(*scheme_dynamic_require)(int argc, Scheme_Object *argv[]);
 Scheme_Object *(*scheme_namespace_require)(Scheme_Object *);
 int (*scheme_is_module_path)(Scheme_Object *);
 Scheme_Object *(*scheme_datum_to_kernel_stx)(Scheme_Object *e);
+int (*scheme_module_is_declared)(Scheme_Object *name, int try_load);
 /*========================================================================*/
 /*                                symbols                                 */
 /*========================================================================*/
@@ -936,11 +953,13 @@ int (*scheme_check_proc_arity2)(const char *where, int a,
 				       int which, int argc, Scheme_Object **argv,
 				       int false_ok);
 char *(*scheme_make_provided_string)(Scheme_Object *o, int count, intptr_t *len);
-char *(*scheme_make_args_string)(char *s, int which, int argc, Scheme_Object **argv, intptr_t *len);
+char *(*scheme_make_args_string)(const char *s, int which, int argc, Scheme_Object **argv, intptr_t *len);
+char *(*scheme_make_arg_lines_string)(const char *s, int which, int argc, Scheme_Object **argv, intptr_t *len);
 const char *(*scheme_system_library_subpath)();
 void (*scheme_signal_received)(void);
 void (*scheme_signal_received_at)(void *);
 void *(*scheme_get_signal_handle)();
+void (*scheme_wait_until_signal_received)(void);
 intptr_t (*scheme_char_strlen)(const mzchar *s);
 Scheme_Object *(*scheme_stx_extract_marks)(Scheme_Object *stx);
 int (*scheme_get_place_id)(void);
