@@ -30,6 +30,7 @@
 # endif
 #endif
 #define scheme_get_current_thread (scheme_extension_table->scheme_get_current_thread)
+#define scheme_is_atomic (scheme_extension_table->scheme_is_atomic)
 #define scheme_start_atomic (scheme_extension_table->scheme_start_atomic)
 #define scheme_end_atomic (scheme_extension_table->scheme_end_atomic)
 #define scheme_end_atomic_no_swap (scheme_extension_table->scheme_end_atomic_no_swap)
@@ -42,8 +43,10 @@
 #define scheme_thread_w_details (scheme_extension_table->scheme_thread_w_details)
 #define scheme_kill_thread (scheme_extension_table->scheme_kill_thread)
 #define scheme_break_thread (scheme_extension_table->scheme_break_thread)
+#define scheme_break_kind_thread (scheme_extension_table->scheme_break_kind_thread)
 #define scheme_break_main_thread (scheme_extension_table->scheme_break_main_thread)
 #define scheme_break_main_thread_at (scheme_extension_table->scheme_break_main_thread_at)
+#define scheme_break_kind_main_thread_at (scheme_extension_table->scheme_break_kind_main_thread_at)
 #define scheme_get_main_thread_break_handle (scheme_extension_table->scheme_get_main_thread_break_handle)
 #define scheme_set_break_main_target (scheme_extension_table->scheme_set_break_main_target)
 #define scheme_thread_block (scheme_extension_table->scheme_thread_block)
@@ -71,6 +74,7 @@
 #define scheme_tls_get (scheme_extension_table->scheme_tls_get)
 #define scheme_make_custodian (scheme_extension_table->scheme_make_custodian)
 #define scheme_add_managed (scheme_extension_table->scheme_add_managed)
+#define scheme_add_managed_close_on_exit (scheme_extension_table->scheme_add_managed_close_on_exit)
 #define scheme_custodian_check_available (scheme_extension_table->scheme_custodian_check_available)
 #define scheme_custodian_is_available (scheme_extension_table->scheme_custodian_is_available)
 #define scheme_remove_managed (scheme_extension_table->scheme_remove_managed)
@@ -116,13 +120,17 @@
 #define scheme_wrong_count_m (scheme_extension_table->scheme_wrong_count_m)
 #define scheme_case_lambda_wrong_count (scheme_extension_table->scheme_case_lambda_wrong_count)
 #define scheme_wrong_type (scheme_extension_table->scheme_wrong_type)
+#define scheme_wrong_contract (scheme_extension_table->scheme_wrong_contract)
 #define scheme_wrong_field_type (scheme_extension_table->scheme_wrong_field_type)
+#define scheme_wrong_field_contract (scheme_extension_table->scheme_wrong_field_contract)
 #define scheme_arg_mismatch (scheme_extension_table->scheme_arg_mismatch)
+#define scheme_contract_error (scheme_extension_table->scheme_contract_error)
 #define scheme_wrong_return_arity (scheme_extension_table->scheme_wrong_return_arity)
 #define scheme_unbound_global (scheme_extension_table->scheme_unbound_global)
 #define scheme_dynamic_wind (scheme_extension_table->scheme_dynamic_wind)
 #define scheme_make_type (scheme_extension_table->scheme_make_type)
 #define scheme_get_type_name (scheme_extension_table->scheme_get_type_name)
+#define scheme_get_type_name_or_null (scheme_extension_table->scheme_get_type_name_or_null)
 #define scheme_eof (scheme_extension_table->scheme_eof)
 #define scheme_make_eof (scheme_extension_table->scheme_make_eof)
 #define scheme_null (scheme_extension_table->scheme_null)
@@ -369,6 +377,7 @@
 #define scheme_make_offset_cptr (scheme_extension_table->scheme_make_offset_cptr)
 #define scheme_make_external_cptr (scheme_extension_table->scheme_make_external_cptr)
 #define scheme_make_offset_external_cptr (scheme_extension_table->scheme_make_offset_external_cptr)
+#define scheme_is_cpointer (scheme_extension_table->scheme_is_cpointer)
 #define scheme_get_proc_name (scheme_extension_table->scheme_get_proc_name)
 #define scheme_utf8_decode (scheme_extension_table->scheme_utf8_decode)
 #define scheme_utf8_decode_as_prefix (scheme_extension_table->scheme_utf8_decode_as_prefix)
@@ -473,6 +482,7 @@
 #define scheme_get_bytes (scheme_extension_table->scheme_get_bytes)
 #define scheme_get_ready_special (scheme_extension_table->scheme_get_ready_special)
 #define scheme_tell (scheme_extension_table->scheme_tell)
+#define scheme_tell_can_redirect (scheme_extension_table->scheme_tell_can_redirect)
 #define scheme_output_tell (scheme_extension_table->scheme_output_tell)
 #define scheme_tell_line (scheme_extension_table->scheme_tell_line)
 #define scheme_tell_column (scheme_extension_table->scheme_tell_column)
@@ -531,6 +541,7 @@
 #define scheme_build_path (scheme_extension_table->scheme_build_path)
 #define scheme_path_to_directory_path (scheme_extension_table->scheme_path_to_directory_path)
 #define scheme_path_to_complete_path (scheme_extension_table->scheme_path_to_complete_path)
+#define scheme_simplify_path (scheme_extension_table->scheme_simplify_path)
 #define scheme_make_path (scheme_extension_table->scheme_make_path)
 #define scheme_make_sized_path (scheme_extension_table->scheme_make_sized_path)
 #define scheme_make_sized_offset_path (scheme_extension_table->scheme_make_sized_offset_path)
@@ -589,6 +600,7 @@
 #define scheme_namespace_require (scheme_extension_table->scheme_namespace_require)
 #define scheme_is_module_path (scheme_extension_table->scheme_is_module_path)
 #define scheme_datum_to_kernel_stx (scheme_extension_table->scheme_datum_to_kernel_stx)
+#define scheme_module_is_declared (scheme_extension_table->scheme_module_is_declared)
 #define scheme_intern_symbol (scheme_extension_table->scheme_intern_symbol)
 #define scheme_intern_exact_symbol (scheme_extension_table->scheme_intern_exact_symbol)
 #define scheme_intern_exact_char_symbol (scheme_extension_table->scheme_intern_exact_char_symbol)
@@ -670,10 +682,12 @@
 #define scheme_check_proc_arity2 (scheme_extension_table->scheme_check_proc_arity2)
 #define scheme_make_provided_string (scheme_extension_table->scheme_make_provided_string)
 #define scheme_make_args_string (scheme_extension_table->scheme_make_args_string)
+#define scheme_make_arg_lines_string (scheme_extension_table->scheme_make_arg_lines_string)
 #define scheme_system_library_subpath (scheme_extension_table->scheme_system_library_subpath)
 #define scheme_signal_received (scheme_extension_table->scheme_signal_received)
 #define scheme_signal_received_at (scheme_extension_table->scheme_signal_received_at)
 #define scheme_get_signal_handle (scheme_extension_table->scheme_get_signal_handle)
+#define scheme_wait_until_signal_received (scheme_extension_table->scheme_wait_until_signal_received)
 #define scheme_char_strlen (scheme_extension_table->scheme_char_strlen)
 #define scheme_stx_extract_marks (scheme_extension_table->scheme_stx_extract_marks)
 #define scheme_get_place_id (scheme_extension_table->scheme_get_place_id)
