@@ -1,8 +1,8 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2012 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
- 
+
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
@@ -33,10 +33,12 @@ static Scheme_Object *block_sema_p(int n, Scheme_Object **p);
 static Scheme_Object *block_sema(int n, Scheme_Object **p);
 static Scheme_Object *block_sema_breakable(int n, Scheme_Object **p);
 static Scheme_Object *make_sema_repost(int n, Scheme_Object **p);
+static Scheme_Object *is_sema_repost(int n, Scheme_Object **p);
 
 static Scheme_Object *make_channel(int n, Scheme_Object **p);
 static Scheme_Object *make_channel_put(int n, Scheme_Object **p);
 static Scheme_Object *channel_p(int n, Scheme_Object **p);
+static Scheme_Object *channel_put_p(int n, Scheme_Object **p);
 
 static Scheme_Object *thread_send(int n, Scheme_Object **p);
 static Scheme_Object *thread_receive(int n, Scheme_Object **p);
@@ -126,6 +128,11 @@ void scheme_init_sema(Scheme_Env *env)
 						      "semaphore-peek-evt", 
 						      1, 1), 
 			     env);
+  scheme_add_global_constant("semaphore-peek-evt?", 
+			     scheme_make_folding_prim(is_sema_repost,
+						      "semaphore-peek-evt?", 
+						      1, 1, 1), 
+			     env);
 
   scheme_add_global_constant("make-channel", 
 			     scheme_make_prim_w_arity(make_channel,
@@ -142,6 +149,11 @@ void scheme_init_sema(Scheme_Env *env)
 						      "channel?",
 						      1, 1, 1), 
 			     env);  
+  scheme_add_global_constant("channel-put-evt?", 
+			     scheme_make_folding_prim(channel_put_p,
+                                                      "channel-put-evt?",
+                                                      1, 1, 1), 
+			     env);
 
   scheme_add_global_constant("thread-send", 
 			     scheme_make_prim_w_arity(thread_send,
@@ -230,7 +242,7 @@ intptr_t scheme_get_semaphore_init(const char *who, int n, Scheme_Object **p)
   if (n) {
     if (!SCHEME_INTP(p[0])) {
       if (!SCHEME_BIGNUMP(p[0]) || !SCHEME_BIGPOS(p[0]))
-	scheme_wrong_type(who, "non-negative exact integer", 0, n, p);
+	scheme_wrong_contract(who, "exact-nonnegative-integer?", 0, n, p);
     }
 
     if (!scheme_get_int_val(p[0], &v)) {
@@ -239,7 +251,7 @@ intptr_t scheme_get_semaphore_init(const char *who, int n, Scheme_Object **p)
                        who,
 		       scheme_make_provided_string(p[0], 0, NULL));
     } else if (v < 0)
-      scheme_wrong_type(who, "non-negative exact integer", 0, n, p);
+      scheme_wrong_contract(who, "exact-nonnegative-integer?", 0, n, p);
   } else
     v = 0;
   
@@ -256,7 +268,7 @@ static Scheme_Object *make_sema(int n, Scheme_Object **p)
 static Scheme_Object *make_sema_repost(int n, Scheme_Object **p)
 {
   if (!SCHEME_SEMAP(p[0]))
-    scheme_wrong_type("semaphore-peek-evt", "semaphore", 0, n, p);
+    scheme_wrong_contract("semaphore-peek-evt", "semaphore?", 0, n, p);
  
   return scheme_make_sema_repost(p[0]);
 }
@@ -272,6 +284,13 @@ Scheme_Object *scheme_make_sema_repost(Scheme_Object *sema)
   return o;
 }
 
+static Scheme_Object *is_sema_repost(int n, Scheme_Object **p)
+{
+  return (SAME_TYPE(SCHEME_TYPE(p[0]), scheme_semaphore_repost_type)
+          ? scheme_true
+          : scheme_false);
+}
+
 static Scheme_Object *semap(int n, Scheme_Object **p)
 {
   return SCHEME_SEMAP(p[0]) ? scheme_true : scheme_false;
@@ -355,7 +374,7 @@ void scheme_post_sema_all(Scheme_Object *o)
 static Scheme_Object *hit_sema(int n, Scheme_Object **p)
 {
   if (!SCHEME_SEMAP(p[0]))
-    scheme_wrong_type("semaphore-post", "semaphore", 0, n, p);
+    scheme_wrong_contract("semaphore-post", "semaphore?", 0, n, p);
 
   scheme_post_sema(p[0]);
 
@@ -630,6 +649,22 @@ int scheme_wait_semas_chs(int n, Scheme_Object **o, int just_try, Syncing *synci
   } else {
     int start_pos;
 
+#if 0
+    /* Use the "immutable" flag bit on a semaphore to check for
+       inconsistent use in atomic and non-atomic modes, which
+       can lead to an attempt to suspend in atomic mode. */
+    if ((n == 1) && SCHEME_SEMAP(o[0])) {
+      if (!do_atomic) {
+        SCHEME_SET_IMMUTABLE(o[0]);
+      } else if (SCHEME_IMMUTABLEP(o[0])) {
+        if (!on_atomic_timeout
+            || (do_atomic > atomic_timeout_atomic_level)) {
+          scheme_signal_error("using a seaphore in both atomic and non-atomic mode");
+        }
+      }
+    }
+#endif
+
     if (n > 1) {
       if (syncing)
 	start_pos = syncing->start_pos;
@@ -673,7 +708,12 @@ int scheme_wait_semas_chs(int n, Scheme_Object **o, int just_try, Syncing *synci
           break;
         } else {
           /* there may have been some action on one of the waitables;
-             try again */
+             try again, if no result, yet */
+          if (syncing && syncing->result) {
+            i = syncing->result - 1;
+            ii = 0;
+            break;
+          }
         }
       } else
         break;
@@ -904,7 +944,7 @@ int scheme_wait_sema(Scheme_Object *o, int just_try)
 static Scheme_Object *block_sema_p(int n, Scheme_Object **p)
 {
   if (!SCHEME_SEMAP(p[0]))
-    scheme_wrong_type("semaphore-try-wait?", "sema", 0, n, p);
+    scheme_wrong_contract("semaphore-try-wait?", "semaphore?", 0, n, p);
 
   return scheme_wait_sema(p[0], 1) ? scheme_true : scheme_false;
 }
@@ -912,7 +952,7 @@ static Scheme_Object *block_sema_p(int n, Scheme_Object **p)
 static Scheme_Object *block_sema(int n, Scheme_Object **p)
 {
   if (!SCHEME_SEMAP(p[0]))
-    scheme_wrong_type("semaphore-wait", "sema", 0, n, p);
+    scheme_wrong_contract("semaphore-wait", "semaphore?", 0, n, p);
 
   scheme_wait_sema(p[0], 0);
 
@@ -926,7 +966,7 @@ static Scheme_Object *block_sema(int n, Scheme_Object **p)
 static Scheme_Object *block_sema_breakable(int n, Scheme_Object **p)
 {
   if (!SCHEME_SEMAP(p[0]))
-    scheme_wrong_type("semaphore-wait/enable-break", "sema", 0, n, p);
+    scheme_wrong_contract("semaphore-wait/enable-break", "semaphore?", 0, n, p);
 
   scheme_wait_sema(p[0], -1);
 
@@ -1005,7 +1045,7 @@ int scheme_try_channel_put(Scheme_Object *ch, Scheme_Object *v)
 static Scheme_Object *make_channel_put(int argc, Scheme_Object **argv)
 {
   if (!SCHEME_CHANNELP(argv[0]))
-    scheme_wrong_type("channel-put-evt", "channel", 0, argc, argv);
+    scheme_wrong_contract("channel-put-evt", "channel?", 0, argc, argv);
 
   return scheme_make_channel_put_evt(argv[0], argv[1]);
 }
@@ -1017,6 +1057,13 @@ static Scheme_Object *channel_p(int n, Scheme_Object **p)
 	  : scheme_false);
 }
 
+static Scheme_Object *channel_put_p(int n, Scheme_Object **p)
+{
+  return (SAME_TYPE(SCHEME_TYPE(p[0]), scheme_channel_put_type)
+	  ? scheme_true
+	  : scheme_false);
+}
+
 static int channel_get_ready(Scheme_Object *ch, Scheme_Schedule_Info *sinfo)
 {
   Scheme_Object *result;
@@ -1164,7 +1211,7 @@ static Scheme_Object *thread_send(int argc, Scheme_Object **argv)
         scheme_raise_exn(MZEXN_FAIL_CONTRACT, "thread-send: target thread is not running");
     }
   } else 
-    scheme_wrong_type("thread-send", "thread", 0, argc, argv);
+    scheme_wrong_contract("thread-send", "thread?", 0, argc, argv);
 
   return NULL;
 }
@@ -1229,7 +1276,7 @@ static Scheme_Object *thread_rewind_receive(int argc, Scheme_Object **argv)
     mbox_push_front(scheme_current_thread, argv[0]);
     return scheme_void;
   } else
-    scheme_wrong_type("thread-rewind", "list", 0, argc, argv);
+    scheme_wrong_contract("thread-rewind-receive", "list?", 0, argc, argv);
 
   return NULL;
 }
@@ -1244,7 +1291,7 @@ static Scheme_Object *make_alarm(int argc, Scheme_Object **argv)
   double sleep_end;
 
   if (!SCHEME_REALP(argv[0])) {
-    scheme_wrong_type("alarm-evt", "real number", 0, argc, argv);
+    scheme_wrong_contract("alarm-evt", "real?", 0, argc, argv);
   }
 
   sleep_end = scheme_get_val_as_double(argv[0]);
