@@ -172,108 +172,110 @@ enum {
   scheme_raw_pair_type,                 /* 152 */
   scheme_prompt_type,                   /* 153 */
   scheme_prompt_tag_type,               /* 154 */
-  scheme_expanded_syntax_type,          /* 155 */
-  scheme_delay_syntax_type,             /* 156 */
-  scheme_cust_box_type,                 /* 157 */
-  scheme_resolved_module_path_type,     /* 158 */
-  scheme_module_phase_exports_type,     /* 159 */
-  scheme_logger_type,                   /* 160 */
-  scheme_log_reader_type,               /* 161 */
-  scheme_free_id_info_type,             /* 162 */
-  scheme_rib_delimiter_type,            /* 163 */
-  scheme_noninline_proc_type,           /* 164 */
-  scheme_prune_context_type,            /* 165 */
-  scheme_future_type,                   /* 166 */
-  scheme_flvector_type,                 /* 167 */
-  scheme_fxvector_type,                 /* 168 */
-  scheme_place_type,                    /* 169 */
-  scheme_place_object_type,             /* 170 */
-  scheme_place_async_channel_type,      /* 171 */
-  scheme_place_bi_channel_type,         /* 172 */
-  scheme_once_used_type,                /* 173 */
-  scheme_serialized_symbol_type,        /* 174 */
-  scheme_serialized_structure_type,     /* 175 */
-  scheme_fsemaphore_type,               /* 176 */
-  scheme_serialized_tcp_fd_type,        /* 177 */
-  scheme_serialized_file_fd_type,       /* 178 */
-  scheme_port_closed_evt_type,          /* 179 */
+  scheme_continuation_mark_key_type,    /* 155 */
+  scheme_expanded_syntax_type,          /* 156 */
+  scheme_delay_syntax_type,             /* 157 */
+  scheme_cust_box_type,                 /* 158 */
+  scheme_resolved_module_path_type,     /* 159 */
+  scheme_module_phase_exports_type,     /* 160 */
+  scheme_logger_type,                   /* 161 */
+  scheme_log_reader_type,               /* 162 */
+  scheme_free_id_info_type,             /* 163 */
+  scheme_rib_delimiter_type,            /* 164 */
+  scheme_noninline_proc_type,           /* 165 */
+  scheme_prune_context_type,            /* 166 */
+  scheme_future_type,                   /* 167 */
+  scheme_flvector_type,                 /* 168 */
+  scheme_fxvector_type,                 /* 169 */
+  scheme_place_type,                    /* 170 */
+  scheme_place_object_type,             /* 171 */
+  scheme_place_async_channel_type,      /* 172 */
+  scheme_place_bi_channel_type,         /* 173 */
+  scheme_once_used_type,                /* 174 */
+  scheme_serialized_symbol_type,        /* 175 */
+  scheme_serialized_structure_type,     /* 176 */
+  scheme_fsemaphore_type,               /* 177 */
+  scheme_serialized_tcp_fd_type,        /* 178 */
+  scheme_serialized_file_fd_type,       /* 179 */
+  scheme_port_closed_evt_type,          /* 180 */
 
 #ifdef MZTAG_REQUIRED
-  _scheme_last_normal_type_,            /* 180 */
+  _scheme_last_normal_type_,            /* 181 */
 
-  scheme_rt_weak_array,                 /* 181 */
+  scheme_rt_weak_array,                 /* 182 */
 
-  scheme_rt_comp_env,                   /* 182 */
-  scheme_rt_constant_binding,           /* 183 */
-  scheme_rt_resolve_info,               /* 184 */
-  scheme_rt_unresolve_info,             /* 185 */
-  scheme_rt_optimize_info,              /* 186 */
-  scheme_rt_compile_info,               /* 187 */
-  scheme_rt_cont_mark,                  /* 188 */
-  scheme_rt_saved_stack,                /* 189 */
-  scheme_rt_reply_item,                 /* 190 */
-  scheme_rt_closure_info,               /* 191 */
-  scheme_rt_overflow,                   /* 192 */
-  scheme_rt_overflow_jmp,               /* 193 */
-  scheme_rt_meta_cont,                  /* 194 */
-  scheme_rt_dyn_wind_cell,              /* 195 */
-  scheme_rt_dyn_wind_info,              /* 196 */
-  scheme_rt_dyn_wind,                   /* 197 */
-  scheme_rt_dup_check,                  /* 198 */
-  scheme_rt_thread_memory,              /* 199 */
-  scheme_rt_input_file,                 /* 200 */
-  scheme_rt_input_fd,                   /* 201 */
-  scheme_rt_oskit_console_input,        /* 202 */
-  scheme_rt_tested_input_file,          /* 203 */
-  scheme_rt_tested_output_file,         /* 204 */
-  scheme_rt_indexed_string,             /* 205 */
-  scheme_rt_output_file,                /* 206 */
-  scheme_rt_load_handler_data,          /* 207 */
-  scheme_rt_pipe,                       /* 208 */
-  scheme_rt_beos_process,               /* 209 */
-  scheme_rt_system_child,               /* 210 */
-  scheme_rt_tcp,                        /* 211 */
-  scheme_rt_write_data,                 /* 212 */
-  scheme_rt_tcp_select_info,            /* 213 */
-  scheme_rt_param_data,                 /* 214 */
-  scheme_rt_will,                       /* 215 */
-  scheme_rt_struct_proc_info,           /* 216 */
-  scheme_rt_linker_name,                /* 217 */
-  scheme_rt_param_map,                  /* 218 */
-  scheme_rt_finalization,               /* 219 */
-  scheme_rt_finalizations,              /* 220 */
-  scheme_rt_cpp_object,                 /* 221 */
-  scheme_rt_cpp_array_object,           /* 222 */
-  scheme_rt_stack_object,               /* 223 */
-  scheme_rt_preallocated_object,        /* 224 */
-  scheme_thread_hop_type,               /* 225 */
-  scheme_rt_srcloc,                     /* 226 */
-  scheme_rt_evt,                        /* 227 */
-  scheme_rt_syncing,                    /* 228 */
-  scheme_rt_comp_prefix,                /* 229 */
-  scheme_rt_user_input,                 /* 230 */
-  scheme_rt_user_output,                /* 231 */
-  scheme_rt_compact_port,               /* 232 */
-  scheme_rt_read_special_dw,            /* 233 */
-  scheme_rt_regwork,                    /* 234 */
-  scheme_rt_rx_lazy_string,             /* 235 */
-  scheme_rt_buf_holder,                 /* 236 */
-  scheme_rt_parameterization,           /* 237 */
-  scheme_rt_print_params,               /* 238 */
-  scheme_rt_read_params,                /* 239 */
-  scheme_rt_native_code,                /* 240 */
-  scheme_rt_native_code_plus_case,      /* 241 */
-  scheme_rt_jitter_data,                /* 242 */
-  scheme_rt_module_exports,             /* 243 */
-  scheme_rt_delay_load_info,            /* 244 */
-  scheme_rt_marshal_info,               /* 245 */
-  scheme_rt_unmarshal_info,             /* 246 */
-  scheme_rt_runstack,                   /* 247 */
-  scheme_rt_sfs_info,                   /* 248 */
-  scheme_rt_validate_clearing,          /* 249 */
-  scheme_rt_rb_node,                    /* 250 */
-  scheme_rt_lightweight_cont,           /* 251 */
-  scheme_rt_export_info,                /* 252 */
+  scheme_rt_comp_env,                   /* 183 */
+  scheme_rt_constant_binding,           /* 184 */
+  scheme_rt_resolve_info,               /* 185 */
+  scheme_rt_unresolve_info,             /* 186 */
+  scheme_rt_optimize_info,              /* 187 */
+  scheme_rt_compile_info,               /* 188 */
+  scheme_rt_cont_mark,                  /* 189 */
+  scheme_rt_saved_stack,                /* 190 */
+  scheme_rt_reply_item,                 /* 191 */
+  scheme_rt_closure_info,               /* 192 */
+  scheme_rt_overflow,                   /* 193 */
+  scheme_rt_overflow_jmp,               /* 194 */
+  scheme_rt_meta_cont,                  /* 195 */
+  scheme_rt_dyn_wind_cell,              /* 196 */
+  scheme_rt_dyn_wind_info,              /* 197 */
+  scheme_rt_dyn_wind,                   /* 198 */
+  scheme_rt_dup_check,                  /* 199 */
+  scheme_rt_thread_memory,              /* 200 */
+  scheme_rt_input_file,                 /* 201 */
+  scheme_rt_input_fd,                   /* 202 */
+  scheme_rt_oskit_console_input,        /* 203 */
+  scheme_rt_tested_input_file,          /* 204 */
+  scheme_rt_tested_output_file,         /* 205 */
+  scheme_rt_indexed_string,             /* 206 */
+  scheme_rt_output_file,                /* 207 */
+  scheme_rt_load_handler_data,          /* 208 */
+  scheme_rt_pipe,                       /* 209 */
+  scheme_rt_beos_process,               /* 210 */
+  scheme_rt_system_child,               /* 211 */
+  scheme_rt_tcp,                        /* 212 */
+  scheme_rt_write_data,                 /* 213 */
+  scheme_rt_tcp_select_info,            /* 214 */
+  scheme_rt_param_data,                 /* 215 */
+  scheme_rt_will,                       /* 216 */
+  scheme_rt_struct_proc_info,           /* 217 */
+  scheme_rt_linker_name,                /* 218 */
+  scheme_rt_param_map,                  /* 219 */
+  scheme_rt_finalization,               /* 220 */
+  scheme_rt_finalizations,              /* 221 */
+  scheme_rt_cpp_object,                 /* 222 */
+  scheme_rt_cpp_array_object,           /* 223 */
+  scheme_rt_stack_object,               /* 224 */
+  scheme_rt_preallocated_object,        /* 225 */
+  scheme_thread_hop_type,               /* 226 */
+  scheme_rt_srcloc,                     /* 227 */
+  scheme_rt_evt,                        /* 228 */
+  scheme_rt_syncing,                    /* 229 */
+  scheme_rt_comp_prefix,                /* 230 */
+  scheme_rt_user_input,                 /* 231 */
+  scheme_rt_user_output,                /* 232 */
+  scheme_rt_compact_port,               /* 233 */
+  scheme_rt_read_special_dw,            /* 234 */
+  scheme_rt_regwork,                    /* 235 */
+  scheme_rt_rx_lazy_string,             /* 236 */
+  scheme_rt_buf_holder,                 /* 237 */
+  scheme_rt_parameterization,           /* 238 */
+  scheme_rt_print_params,               /* 239 */
+  scheme_rt_read_params,                /* 240 */
+  scheme_rt_native_code,                /* 241 */
+  scheme_rt_native_code_plus_case,      /* 242 */
+  scheme_rt_jitter_data,                /* 243 */
+  scheme_rt_module_exports,             /* 244 */
+  scheme_rt_delay_load_info,            /* 245 */
+  scheme_rt_marshal_info,               /* 246 */
+  scheme_rt_unmarshal_info,             /* 247 */
+  scheme_rt_runstack,                   /* 248 */
+  scheme_rt_sfs_info,                   /* 249 */
+  scheme_rt_validate_clearing,          /* 250 */
+  scheme_rt_avl_node,                   /* 251 */
+  scheme_rt_lightweight_cont,           /* 252 */
+  scheme_rt_export_info,                /* 253 */
+  scheme_rt_cont_jmp,                   /* 254 */
 #endif
 
   _scheme_last_type_
