@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2012 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -192,8 +192,7 @@ static Scheme_Object *rehash_symbol_bucket(Scheme_Hash_Table *table,
   {
     Scheme_Object **ba;
 #ifdef MZ_PRECISE_GC
-    ba = (Scheme_Object **)GC_malloc_weak_array(sizeof(Scheme_Object *) * newsize,
-                                                SYMTAB_LOST_CELL);
+    ba = (Scheme_Object **)GC_malloc_weak_array(asize, SYMTAB_LOST_CELL);
 #else
     ba = MALLOC_N_ATOMIC(Scheme_Object *, newsize);
     memset((char *)ba, 0, asize);
@@ -325,7 +324,8 @@ scheme_init_symbol (Scheme_Env *env)
   Scheme_Object *p;
 
   p = scheme_make_folding_prim(symbol_p_prim, "symbol?", 1, 1, 1);
-  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
+  SCHEME_PRIM_PROC_FLAGS(p) |= (SCHEME_PRIM_IS_UNARY_INLINED
+                                | SCHEME_PRIM_IS_OMITABLE);
   scheme_add_global_constant("symbol?", p, env);
 
   p = scheme_make_folding_prim(symbol_unreadable_p_prim, "symbol-unreadable?", 1, 1, 1);
@@ -589,11 +589,12 @@ const char *scheme_symbol_name_and_size(Scheme_Object *sym, uintptr_t *length, i
     if (flags & SCHEME_SNF_KEYWORD) {
       digit_start = 0;
     } else {
-      digit_start = (isdigit((unsigned char)s[0]) || (s[0] == '.')
-		     || (s[0] == '+') || (s[0] == '-'));
-      if (s[0] == '#' && (len == 1 || s[1] != '%'))
+      int ch = ((unsigned char *)s)[0];
+      digit_start = (((ch < 128) && isdigit(ch)) || (ch == '.')
+		     || (ch == '+') || (ch == '-'));
+      if (ch == '#' && (len == 1 || s[1] != '%'))
 	has_special = 1;
-      if (s[0] == '.' && len == 1)
+      if (ch == '.' && len == 1)
 	has_special = 1;
     }
   } else {
@@ -603,39 +604,41 @@ const char *scheme_symbol_name_and_size(Scheme_Object *sym, uintptr_t *length, i
   }
 
   for (i = 0; i < len; i++) {
-    if (isspace((unsigned char)s[i])) { /* used to have || !isprint((unsigned char)s[i]) */
-      if ((flags & SCHEME_SNF_FOR_TS) && (s[i] == ' ')) {
+    int ch = ((unsigned char *)s)[i];
+
+    if (ch > 127) {
+      /* Decode UTF-8. */
+      mzchar buf[2];
+      int ul = 2;
+      while (1) {
+        if (scheme_utf8_decode((unsigned char *)s, i, i + ul,
+                               buf, 0, 1,
+                               NULL, 0, 0) > 0)
+          break;
+        ul++;
+      }
+      ch = buf[0];
+      if ((flags & SCHEME_SNF_NEED_CASE) && scheme_isspecialcasing(ch)) {
+        mzchar *rc;
+        buf[1] = 0;
+        rc = scheme_string_recase(buf, 0, 1, 3, 1, NULL);
+        if ((rc != buf) || (rc[0] != ch))
+          has_upper = 1;
+        ch = 'a';
+      }
+      i += (ul - 1);
+    }
+
+    if (scheme_isspace(ch)) { /* used to have || !isprint(ch) */
+      if ((flags & SCHEME_SNF_FOR_TS) && (ch == ' ')) {
 	/* space is OK in type symbols */
       } else
 	has_space = 1;
-    } else if (isSpecial(s[i]))
+    } else if (isSpecial(ch))
       has_special = 1;
-    else if (s[i] == '|')
+    else if (ch == '|')
       has_pipe = 1;
     else if (flags & SCHEME_SNF_NEED_CASE) {
-      int ch = ((unsigned char *)s)[i];
-      if (ch > 127) {
-	/* Decode UTF-8. */
-	mzchar buf[2];
-	int ul = 2;
-	while (1) {
-	  if (scheme_utf8_decode((unsigned char *)s, i, i + ul,
-				 buf, 0, 1,
-				 NULL, 0, 0) > 0)
-	    break;
-	  ul++;
-	}
-	ch = buf[0];
-	if (scheme_isspecialcasing(ch)) {
-	  mzchar *rc;
-	  buf[1] = 0;
-	  rc = scheme_string_recase(buf, 0, 1, 3, 1, NULL);
-	  if ((rc != buf) || (rc[0] != ch))
-	    has_upper = 1;
-	  ch = 'a';
-	}
-	i += (ul - 1);
-      }
       if (scheme_tofold(ch) != ch)
 	has_upper = 1;
     }
@@ -680,24 +683,30 @@ const char *scheme_symbol_name_and_size(Scheme_Object *sym, uintptr_t *length, i
       result[len + 1] = '|';
       result[len + 2] = 0;
     } else {
+      mzchar cbuf[100], *cs, *cresult;
+      intptr_t clen;
       int p = 0;
       uintptr_t i = 0;
+      intptr_t rlen;
 
-      result = (char *)scheme_malloc_atomic((2 * len) + 1);
+      dz = 0;
+      cs = scheme_utf8_decode_to_buffer_len((unsigned char *)s, len, cbuf, 100, &clen);
 
-      for (i = 0; i < len; i++) {
-	if (isspace((unsigned char)s[i])
-	    || isSpecial(s[i])
-	    || ((s[i] == '|') && pipe_quote)
+      cresult = (mzchar *)scheme_malloc_atomic(((2 * len) + 1) * sizeof(mzchar));
+
+      for (i = 0; i < clen; i++) {
+        mzchar ch = cs[i];
+        if (scheme_isspace(ch)
+	    || isSpecial(ch)
+	    || ((ch == '|') && pipe_quote)
 	    || (!i && s[0] == '#')
-	    || (has_upper && ((((unsigned char)s[i]) >= 'A')
-			      && (((unsigned char)s[i]) <= 'Z'))))
-	  result[p++] = '\\';
-	result[p++] = s[i];
+	    || (has_upper && (ch >= 'A') && (ch <= 'Z')))
+	  cresult[p++] = '\\';
+	cresult[p++] = ch;
       }
 
-      result[p] = 0;
-      total_length = p;
+      result = scheme_utf8_encode_to_buffer_len(cresult, p, NULL, 0, &rlen);
+      total_length = rlen;
     }
   }
 
@@ -734,7 +743,7 @@ symbol_interned_p_prim (int argc, Scheme_Object *argv[])
   if (SCHEME_SYMBOLP(argv[0]))
     return (SCHEME_SYM_WEIRDP(argv[0]) ? scheme_false : scheme_true);
   
-  scheme_wrong_type("symbol-interned?", "symbol", 0, argc, argv);
+  scheme_wrong_contract("symbol-interned?", "symbol?", 0, argc, argv);
   return NULL;
 }
 
@@ -744,7 +753,7 @@ symbol_unreadable_p_prim (int argc, Scheme_Object *argv[])
   if (SCHEME_SYMBOLP(argv[0]))
     return (SCHEME_SYM_PARALLELP(argv[0]) ? scheme_true : scheme_false);
   
-  scheme_wrong_type("symbol-unreadable?", "symbol", 0, argc, argv);
+  scheme_wrong_contract("symbol-unreadable?", "symbol?", 0, argc, argv);
   return NULL;
 }
 
@@ -752,7 +761,7 @@ static Scheme_Object *
 string_to_symbol_prim (int argc, Scheme_Object *argv[])
 {
   if (!SCHEME_CHAR_STRINGP(argv[0]))
-    scheme_wrong_type("string->symbol", "string", 0, argc, argv);
+    scheme_wrong_contract("string->symbol", "string?", 0, argc, argv);
   return scheme_intern_exact_char_symbol(SCHEME_CHAR_STR_VAL(argv[0]),
 					 SCHEME_CHAR_STRTAG_VAL(argv[0]));
 }
@@ -761,7 +770,7 @@ static Scheme_Object *
 string_to_uninterned_symbol_prim (int argc, Scheme_Object *argv[])
 {
   if (!SCHEME_CHAR_STRINGP(argv[0]))
-    scheme_wrong_type("string->uninterned-symbol", "string", 0, argc, argv);
+    scheme_wrong_contract("string->uninterned-symbol", "string?", 0, argc, argv);
   return scheme_make_exact_char_symbol(SCHEME_CHAR_STR_VAL(argv[0]),
 				       SCHEME_CHAR_STRTAG_VAL(argv[0]));
 }
@@ -773,7 +782,7 @@ string_to_unreadable_symbol_prim (int argc, Scheme_Object *argv[])
   intptr_t blen;
 
   if (!SCHEME_CHAR_STRINGP(argv[0]))
-    scheme_wrong_type("string->unreadable-symbol", "string", 0, argc, argv);
+    scheme_wrong_contract("string->unreadable-symbol", "string?", 0, argc, argv);
 
   bs = scheme_utf8_encode_to_buffer_len(SCHEME_CHAR_STR_VAL(argv[0]),
                                         SCHEME_CHAR_STRTAG_VAL(argv[0]), 
@@ -782,19 +791,13 @@ string_to_unreadable_symbol_prim (int argc, Scheme_Object *argv[])
   return scheme_intern_exact_parallel_symbol(bs, blen);
 }
 
-static Scheme_Object *
-symbol_to_string_prim (int argc, Scheme_Object *argv[])
+Scheme_Object *scheme_symbol_to_string(Scheme_Object *sym)
 {
-  Scheme_Object *sym, *str;
+  Scheme_Object *str;
   GC_CAN_IGNORE unsigned char *s;
   GC_CAN_IGNORE mzchar *s2;
   intptr_t len, i;
 
-  sym = argv[0];
-
-  if (!SCHEME_SYMBOLP(sym))
-    scheme_wrong_type("symbol->string", "symbol", 0, argc, argv);
-
   s = (unsigned char *)SCHEME_SYM_VAL(sym);
   len = SCHEME_SYM_LEN(sym);
   for (i = 0; i < len; i++) {
@@ -819,6 +822,20 @@ symbol_to_string_prim (int argc, Scheme_Object *argv[])
   }
 }
 
+static Scheme_Object *
+symbol_to_string_prim (int argc, Scheme_Object *argv[])
+{
+  Scheme_Object *sym;
+
+  sym = argv[0];
+
+  if (!SCHEME_SYMBOLP(sym))
+    scheme_wrong_contract("symbol->string", "symbol?", 0, argc, argv);
+
+  return scheme_symbol_to_string(sym);
+}
+
+
 static Scheme_Object *
 keyword_p_prim (int argc, Scheme_Object *argv[])
 {
@@ -832,12 +849,12 @@ static Scheme_Object *keyword_lt (int argc, Scheme_Object *argv[])
   int i, al, bl, t;
 
   if (!SCHEME_KEYWORDP(prev))
-    scheme_wrong_type("keyword<?", "keyword", 0, argc, argv);
+    scheme_wrong_contract("keyword<?", "keyword?", 0, argc, argv);
 
   for (i = 1; i < argc; i++) {
     kw = argv[i];
     if (!SCHEME_KEYWORDP(kw))
-      scheme_wrong_type("keyword<?", "keyword", i, argc, argv);
+      scheme_wrong_contract("keyword<?", "keyword?", i, argc, argv);
 
     a = (unsigned char *)SCHEME_SYM_VAL(prev);
     al = SCHEME_SYM_LEN(prev);
@@ -863,7 +880,7 @@ static Scheme_Object *keyword_lt (int argc, Scheme_Object *argv[])
       /* Check remaining types */
       for (i++; i < argc; i++) {
         if (!SCHEME_KEYWORDP(argv[i]))
-          scheme_wrong_type("keyword<?", "keyword", i, argc, argv);
+          scheme_wrong_contract("keyword<?", "keyword?", i, argc, argv);
       }
       return scheme_false;
     }
@@ -878,7 +895,7 @@ static Scheme_Object *
 string_to_keyword_prim (int argc, Scheme_Object *argv[])
 {
   if (!SCHEME_CHAR_STRINGP(argv[0]))
-    scheme_wrong_type("string->keyword", "string", 0, argc, argv);
+    scheme_wrong_contract("string->keyword", "string?", 0, argc, argv);
   return scheme_intern_exact_char_keyword(SCHEME_CHAR_STR_VAL(argv[0]),
 					  SCHEME_CHAR_STRTAG_VAL(argv[0]));
 }
@@ -887,7 +904,7 @@ static Scheme_Object *
 keyword_to_string_prim (int argc, Scheme_Object *argv[])
 {
   if (!SCHEME_KEYWORDP(argv[0]))
-    scheme_wrong_type("keyword->string", "keyword", 0, argc, argv);
+    scheme_wrong_contract("keyword->string", "keyword?", 0, argc, argv);
   
   return scheme_make_sized_offset_utf8_string((char *)(argv[0]),
 					      SCHEME_SYMSTR_OFFSET(argv[0]),
@@ -905,7 +922,7 @@ static Scheme_Object *gensym(int argc, Scheme_Object *argv[])
     r = NULL;
 
   if (r && !SCHEME_SYMBOLP(r) && !SCHEME_CHAR_STRINGP(r))
-    scheme_wrong_type("gensym", "symbol or string", 0, argc, argv);
+    scheme_wrong_contract("gensym", "(or/c symbol? string?)", 0, argc, argv);
 
   if (r) {
     char buf[64];
